import { AbpSessionService } from '@abp/session/abp-session.service';
import { Component, Injector, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { accountModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { SessionServiceProxy, UpdateUserSignInTokenOutput, TenantLoginInfoDto } from '@shared/service-proxies/service-proxies';
import { UrlHelper } from 'shared/helpers/UrlHelper';
import { ExternalLoginProvider, LoginService } from './login.service';
import { RecaptchaComponent } from 'ng-recaptcha';
import { AppConsts } from '@shared/AppConsts';
import { AppSessionService } from '@shared/common/session/app-session.service';
import { DomHelper } from '@shared/helpers/DomHelper';
import { LocalStorageService } from '@shared/utils/local-storage.service';
import * as moment from 'moment';

@Component({
    templateUrl: './login.component.html',
    animations: [accountModuleAnimation()],
    styleUrls: ['./login.component.less']
})
export class LoginComponent extends AppComponentBase implements OnInit {
    @ViewChild('recaptchaRef', { static: false }) recaptchaRef: RecaptchaComponent;

    submitting = false;
    isMultiTenancyEnabled: boolean = this.multiTenancy.isEnabled;
    recaptchaSiteKey: string = AppConsts.recaptchaSiteKey;
    captchaResponse?: string;
    cssTrigger= false;
    todaysDate: string;
    backgroundImg : string;
    currenTenant: TenantLoginInfoDto;

    constructor(
        injector: Injector,
        public loginService: LoginService,
        private _router: Router,
        private _sessionService: AbpSessionService,
        private _sessionAppService: SessionServiceProxy,
        private _localStorageService: LocalStorageService
    ) {
        super(injector);
        this.initializeTenantResources(injector);
        var d = new Date();    
        d = new Date(d.getTime() - d.getTimezoneOffset() * 60000)
         this.todaysDate = d.toISOString().slice(0,10); 
         console.log(d.toISOString().slice(0,))
    }

    get multiTenancySideIsTeanant(): boolean {
        return this._sessionService.tenantId > 0;
    }

    get isTenantSelfRegistrationAllowed(): boolean {
        return this.setting.getBoolean('App.TenantManagement.AllowSelfRegistration');
    }

    get isSelfRegistrationAllowed(): boolean {
        if (!this._sessionService.tenantId) {
            return false;
        }

        return this.setting.getBoolean('App.UserManagement.AllowSelfRegistration');
    }

    ngOnInit(): void {
        var element = document.querySelectorAll('.kt-login__aside ')
        element[0].classList.remove('w-adjust');
        element[0].classList.remove('b-none');
        if (this._sessionService.userId > 0 && UrlHelper.getReturnUrl() && UrlHelper.getSingleSignIn()) {
            this._sessionAppService.updateUserSignInToken()
                .subscribe((result: UpdateUserSignInTokenOutput) => {
                    const initialReturnUrl = UrlHelper.getReturnUrl();
                    const returnUrl = initialReturnUrl + (initialReturnUrl.indexOf('?') >= 0 ? '&' : '?') +
                        'accessToken=' + result.signInToken +
                        '&userId=' + result.encodedUserId +
                        '&tenantId=' + result.encodedTenantId;

                    location.href = returnUrl;
                });
        }

        let state = UrlHelper.getQueryParametersUsingHash().state;
        if (state && state.indexOf('openIdConnect') >= 0) {
            this.loginService.openIdConnectLoginCallback({});
        }
           
    }

    login(): void {
        if (this.useCaptcha && !this.captchaResponse) {
            this.message.warn(this.l('CaptchaCanNotBeEmpty'));
            return;
        }

        this.spinnerService.show();
        this.submitting = true;
        this.loginService.authenticate(
            () => {
                this.submitting = false;
                this.spinnerService.hide();
                if (this.recaptchaRef) {
                    this.recaptchaRef.reset();
                }
                localStorage.removeItem('loadDate');
            },
            null,
            this.captchaResponse
        );
    }

    externalLogin(provider: ExternalLoginProvider) {
        this.loginService.externalAuthenticate(provider);
    }

    get useCaptcha(): boolean {
        return this.setting.getBoolean('App.UserManagement.UseCaptchaOnLogin');
    }

    captchaResolved(captchaResponse: string): void {
        this.captchaResponse = captchaResponse;
    }

    initializeTenantResources(injector: Injector) {
        let appSessionService: AppSessionService = injector.get(AppSessionService);
        // console.log(appSessionService.tenant)
        // console.log(appSessionService.tenant.customCssId)
        if (appSessionService.tenant && appSessionService.tenant.customCssId) {
            this.cssTrigger= true
            document.body.appendChild(
                DomHelper.createElement('link', [
                    {
                        key: 'id',
                        value: 'TenantCustomCss'
                    },
                    {
                        key: 'rel',
                        value: 'stylesheet'
                    },
                    {
                        key: 'href',
                        value: AppConsts.remoteServiceBaseUrl + '/TenantCustomization/GetCustomCss?tenantId=' + appSessionService.tenant.id
                    }])
            );
        }
        else{
            this.cssTrigger=false
        }
        
        if(appSessionService.tenant!=null){
            if(appSessionService.tenant.tenancyName=="fbx"){
                this.backgroundChanger();
            }
        }
        
    }

    backgroundChanger() :void {
        let totalCount = 8;

        let num = Math.ceil(Math.random() * totalCount);

        document.body.style.background = "url(/assets/images/backgrounds/" + num + ".jpg)";
        document.body.style.backgroundRepeat = "repeat";

        // this.backgroundImg = "rgba(0, 0, 0, 10) url(/assets/images/backgrounds/" + num + ".jpg)";

        // document.body.style.backgroundRepeat = "repeat";
        // $('.bg-login-primary').css('background-color', 'red');
        // $('.bg-login-primary').css('backgroundRepeat', 'repeat');



        // console.log('background no.: ' + num);
    }
}
