import { Component, ViewChild, Injector, Output, EventEmitter } from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { DriverRatingServiceProxy, CreateDriverRatingInput} from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
import { FeatureCheckerService } from '@abp/features/feature-checker.service';


@Component({
    selector: 'rateDriverModal',
    templateUrl: './rate-driver-modal.component.html',
     styleUrls: ['./rate-driver-modal.less']
})
export class RateDriverModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;
    tracking: any;
    currentRate = 0;
    driverRating: CreateDriverRatingInput = new CreateDriverRatingInput();
    getDriverRating: any;
    max=5;
    enableComment:boolean = false;
    edit:boolean = false;

    constructor(
        injector: Injector,
        private _driverRatingServiceProxy: DriverRatingServiceProxy,
        private _featureCheckerService: FeatureCheckerService,
        
    ) {
        super(injector);
    }

    show(tracking?: any): void {
        this.tracking = tracking;
        
        this.getDriverRating = tracking;
        this.max = parseInt(this._featureCheckerService.getValue('App.DriverRating.Star'));
        //alert(this.max);
        this.enableComment = this._featureCheckerService.isEnabled('App.DriverRating.Comment');
        
        this._driverRatingServiceProxy.getDriverRating(this.getDriverRating).subscribe(result => {
               if(result.orderId != undefined) {
                this.edit = true;
                this.driverRating.details = result.details;
                this.currentRate = result.starRating;
               }
                
        this.active = true;
        this.modal.show();
        });



       
    }

    save(): void {
        this.saving = true;
        //this.max = 

        this.driverRating.trackId = this.tracking;
        this.driverRating.starRating = this.currentRate;
        this.driverRating.subject = "";


        this._driverRatingServiceProxy.createDriverRating(this.driverRating)
            .pipe(finalize(() => { this.saving = false; }))
                    .subscribe(() => {
                        this.notify.info(this.l('SavedSuccessfully'));
                        this.close();
                        this.modalSave.emit(null);
            });
                
       
       
    }







    close(): void {
        this.driverRating.details = "";
        this.driverRating.subject = "";
        this.currentRate = 0;
        this.active = false;
        this.modal.hide();
    }
}
