import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewChildren, Output, EventEmitter } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrderServiceProxy, OrderListDto, ControllerOrderPackagesListServiceProxy, OrderStatusServiceProxy, OrderStatusListDto, OrderStatusTypeServiceProxy, OrderIncidentsServiceProxy } from '@shared/service-proxies/service-proxies';
// ControllerOrderPackagesListDto, 
import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize, count } from 'rxjs/operators';
import { formatDate } from "@angular/common";
import { ControllerOrderIncidentsComponent } from '@app/sprintship/controller/order-incident/order-incident-list.component';

@Component({
    selector: 'trackingPageViewOrderIncident',
    templateUrl: './tracking-page-view-order-incident-list.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    providers: [ControllerOrderIncidentsComponent]
})

export class TrackingPageViewOrderIncidentComponent extends AppComponentBase  implements OnInit{
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @Input() filters: {
        locationFilter : number;
        startDate: Date;
        endDate: Date
    } = < any > {};
    filterText: string;
    @Output() viewOrderStatus = new EventEmitter<boolean>();


    @Input() orderId:number;
    
    @Input() scrollHeight: string

    // @Output() statusEvent = new EventEmitter<boolean>();

    orderPackageId : number;
    // orderId:number
    orderPackageCustomer: string
    space=" ";
    latitude:number

    // orderPackage: ControllerOrderPackagesListDto = new ControllerOrderPackagesListDto();
    orderStatuses: Array<{}> = new Array();

    orderPackageDateAndTime: any
    orderStatusTypes: any
    formattedOrderPackageDateAndTime: any
    formattedOrderPackageStatusDateAndTime: any
    formattedOrderPackagePickupDetail: any
    formattedOrderPackageDeliveryDetail: any
    formattedOrderPackageStatusTime:any
    maxResultCount=1000;
    paginatorAccess: boolean

    event: LazyLoadEvent

    orderStatusTotalCount: any

    statusFunction: Function;
    
  constructor(
        injector: Injector,
        private id:ActivatedRoute,
        private _controllerOrderPackageAppService: ControllerOrderPackagesListServiceProxy,
        private _orderStatusAppService: OrderStatusServiceProxy,
        private _orderStatusTypeAppService : OrderStatusTypeServiceProxy,
        private _orderIncidentAppService: OrderIncidentsServiceProxy
        
    ) {
        super(injector);
    }

    ngOnInit(){
    this.id.paramMap.subscribe(params => {
            this.orderPackageId = Number(params.get('id'));
        });

        // this.statusFunction = this.getOrderStatus.bind(this)
        // this.getOrderPackageDetail();
        // console.log(this.orderId)
        
        // this.getOrderStatus(this.orderId)
        this.totalCount(event)
    }

    totalCount(event){
        let stringOrderId = this.orderId.toString()
        this._orderIncidentAppService.getOrderIncidentTotalCount(
            this.orderId
        ).subscribe(resultCount => {
            if(resultCount>100){
                this.paginatorAccess=true
                if (this.primengTableHelper.shouldResetPaging(event)) {
                    this.paginator.changePage(0);
                    return;
                }
        
                this.spinnerService.show();
                console.log("order id" + this.orderId)
                this._orderIncidentAppService.getOrderIncidentsByOrderId(
                    this.orderId,
                    this.primengTableHelper.getSorting(this.dataTable),
                    this.primengTableHelper.getSkipCount(this.paginator, event),
                    this.primengTableHelper.getMaxResultCount(this.paginator, event))  
                    .subscribe(result => {
                        this.primengTableHelper.totalRecordsCount = result.totalCount;
                        this.primengTableHelper.records = result.items;
                        this.spinnerService.hide();
                    });
            }
            else{
                this.paginatorAccess = false
                this.spinnerService.show();
                console.log("order id" + this.orderId)
                this._orderIncidentAppService.getOrderIncidentsByOrderId(
                    this.orderId,
                    this.primengTableHelper.getSorting(this.dataTable),
                    undefined,
                    100)  
                    .subscribe(result => {
                        this.primengTableHelper.totalRecordsCount = result.totalCount;
                        this.primengTableHelper.records = result.items;
                        this.spinnerService.hide();
                    });

            }
        })
    }

    getOrderIncident(event?: LazyLoadEvent){
        let stringOrderId = this.orderId.toString()
       

       
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        console.log("order id" + this.orderId)
        this._orderIncidentAppService.getAll(
            undefined,
            this.orderId,
            undefined,
            undefined,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event))  
            .subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.totalCount;
                this.primengTableHelper.records = result.items;
                this.spinnerService.hide();
            });
    }

    // getOrderStatus(event?: LazyLoadEvent){
    //     this._controllerOrderPackageAppService.getOrderStatusTotalCount(
    //         this.orderId
    //     ).subscribe(totalCount =>{
    //         this.orderStatusTotalCount = totalCount
    //         if(this.orderStatusTotalCount > 100){
    //             this.paginatorAccess = true
    //             console.log(this.paginatorAccess)
    //             if (this.primengTableHelper.shouldResetPaging(event)) {
    //                 this.paginator.changePage(0);
    //                 return;
    //             }
                
    //             this.spinnerService.show();
    //             this._orderStatusAppService.getOrderStatus(
    //                 undefined,
    //                 undefined,
    //                 this.orderId,
    //                 undefined,
    //                 undefined,
    //                 undefined,
    //                 this.primengTableHelper.getSorting(this.dataTable),
    //                 this.primengTableHelper.getMaxResultCount(this.paginator, event),
    //                 this.primengTableHelper.getSkipCount(this.paginator, event)
    //             ).pipe().subscribe(statusResult => {
    //                 this.orderStatuses=statusResult.items;
    //                 // this.formattedOrderPackageStatusTime=moment(statusResult.items['time']).utc().format('MM/DD/YYYY, h:mm a')
    //                 this.primengTableHelper.totalRecordsCount = statusResult.totalCount;
    //                 this.spinnerService.hide()
    //             });
    //         }
    //         else{
    //             this.paginatorAccess = false
    //             console.log(this.paginatorAccess)
    //             this.spinnerService.show();
    //             this._orderStatusAppService.getOrderStatus(
    //                 undefined,
    //                 undefined,
    //                 this.orderId,
    //                 undefined,
    //                 undefined,
    //                 undefined,
    //                 this.primengTableHelper.getSorting(this.dataTable),
    //                 this.maxResultCount,
    //                 undefined
    //             ).pipe().subscribe(statusResult => {
    //                 this.orderStatuses=statusResult.items;
    //                 // this.formattedOrderPackageStatusTime=moment(statusResult.items['time']).utc().format('MM/DD/YYYY, h:mm a')
    //                 // this.primengTableHelper.totalRecordsCount = statusResult.totalCount;
    //                 this.spinnerService.hide()
    //             });
    //         }
    //     });

        

    //     console.log("totalCount"+this.orderStatusTotalCount)
        
        
    // }

    // getOrderStatusType(): void{
    //     this._orderStatusTypeAppService.getOrderStatusType(
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined
    //     ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
    //         this.orderStatusTypes=result.items;   
    //     });
    // }
    
}