﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ImportLogsServiceProxy, CreateOrEditImportLogDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditImportLogModal',
    templateUrl: './create-or-edit-importLog-modal.component.html'
})
export class CreateOrEditImportLogModalComponent extends AppComponentBase {
   
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    importLog: CreateOrEditImportLogDto = new CreateOrEditImportLogDto();



    constructor(
        injector: Injector,
        private _importLogsServiceProxy: ImportLogsServiceProxy
    ) {
        super(injector);
    }
    
    show(importLogId?: number): void {
    

        if (!importLogId) {
            this.importLog = new CreateOrEditImportLogDto();
            this.importLog.id = importLogId;

            this.active = true;
            this.modal.show();
        } else {
            this._importLogsServiceProxy.getImportLogForEdit(importLogId).subscribe(result => {
                this.importLog = result.importLog;


                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

			
			
            this._importLogsServiceProxy.createOrEdit(this.importLog)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
