﻿import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import {Location} from '@angular/common'; 
import { Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { EntityDtoOfInt64, DriversServiceProxy, UserListDto, GetDriverDetailsDto, VendorsServiceProxy,OrderStatusAttachmentServiceProxy, ImportLogDto, ImportLogsServiceProxy, ImportType, TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { Observable } from 'rxjs';
import { HttpEventType, HttpClient } from '@angular/common/http';
import { map } from "rxjs/operators";
import { saveAs } from 'file-saver';
import { NotifyService } from '@abp/notify/notify.service';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { CreateOrEditImportLogModalComponent } from './create-or-edit-importLog-modal.component';
import { ViewImportLogModalComponent } from './view-importLog-modal.component';


@Component({
    templateUrl: './importLogs.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class ImportLogsComponent extends AppComponentBase {

    @ViewChild('createOrEditImportLogModal', { static: true }) createOrEditImportLogModal: CreateOrEditImportLogModalComponent;
    @ViewChild('viewImportLogModalComponent', { static: true }) viewImportLogModal: ViewImportLogModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    importTypeFilter = -1;
    fileNameFilter = '';
    maxSuccessFilter : number;
		maxSuccessFilterEmpty : number;
		minSuccessFilter : number;
		minSuccessFilterEmpty : number;
    maxFailedFilter : number;
		maxFailedFilterEmpty : number;
		minFailedFilter : number;
		minFailedFilterEmpty : number;
    messageFilter = '';
    maxUserIdFilter : number;
		maxUserIdFilterEmpty : number;
		minUserIdFilter : number;
		minUserIdFilterEmpty : number;

    importType = ImportType;
    constructor(
        injector: Injector,
        private _importLogsServiceProxy: ImportLogsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _httpClient: HttpClient
    ) {
        super(injector);
    }

    getImportLogs(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._importLogsServiceProxy.getAll(
            this.filterText,
            this.importTypeFilter,
            this.fileNameFilter,
            this.maxSuccessFilter == null ? this.maxSuccessFilterEmpty: this.maxSuccessFilter,
            this.minSuccessFilter == null ? this.minSuccessFilterEmpty: this.minSuccessFilter,
            this.maxFailedFilter == null ? this.maxFailedFilterEmpty: this.maxFailedFilter,
            this.minFailedFilter == null ? this.minFailedFilterEmpty: this.minFailedFilter,
            this.messageFilter,
            this.maxUserIdFilter == null ? this.maxUserIdFilterEmpty: this.maxUserIdFilter,
            this.minUserIdFilter == null ? this.minUserIdFilterEmpty: this.minUserIdFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createImportLog(): void {
        this.createOrEditImportLogModal.show();
    }

    deleteImportLog(importLog: ImportLogDto): void {
        this.message.confirm(
            '',
            this.l(''),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._importLogsServiceProxy.delete(importLog.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    downloadFile(id : number, filename: string): void{
        this.spinnerService.show();
        this.notify.info(this.l('Downloading File.'));
        this._importLogsServiceProxy
            .getFileToExcel(id, filename)
            .subscribe(result => {
                if (result.fileName!=undefined) {
                    this.spinnerService.hide();
                    this.notify.success(this.l('Downloaded Successfully'));
                    window.open(AppConsts.remoteServiceBaseUrl + '/File/DownloadTempFile?fileType=' + result.fileType + '&fileToken=' + result.fileToken + '&fileName=' + result.fileName, '_blank');
                }else{
                    this.spinnerService.hide();
                    this.notify.error(this.l('No data found.'));
                }
            });
    }

    downloadImportFile(id:number){
        this._importLogsServiceProxy.downloadFile(id).subscribe( response => {

            let blob = this.base64ToBlob(response.content);
            saveAs(blob, response.fileName+'.xlsx');

        })
    }

    base64ToBlob(b64Data, sliceSize=512) {
        let byteCharacters = atob(b64Data); //data.file there
        let byteArrays = [];
        for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            let slice = byteCharacters.slice(offset, offset + sliceSize);
        
            let byteNumbers = new Array(slice.length);
            for (var i = 0; i < slice.length; i++) {
                byteNumbers[i] = slice.charCodeAt(i);
            }
            let byteArray = new Uint8Array(byteNumbers);
            byteArrays.push(byteArray);
        }
        return new Blob(byteArrays, {type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'});
    }
}
