﻿import { Component, ViewChild, Injector, Output, EventEmitter } from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { ImportLogsServiceProxy, GetImportLogForViewDto, GetImportLogMessageForViewDto, ImportLogDto , ImportLogMessagesServiceProxy, ImportType} from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { AppConsts } from '@shared/AppConsts';
import { saveAs } from 'file-saver';

@Component({
    selector: 'viewImportLogModal',
    templateUrl: './view-importLog-modal.component.html'
})
export class ViewImportLogModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    item: GetImportLogForViewDto;
    importType = ImportType;
    messages : GetImportLogMessageForViewDto[];


    constructor(
        injector: Injector,
        private _importLogMessageServiceProxy : ImportLogMessagesServiceProxy,
        private _importLogsServiceProxy: ImportLogsServiceProxy

    ) {
        super(injector);
        this.item = new GetImportLogForViewDto();
        this.item.importLog = new ImportLogDto();
    }

    show(item: GetImportLogForViewDto): void {
        this.item = item;
        this.active = true;
        this.dataTable
        
        // this._importLogMessageServiceProxy.getAllMessageByImportLogId(item.importLog.id)
        // .subscribe(result => {
        //     console.log(result)
        //     this.messages = result;
        // })
        // this._importLogMessageServiceProxy.getAll(
        //     item.importLog.id,
        //     undefined,
        //     this.primengTableHelper.getSorting(this.dataTable),
        //     this.primengTableHelper.getSkipCount(this.paginator, event),
        //     this.primengTableHelper.getMaxResultCount(this.paginator, event)
        // ).subscribe(result => {
        //     this.primengTableHelper.totalRecordsCount = result.totalCount;
        //     this.primengTableHelper.records = result.items;
        //     this.spinnerService.hide();
        // });

        this.modal.show();
    }
    getMessages(event?: LazyLoadEvent){
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        this._importLogMessageServiceProxy.getAllMessageByImportLogId(
            undefined,
            this.item.importLog.id,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });    
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }

    downloadFile(id : number, filename: string): void{
        this.spinnerService.show();
        this.notify.info(this.l('Downloading File.'));
        this._importLogsServiceProxy
            .getFileToExcel(id, filename)
            .subscribe(result => {
                if (result.fileName!=undefined) {
                    this.spinnerService.hide();
                    this.notify.success(this.l('Downloaded Successfully'));
                    window.open(AppConsts.remoteServiceBaseUrl + '/File/DownloadTempFile?fileType=' + result.fileType + '&fileToken=' + result.fileToken + '&fileName=' + result.fileName, '_blank');
                }else{
                    this.spinnerService.hide();
                    this.notify.error(this.l('No data found.'));
                }
            });
    }


    downloadImportFile(id:number){
        this._importLogsServiceProxy.downloadFile(id).subscribe( response => {

            let blob = this.base64ToBlob(response.content);
            saveAs(blob, response.fileName+'.xlsx');

        })
    }

    base64ToBlob(b64Data, sliceSize=512) {
        let byteCharacters = atob(b64Data); //data.file there
        let byteArrays = [];
        for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            let slice = byteCharacters.slice(offset, offset + sliceSize);
        
            let byteNumbers = new Array(slice.length);
            for (var i = 0; i < slice.length; i++) {
                byteNumbers[i] = slice.charCodeAt(i);
            }
            let byteArray = new Uint8Array(byteNumbers);
            byteArrays.push(byteArray);
        }
        return new Blob(byteArrays, {type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'});
    }
}
