import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { SecondaryConnectionServiceProxy, UserServiceProxy, UserListDto, SecondaryConnectionDto } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
//import { ViewRouteTypeModalComponent } from './view-routeType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import { CreateOrEditSecondaryConnectionModalComponent } from './create-or-edit-secondary-connection-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
import { Router } from '@angular/router';

@Component({
    templateUrl: './secondary-connection.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class SecondaryConnectionComponent extends AppComponentBase {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('createOrEditSecondaryConnectionModal', { static: true }) createOrEditSecondaryConnectionModal: CreateOrEditSecondaryConnectionModalComponent;
    secondaryConnections: SecondaryConnectionDto[] = [];
    filterText = '';
    
    
    constructor(
        injector: Injector,
        private _secondaryConnectionServiceProxy: SecondaryConnectionServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _router: Router
        
    ) {
        super(injector);
    }

    createSecondaryConnection(): void {
        this.createOrEditSecondaryConnectionModal.show();
    }

    editSecondaryConnection(id, tenantName, connString): void {
        this.createOrEditSecondaryConnectionModal.show(id, tenantName, connString);
    }

    ngOnInit(): void {
        //this.getData();
    }

    getData(event?: LazyLoadEvent) {
        //if (this.primengTableHelper.shouldResetPaging(event)) {
            //this.paginator.changePage(0);
            //return;
        //}



        this.spinnerService.show();
        this._secondaryConnectionServiceProxy.getSecondaryConnection(
            this.filterText,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
           
            //this.primengTableHelper.hideLoadingIndicator();
            this.spinnerService.hide();

        });

        
    }

    reloadPage(): void {
        //this.paginator.changePage(this.paginator.getPage());
         this.getData();
    }


    redirectUrl(url: any, name?: string){
        if(name != null){
            var myurl = `${url}/${name}`;
        }else{
            var myurl = `${url}`;
        }
        console.log(myurl);
        this._router.navigateByUrl(myurl);
    }

    delete(id): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._secondaryConnectionServiceProxy.delete(id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

   
}
