import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { BackgroundProcessesServiceProxy, BackgroundProcessDto} from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditBackgroundProcessModalComponent } from './create-or-edit-backgroundProcess-modal.component';
import { ViewBackgroundProcessModalComponent } from './view-backgroundProcess-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './backgroundProcesses.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class BackgroundProcessesComponent extends AppComponentBase {

    @ViewChild('createOrEditBackgroundProcessModal', { static: true }) createOrEditBackgroundProcessModal: CreateOrEditBackgroundProcessModalComponent;
    @ViewChild('viewBackgroundProcessModalComponent', { static: true }) viewBackgroundProcessModal: ViewBackgroundProcessModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    jobIdFilter = '';
    jobClassFilter = '';
    cronExpressionFilter = '';
    isActiveFilter = -1;
    descriptionFilter = '';
    maxLastStartTimeFilter : moment.Moment;
    minLastStartTimeFilter : moment.Moment;




    constructor(
        injector: Injector,
        private _backgroundProcessesServiceProxy: BackgroundProcessesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,
    ) {
        super(injector);
    }

    getBackgroundProcesses(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._backgroundProcessesServiceProxy.getAll(
            this.filterText,
            this.jobIdFilter,
            this.jobClassFilter,
            this.cronExpressionFilter,
            this.isActiveFilter,
            this.descriptionFilter,
            this.maxLastStartTimeFilter,
            this.minLastStartTimeFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createBackgroundProcess(): void {
        this.createOrEditBackgroundProcessModal.show();
    }

    deleteBackgroundProcess(backgroundProcess: BackgroundProcessDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._backgroundProcessesServiceProxy.delete(backgroundProcess.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._backgroundProcessesServiceProxy.getBackgroundProcessesToExcel(
        this.filterText,
            this.jobIdFilter,
            this.jobClassFilter,
            this.cronExpressionFilter,
            this.isActiveFilter,
            this.descriptionFilter,
            this.maxLastStartTimeFilter,
            this.minLastStartTimeFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
