﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { BackgroundProcessesServiceProxy, CreateOrEditBackgroundProcessDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditBackgroundProcessModal',
    templateUrl: './create-or-edit-backgroundProcess-modal.component.html'
})
export class CreateOrEditBackgroundProcessModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    backgroundProcess: CreateOrEditBackgroundProcessDto = new CreateOrEditBackgroundProcessDto();



    constructor(
        injector: Injector,
        private _backgroundProcessesServiceProxy: BackgroundProcessesServiceProxy
    ) {
        super(injector);
    }

    show(backgroundProcessId?: number): void {

        if (!backgroundProcessId) {
            this.backgroundProcess = new CreateOrEditBackgroundProcessDto();
            this.backgroundProcess.id = backgroundProcessId;
            this.backgroundProcess.lastStartTime = moment().startOf('day');

            this.active = true;
            this.modal.show();
        } else {
            this._backgroundProcessesServiceProxy.getBackgroundProcessForEdit(backgroundProcessId).subscribe(result => {
                this.backgroundProcess = result.backgroundProcess;


                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._backgroundProcessesServiceProxy.createOrEdit(this.backgroundProcess)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
