import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ExportMappingsServiceProxy, CreateOrEditExportMappingDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditExportMappingModal',
    templateUrl: './create-or-edit-exportMapping-modal.component.html'
})
export class CreateOrEditExportMappingModalComponent extends AppComponentBase {
   
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    exportMapping: CreateOrEditExportMappingDto = new CreateOrEditExportMappingDto();
    columnNo = '';



    constructor(
        injector: Injector,
        private _exportMappingsServiceProxy: ExportMappingsServiceProxy
    ) {
        super(injector);
    }
    
    show(exportMappingId?: number): void {
    

        if (!exportMappingId) {
            this.exportMapping = new CreateOrEditExportMappingDto();
            this.exportMapping.id = exportMappingId;

            this.active = true;
            this.modal.show();
        } else {
            this._exportMappingsServiceProxy.getExportMappingForEdit(exportMappingId).subscribe(result => {
                this.exportMapping = result.exportMapping;
                this.columnNo = this.exportMapping.columnNo == 0 ? '' : String(this.exportMapping.columnNo);

                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;
			this.exportMapping.columnNo = this.columnNo == '' || this.columnNo === undefined || this.columnNo == null || Number(this.columnNo) < 1 ? 0 : Number(this.columnNo);
            this._exportMappingsServiceProxy.createOrEdit(this.exportMapping)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
