import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ExportMappingsServiceProxy, ExportMappingDto , ExportType, DataType } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditExportMappingModalComponent } from './create-or-edit-exportMapping-modal.component';
import { ViewExportMappingModalComponent } from './view-exportMapping-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './export-mapping.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class ExportMappingsComponent extends AppComponentBase {

    @ViewChild('createOrEditExportMappingModal', { static: true }) createOrEditExportMappingModal: CreateOrEditExportMappingModalComponent;
    @ViewChild('viewExportMappingModalComponent', { static: true }) viewExportMappingModal: ViewExportMappingModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    nameFilter = '';
    ExportTypeFilter = -1;
    ignoreFilter = -1;
    columnFilter = '';
    dataTypeFilter = -1;
    requiredFilter = -1;
    columnToMapFilter = '';
    columnNoFilter = 0;
    orderSourceId = 0;

    ExportType = ExportType;
    dataType = DataType;

    _entityTypeFullName = 'SprintTek.ExportMappings.ExportMapping';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _ExportMappingsServiceProxy: ExportMappingsServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private router: Router,
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getExportMappings(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._ExportMappingsServiceProxy.getAll(
            this.filterText,
            this.nameFilter,
            this.ExportTypeFilter,
            this.ignoreFilter,
            this.columnFilter,
            this.dataTypeFilter,
            this.requiredFilter,
            this.columnToMapFilter,
            this.columnNoFilter,
            this.orderSourceId,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createExportMapping(): void {
        this.createOrEditExportMappingModal.show();
    }

    showHistory(ExportMapping: ExportMappingDto): void {
        this.entityTypeHistoryModal.show({
            entityId: ExportMapping.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteExportMapping(ExportMapping: ExportMappingDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._ExportMappingsServiceProxy.delete(ExportMapping.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._ExportMappingsServiceProxy.getExportMappingsToExcel(
        this.filterText,
            this.nameFilter,
            this.ExportTypeFilter,
            this.ignoreFilter,
            this.columnFilter,
            this.dataTypeFilter,
            this.requiredFilter,
            this.columnToMapFilter
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    
}
