import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ImportFieldsServiceProxy, CreateOrEditImportFieldDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
import { ImportFieldImportTableLookupTableModalComponent } from './importField-importTable-lookup-table-modal.component';


@Component({
    selector: 'createOrEditImportFieldModal',
    templateUrl: './create-or-edit-importField-modal.component.html'
})
export class CreateOrEditImportFieldModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @ViewChild('importFieldImportTableLookupTableModal', { static: true }) importFieldImportTableLookupTableModal: ImportFieldImportTableLookupTableModalComponent;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    importField: CreateOrEditImportFieldDto = new CreateOrEditImportFieldDto();

    importTableTableName = '';


    constructor(
        injector: Injector,
        private _importFieldsServiceProxy: ImportFieldsServiceProxy
    ) {
        super(injector);
    }

    show(importFieldId?: number): void {

        if (!importFieldId) {
            this.importField = new CreateOrEditImportFieldDto();
            this.importField.id = importFieldId;
            this.importTableTableName = '';

            this.active = true;
            this.modal.show();
        } else {
            this._importFieldsServiceProxy.getImportFieldForEdit(importFieldId).subscribe(result => {
                this.importField = result.importField;

                this.importTableTableName = result.importTableTableName;

                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._importFieldsServiceProxy.createOrEdit(this.importField)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

        openSelectImportTableModal() {
        this.importFieldImportTableLookupTableModal.id = this.importField.importTableId;
        this.importFieldImportTableLookupTableModal.displayName = this.importTableTableName;
        this.importFieldImportTableLookupTableModal.show();
    }


        setImportTableIdNull() {
        this.importField.importTableId = null;
        this.importTableTableName = '';
    }


        getNewImportTableId() {
        this.importField.importTableId = this.importFieldImportTableLookupTableModal.id;
        this.importTableTableName = this.importFieldImportTableLookupTableModal.displayName;
    }


    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
