import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { ImportFieldsServiceProxy, ImportFieldDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditImportFieldModalComponent } from './create-or-edit-importField-modal.component';
import { ViewImportFieldModalComponent } from './view-importField-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './importFields.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class ImportFieldsComponent extends AppComponentBase {

    @ViewChild('createOrEditImportFieldModal', { static: true }) createOrEditImportFieldModal: CreateOrEditImportFieldModalComponent;
    @ViewChild('viewImportFieldModalComponent', { static: true }) viewImportFieldModal: ViewImportFieldModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    fieldNameFilter = '';
    fieldCodeFilter = '';
    requiredFilter = -1;
        importTableTableNameFilter = '';




    constructor(
        injector: Injector,
        private _importFieldsServiceProxy: ImportFieldsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getImportFields(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._importFieldsServiceProxy.getAll(
            this.filterText,
            this.fieldNameFilter,
            this.fieldCodeFilter,
            this.requiredFilter,
            this.importTableTableNameFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createImportField(): void {
        this.createOrEditImportFieldModal.show();
    }

    deleteImportField(importField: ImportFieldDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._importFieldsServiceProxy.delete(importField.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._importFieldsServiceProxy.getImportFieldsToExcel(
        this.filterText,
            this.fieldNameFilter,
            this.fieldCodeFilter,
            this.requiredFilter,
            this.importTableTableNameFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
