﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ImportMappingsServiceProxy, CreateOrEditImportMappingDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditImportMappingModal',
    templateUrl: './create-or-edit-importMapping-modal.component.html'
})
export class CreateOrEditImportMappingModalComponent extends AppComponentBase {
   
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    importMapping: CreateOrEditImportMappingDto = new CreateOrEditImportMappingDto();
    columnNo = '';


    constructor(
        injector: Injector,
        private _importMappingsServiceProxy: ImportMappingsServiceProxy
    ) {
        super(injector);
    }
    
    show(importMappingId?: number): void {
    

        if (!importMappingId) {
            this.importMapping = new CreateOrEditImportMappingDto();
            this.importMapping.id = importMappingId;

            this.active = true;
            this.modal.show();
        } else {
            this._importMappingsServiceProxy.getImportMappingForEdit(importMappingId).subscribe(result => {
                this.importMapping = result.importMapping;
                this.columnNo = this.importMapping.columnNo == 0 ? '' : String(this.importMapping.columnNo);

                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

            this.importMapping.columnNo = this.columnNo == '' || this.columnNo === undefined || this.columnNo == null || Number(this.columnNo) < 1 ? 0 : Number(this.columnNo);
            
            this._importMappingsServiceProxy.createOrEdit(this.importMapping)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }

}
