import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ImportMapsServiceProxy, CreateOrEditImportMapDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
import { ImportMapImportFieldLookupTableModalComponent } from './importMap-importField-lookup-table-modal.component';


@Component({
    selector: 'createOrEditImportMapModal',
    templateUrl: './create-or-edit-importMap-modal.component.html'
})
export class CreateOrEditImportMapModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @ViewChild('importMapImportFieldLookupTableModal', { static: true }) importMapImportFieldLookupTableModal: ImportMapImportFieldLookupTableModalComponent;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    importMap: CreateOrEditImportMapDto = new CreateOrEditImportMapDto();

    importFieldFieldName = '';


    constructor(
        injector: Injector,
        private _importMapsServiceProxy: ImportMapsServiceProxy
    ) {
        super(injector);
    }

    show(importMapId?: number): void {

        if (!importMapId) {
            this.importMap = new CreateOrEditImportMapDto();
            this.importMap.id = importMapId;
            this.importFieldFieldName = '';

            this.active = true;
            this.modal.show();
        } else {
            this._importMapsServiceProxy.getImportMapForEdit(importMapId).subscribe(result => {
                this.importMap = result.importMap;

                this.importFieldFieldName = result.importFieldFieldName;

                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._importMapsServiceProxy.createOrEdit(this.importMap)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

        openSelectImportFieldModal() {
        this.importMapImportFieldLookupTableModal.id = this.importMap.importFieldId;
        this.importMapImportFieldLookupTableModal.displayName = this.importFieldFieldName;
        this.importMapImportFieldLookupTableModal.show();
    }


        setImportFieldIdNull() {
        this.importMap.importFieldId = null;
        this.importFieldFieldName = '';
    }


        getNewImportFieldId() {
        this.importMap.importFieldId = this.importMapImportFieldLookupTableModal.id;
        this.importFieldFieldName = this.importMapImportFieldLookupTableModal.displayName;
    }


    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
