import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ImportTablesServiceProxy, CreateOrEditImportTableDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';


@Component({
    selector: 'createOrEditImportTableModal',
    templateUrl: './create-or-edit-importTable-modal.component.html'
})
export class CreateOrEditImportTableModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    importTable: CreateOrEditImportTableDto = new CreateOrEditImportTableDto();



    constructor(
        injector: Injector,
        private _importTablesServiceProxy: ImportTablesServiceProxy
    ) {
        super(injector);
    }

    show(importTableId?: number): void {

        if (!importTableId) {
            this.importTable = new CreateOrEditImportTableDto();
            this.importTable.id = importTableId;

            this.active = true;
            this.modal.show();
        } else {
            this._importTablesServiceProxy.getImportTableForEdit(importTableId).subscribe(result => {
                this.importTable = result.importTable;


                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._importTablesServiceProxy.createOrEdit(this.importTable)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
