﻿import { Component, Injector, ViewEncapsulation, ViewChild, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { RouteHandOffScheduleDetailsServiceProxy, RouteHandOffScheduleDetailDto, CreateOrEditClinicDto, ClinicsServiceProxy, RouteHandOffDefinitionsServiceProxy, CreateOrEditRouteHandOffDefinitionDto, LockedEtaScheduleInput, GetRouteHandOffScheduleDetailForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRouteHandOffScheduleDetailModalComponent } from './create-or-edit-routeHandOffScheduleDetail-modal.component';
import { ViewRouteHandOffScheduleDetailModalComponent } from './view-routeHandOffScheduleDetail-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { Location } from '@angular/common';
import { finalize } from 'rxjs/operators';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './line-haul-route-schedule.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RouteHandOffScheduleDetailsComponent extends AppComponentBase implements OnInit {

    @ViewChild('createOrEditRouteHandOffScheduleDetailModal', { static: true }) createOrEditRouteHandOffScheduleDetailModal: CreateOrEditRouteHandOffScheduleDetailModalComponent;
    @ViewChild('viewRouteHandOffScheduleDetailModalComponent', { static: true }) viewRouteHandOffScheduleDetailModal: ViewRouteHandOffScheduleDetailModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    routeCodeFilter = '';
    maxEstimatedTimeOfArrivalFilter : moment.Moment;
		minEstimatedTimeOfArrivalFilter : moment.Moment;
    maxCutOffFilter : moment.Moment;
		minCutOffFilter : moment.Moment;
    automaticFilter = -1;
    willCallFilter = -1;
    maxOorRateFilter : number;
		maxOorRateFilterEmpty : number;
		minOorRateFilter : number;
		minOorRateFilterEmpty : number;
    maxEffectivityDateFilter : moment.Moment;
		minEffectivityDateFilter : moment.Moment;
    maxRouteScheduleIdFilter : number;
		maxRouteScheduleIdFilterEmpty : number;
		minRouteScheduleIdFilter : number;
		minRouteScheduleIdFilterEmpty : number;
    maxRouteTypeIdFilter : number;
		maxRouteTypeIdFilterEmpty : number;
		minRouteTypeIdFilter : number;
    minRouteTypeIdFilterEmpty : number;
    goBackHistory: number;
    navigateUrl: any;
    paramId: any;
    routeHandOffDefinition: CreateOrEditRouteHandOffDefinitionDto = new CreateOrEditRouteHandOffDefinitionDto();
    routeCode: any;
    routeStart: any;
    routeEnd: any;
    schedules = [];
    lockedEtaInput: LockedEtaScheduleInput = new LockedEtaScheduleInput();
    saving = false;
    _entityTypeFullName = 'SprintTek.Laboratory.RouteHandOffScheduleDetail';
    entityHistoryEnabled = false;
    


    constructor(
        injector: Injector,
        private _routeHandOffScheduleDetailsServiceProxy: RouteHandOffScheduleDetailsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _location: Location,
        private _router: Router,
        private _routeHandOffDefinitionServiceProxy: RouteHandOffDefinitionsServiceProxy
    ) {
        super(injector);
    }


    ngOnInit() {
      this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
      this.showSchedule();
      this.goBackHistory = window.history.length;
    }

    private setIsEntityHistoryEnabled(): boolean {
      let customSettings = (abp as any).custom;
      return customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
  }


    showSchedule() {
      this.getParamId();
      this.getRouteDetails();
      this.getSchedules();
    }

    getRouteHandOffScheduleDetails(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._routeHandOffScheduleDetailsServiceProxy.getAll(
          this.filterText,
          this.routeCodeFilter,
          this.maxEstimatedTimeOfArrivalFilter,
          this.minEstimatedTimeOfArrivalFilter,
          this.maxCutOffFilter,
          this.minCutOffFilter,
          this.automaticFilter,
          this.willCallFilter,
          this.maxOorRateFilter == null ? this.maxOorRateFilterEmpty: this.maxOorRateFilter,
          this.minOorRateFilter == null ? this.minOorRateFilterEmpty: this.minOorRateFilter,
          this.maxEffectivityDateFilter,
          this.minEffectivityDateFilter,
          this.maxRouteScheduleIdFilter == null ? this.maxRouteScheduleIdFilterEmpty: this.maxRouteScheduleIdFilter,
          this.minRouteScheduleIdFilter == null ? this.minRouteScheduleIdFilterEmpty: this.minRouteScheduleIdFilter,
          this.maxRouteTypeIdFilter == null ? this.maxRouteTypeIdFilterEmpty: this.maxRouteTypeIdFilter,
          this.minRouteTypeIdFilter == null ? this.minRouteTypeIdFilterEmpty: this.minRouteTypeIdFilter,
          undefined,
          undefined,
          this.primengTableHelper.getSorting(this.dataTable),
          this.primengTableHelper.getSkipCount(this.paginator, event),
          this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.showSchedule();
    }

    createRouteHandOffScheduleDetail(schedule: any): void {
        this.createOrEditRouteHandOffScheduleDetailModal.show(schedule, this.paramId);
    }

    deleteRouteHandOffScheduleDetail(id: number): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeHandOffScheduleDetailsServiceProxy.delete(id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    goBack() {
      if (this.goBackHistory > 1) {
          this._location.back();
      }
    }

    getParamId() {
      this._activatedRoute.paramMap.subscribe(params => {
          this.paramId = params.get('id');
      });
    }

    getRouteDetails() {
      this._routeHandOffDefinitionServiceProxy.getRouteHandOffDefinitionForView(this.paramId).subscribe((result) => {
        this.routeCode = result.routeTemplateName;
        this.routeStart = result.routeStartInactive ? 'FIRST STOP FROM THE ROUTE SCHEDULE' : result.routeStartAddress;
        this.routeEnd = result.address;
      });
    }

    getSchedules() {
      this.schedules = [];
      this._routeHandOffScheduleDetailsServiceProxy.getLineHaulRouteSchedule(this.paramId).subscribe(result => {
          if (result.items != null) {
            result.items.forEach(sched => {
              this.schedules.push(sched);
            });
          }
      });
    }

    lockEta(id: number, value: boolean) {
      this.lockedEtaInput.id = id;
      this.lockedEtaInput.etaLocked = value;

      this.message.confirm(
          '',
          '',
          (isConfirmed) => {
              if (isConfirmed) {
                 this._routeHandOffScheduleDetailsServiceProxy.lockedEtaSchedule(this.lockedEtaInput)
                      .pipe(finalize(() => { this.saving = false; }))
                      .subscribe(() => {
                         this.notify.info(this.l('SavedSuccessfully'));
                         this.reloadPage();

                 });
              }
          }
      );
    }

    showHistory(routeHandOffScheduleDetail: any): void {
      this.entityTypeHistoryModal.show({
          entityId: routeHandOffScheduleDetail.id.toString(),
          entityTypeFullName: this._entityTypeFullName,
          entityTypeDescription: "Route Hand Off Schedule Detail - " + routeHandOffScheduleDetail.routeCode
      });
  }

}
