import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { RouteStopsServiceProxy, RouteStopDto, GetRouteStopForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRouteStopModalComponent } from './create-or-edit-routeStop-modal.component';
import { ViewRouteStopModalComponent } from './view-routeStop-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './routeStops.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RouteStopsComponent extends AppComponentBase {

    @ViewChild('createOrEditRouteStopModal', { static: true }) createOrEditRouteStopModal: CreateOrEditRouteStopModalComponent;
    @ViewChild('viewRouteStopModalComponent', { static: true }) viewRouteStopModal: ViewRouteStopModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
        contactAddressFilter = '';
    contactFirstNameFilter = '';

    _entityTypeFullName = 'SprintTek.Laboratory.RouteStop';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _routeStopsServiceProxy: RouteStopsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getRouteStops(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        this._routeStopsServiceProxy.getAll(
            this.filterText,
            this.contactFirstNameFilter,
            this.contactAddressFilter,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            //this.primengTableHelper.hideLoadingIndicator();
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createRouteStop(): void {
        this.createOrEditRouteStopModal.show();
    }

    showHistory(routeStop: GetRouteStopForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: routeStop.routeStop.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Route Stop -" +routeStop.contactAddress
        });
    }

    deleteRouteStop(routeStop: RouteStopDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeStopsServiceProxy.delete(routeStop.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._routeStopsServiceProxy.getRouteStopsToExcel(
            this.filterText,
            this.contactFirstNameFilter,
            this.contactAddressFilter,
            undefined,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
