import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { RouteTypesServiceProxy, CreateOrEditRouteTypeDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
declare var $: any;

@Component({
    selector: 'createOrEditRouteTypeModal',
    templateUrl: './create-or-edit-routeType-modal.component.html'
})
export class CreateOrEditRouteTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;
    selectedStartDay:string;
    selectedEndDay:string;
    routeType: CreateOrEditRouteTypeDto = new CreateOrEditRouteTypeDto();
    startTime: any;



    constructor(
        injector: Injector,
        private _routeTypesServiceProxy: RouteTypesServiceProxy
    ) {
        super(injector);
    }

    show(routeTypeId?: number): void {

        if (!routeTypeId) {
            this.routeType = new CreateOrEditRouteTypeDto();
            this.routeType.id = routeTypeId;
            this.routeType.displayColor='#000000';
            this.active = true;
            this.modal.show();
        } else {
            this._routeTypesServiceProxy.getRouteTypeForEdit(routeTypeId).subscribe(result => {
                this.routeType = result.routeType;
                this.startTime = new Date(moment().format('YYYY-MM-DD') + ' ' + result.routeType.startTime);
                // this.routeType.startTime= moment.utc(moment(moment.now()).format('ddd MMM DD YYYY') + ' ' + this.routeType.startTime);
                // this.routeType.endTime = moment.utc(moment(moment.now()).format('ddd MMM DD YYYY') + ' ' + this.routeType.endTime);
                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;
            //this.routeType.endDay = this.selectedEndDay;
            //this.routeType.startDay = this.selectedStartDay;
            if (this.startTime === undefined) {
                let nd = new Date(0,0,0,0,0,0);
                this.routeType.startTime = nd.toLocaleTimeString('en-US', { hour12: true, 
                hour: "numeric", 
                minute: "numeric",
                second: "numeric"
                }) as any;
            } else {
                this.routeType.startTime = new Date(this.startTime).toLocaleTimeString('en-US', { hour12: true, 
                    hour: "numeric", 
                    minute: "numeric",
                    second: "numeric"
                    }) as any;
            }
            
            this._routeTypesServiceProxy.createOrEdit(this.routeType)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
