import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { RouteTypesServiceProxy, RouteTypeDto, UpdateMultipleRouteType, CreateOrEditRouteTypeDto, Types  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRouteTypeModalComponent } from './create-or-edit-routeType-modal.component';
import { ViewRouteTypeModalComponent } from './view-routeType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './routeTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RouteTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditRouteTypeModal', { static: true }) createOrEditRouteTypeModal: CreateOrEditRouteTypeModalComponent;
    @ViewChild('viewRouteTypeModalComponent', { static: true }) viewRouteTypeModal: ViewRouteTypeModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    typeFilter = -1;
    routeTypeCodeFilter = '';
    descriptionFilter = '';
    displayColorFilter = '';
    update: Array<any> = [];
    types = Types;

    _entityTypeFullName = 'SprintTek.Laboratory.RouteType';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _routeTypesServiceProxy: RouteTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getRouteTypes(event?: LazyLoadEvent) {
        //if (this.primengTableHelper.shouldResetPaging(event)) {
            //this.paginator.changePage(0);
            //return;
        //}

        //this.primengTableHelper.showLoadingIndicator();
        this.spinnerService.show();
        this._routeTypesServiceProxy.getAll(
            this.filterText,
            this.routeTypeCodeFilter,
            this.descriptionFilter,
            this.displayColorFilter,
            this.typeFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            undefined,
            1000
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            console.log(result);
            //this.primengTableHelper.hideLoadingIndicator();
            this.spinnerService.hide();

        });
    }

    reloadPage(): void {
        //this.paginator.changePage(this.paginator.getPage());
         this.getRouteTypes();
    }

    createRouteType(): void {
        this.createOrEditRouteTypeModal.show();
    }

    showHistory(routeType: RouteTypeDto): void {
        console.log(routeType);
        this.entityTypeHistoryModal.show({
            entityId: routeType.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Route Type - " +routeType.description
        });
    }

    deleteRouteType(routeType: RouteTypeDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeTypesServiceProxy.delete(routeType.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._routeTypesServiceProxy.getRouteTypesToExcel(
        this.filterText,
            this.routeTypeCodeFilter,
            this.descriptionFilter,
            this.displayColorFilter,
            this.typeFilter
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

     rowReorder() {
       console.log(this.primengTableHelper.records);
       for (let index = 0; index < this.primengTableHelper.records.length; index++) {
           var element = this.primengTableHelper.records[index];
           //onsole.log(element.routeType.routeTypeCode);
           //this.update = [];
           // console.log(element)
           let a = new CreateOrEditRouteTypeDto({
               routeTypeCode: element.routeType.routeTypeCode,
               description: element.routeType.description,
               displayColor: element.routeType.displayColor,
               sortOrder: index,
               startTime: element.routeType.startTime,
               monday: element.routeType.monday,
               tuesday: element.routeType.tuesday,
               wednesday: element.routeType.wednesday,
               thursday: element.routeType.thursday,
               friday: element.routeType.friday,
               saturday: element.routeType.saturday,
               sunday: element.routeType.sunday,
               type: element.routeType.type,
               id: parseInt(element.routeType.id),
               
           });
           this.update.push(a);
           //console.log(this.update);
           //console.log(a);
       }

       let input = new UpdateMultipleRouteType({
           createOrEditRouteType: this.update,
       });
       //console.log(input);
       this._routeTypesServiceProxy.updateMultipleRouteType(
                  input
              ).pipe().subscribe(result => {
                  this.notify.info(this.l('SavedSuccessfully'));
       });

      
    }
}
