﻿import { Component, Injector, ViewEncapsulation, ViewChild, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { HolidaysServiceProxy, HolidayDto, GetOrderListDtoNew  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditHolidayModalComponent } from './create-or-edit-holiday-modal.component';
import { ViewHolidayModalComponent } from './view-holiday-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

declare var $ : any


@Component({
    templateUrl: './holidays.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class HolidaysComponent extends AppComponentBase implements OnInit {

    @ViewChild('createOrEditHolidayModal', { static: true }) createOrEditHolidayModal: CreateOrEditHolidayModalComponent;
    @ViewChild('viewHolidayModalComponent', { static: true }) viewHolidayModal: ViewHolidayModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;

    _entityTypeFullName = 'SprintTek.Shipping.Locations.Holiday';
    advancedFiltersAreShown = false;
    filterText = '';
    descriptionFilter = '';
    maxIsActiveFilter : number;
        maxIsActiveFilterEmpty : number;
        minIsActiveFilter : number;
        minIsActiveFilterEmpty : number;
    maxFromFilter : moment.Moment;
        minFromFilter : moment.Moment;
    maxToFilter : moment.Moment;
        minToFilter : moment.Moment;
    colorFilter = '';
    codeFilter = '';
    entityHistoryEnabled = false;





    constructor(
        injector: Injector,
        private _holidaysServiceProxy: HolidaysServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }
    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }
    

    getHolidays(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._holidaysServiceProxy.getAll(
            this.filterText,
            this.descriptionFilter,
            this.maxIsActiveFilter == null ? this.maxIsActiveFilterEmpty: this.maxIsActiveFilter,
            this.minIsActiveFilter == null ? this.minIsActiveFilterEmpty: this.minIsActiveFilter,
            this.maxFromFilter,
            this.minFromFilter,
            this.maxToFilter,
            this.minToFilter,
            this.colorFilter,
            this.codeFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createHoliday(): void {
        this.createOrEditHolidayModal.show();
        $('.kt-select2').select2();
    }

    deleteHoliday(holiday: HolidayDto): void {
        this.message.confirm(
            this.l(''),
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._holidaysServiceProxy.delete(holiday.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._holidaysServiceProxy.getHolidaysToExcel(
        this.filterText,
            this.descriptionFilter,
            this.maxIsActiveFilter == null ? this.maxIsActiveFilterEmpty: this.maxIsActiveFilter,
            this.minIsActiveFilter == null ? this.minIsActiveFilterEmpty: this.minIsActiveFilter,
            this.maxFromFilter,
            this.minFromFilter,
            this.maxToFilter,
            this.minToFilter,
            this.colorFilter,
            this.codeFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(holiday: HolidayDto): void {
        this.entityTypeHistoryModal.show({
            entityId: holiday.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Holiday - " +holiday.code
        });
    }


    actionTypes(value : any) {

        if(value == 0) {
            return "Holiday";
        }
        else if(value == 1) {
            return "Unlock Location"
        }
        else if(value == 2) {
            return "Unfreeze Route"
        }
        else {

            return "";
        }
    }
}
