﻿import { Component, Injector, ViewEncapsulation, ViewChild, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { OrderSourceConnectionsServiceProxy, OrderSourceConnectionDto, ConnectionType  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderSourceConnectionModalComponent } from './create-or-edit-orderSourceConnection-modal.component';
import { ViewOrderSourceConnectionModalComponent } from './view-orderSourceConnection-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { Location } from '@angular/common';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './orderSourceConnections.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class OrderSourceConnectionsComponent extends AppComponentBase implements OnInit{

    @ViewChild('createOrEditOrderSourceConnectionModal', { static: true }) createOrEditOrderSourceConnectionModal: CreateOrEditOrderSourceConnectionModalComponent;
    @ViewChild('viewOrderSourceConnectionModalComponent', { static: true }) viewOrderSourceConnectionModal: ViewOrderSourceConnectionModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    id ;
    connectionType  = ConnectionType


    constructor(
        injector: Injector,
        private _orderSourceConnectionsServiceProxy: OrderSourceConnectionsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private route: ActivatedRoute,
        private _location: Location

    ) {
        super(injector);
    }
    ngOnInit(): void {
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id") ;    
        })
    }

    getOrderSourceConnections(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._orderSourceConnectionsServiceProxy.getAllConnections(
            this.filterText,this.id , 
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createOrderSourceConnection(): void {
        this.createOrEditOrderSourceConnectionModal.show(this.id);
    }
        goBack() {
        this._location.back();
    }

    deleteOrderSourceConnection(orderSourceConnection: OrderSourceConnectionDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderSourceConnectionsServiceProxy.delete(orderSourceConnection.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }
}
