﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { OrderSourcesServiceProxy, CreateOrEditOrderSourceDto, OrderTypesServiceProxy, GetOrderTypeForViewDto, LocationServiceProxy, PostalCodeZoneRegexListDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditOrderSourceModal',
    templateUrl: './create-or-edit-orderSource-modal.component.html'
})
export class CreateOrEditOrderSourceModalComponent extends AppComponentBase {
   
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;
    orderTypeList : GetOrderTypeForViewDto[];
    isActive : number = 1
    isHeaderFixed : number = 1;

    orderSource: CreateOrEditOrderSourceDto = new CreateOrEditOrderSourceDto();
    postalCodeChecker:string;
    postalCodeRegExList: PostalCodeZoneRegexListDto [];
    constructor(
        injector: Injector,
        private _orderSourcesServiceProxy: OrderSourcesServiceProxy,
        private _orderTypeServiceProxy: OrderTypesServiceProxy,
        private _locationService: LocationServiceProxy,
    ) {
        super(injector);
    }
    
    onShown(){
        $('.kt-select2').select2();

    }
    show(orderSourceId?: number): void {
        this.isActive = 1;
        this.getPostalCodeRegEx();
        this._orderTypeServiceProxy.getAllOrderTypeForTableDropdown().subscribe(result => {
            this.orderTypeList = result;
        })

        if (!orderSourceId) {
            this.orderSource = new CreateOrEditOrderSourceDto();
            this.orderSource.id = orderSourceId;

            this.active = true;
            this.modal.show();
        } else {
            this._orderSourcesServiceProxy.getOrderSourceForEdit(orderSourceId).subscribe(result => {
                this.orderSource = result.orderSource;
                this.isActive = result.orderSource.isActive;
                this.isHeaderFixed = result.orderSource.isHeaderFixed;
                this.postalCodeChecker = result.orderSource.zipRegEx;

                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;
            this.orderSource.isActive = this.isActive ? 1 : 0;
            this.orderSource.isHeaderFixed =  this.isHeaderFixed ? 1 : 0;
            this.orderSource.orderTypeId = Number((<HTMLInputElement>document.getElementById('OrderSource_OrderType')).value);
            this.orderSource.zipRegEx = (<HTMLInputElement>document.getElementById('OrderSource_ZipRegEx')).value;
            console.log(this.orderSource);
            this._orderSourcesServiceProxy.createOrEdit(this.orderSource)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

    getPostalCodeRegEx():void{
        this._locationService.getPostalCodeRegExList().subscribe(result =>{
            this.postalCodeRegExList = result;
        });
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
