﻿import { Component, Injector, ViewEncapsulation, ViewChild, ElementRef, ViewContainerRef } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { OrderSourcesServiceProxy, OrderSourceDto, OrderTypesServiceProxy, TrackImportInput, OrderServiceProxy, GetOrderSourceForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderSourceModalComponent } from './create-or-edit-orderSource-modal.component';
import { ViewOrderSourceModalComponent } from './view-orderSource-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
import * as signalR from '@aspnet/signalr';
import { HttpClient } from '@angular/common/http';
import { AppConsts } from '@shared/AppConsts';
import { FileUpload } from 'primeng/components/fileupload/fileupload';
import { ProgressBarService } from '@app/shared/layout/progressbar/progressbar.service';
import { interval } from 'rxjs';

@Component({
    templateUrl: './orderSources.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./orderSources.component.less']

})
export class OrderSourcesComponent extends AppComponentBase {

    @ViewChild('createOrEditOrderSourceModal', { static: true }) createOrEditOrderSourceModal: CreateOrEditOrderSourceModalComponent;
    @ViewChild('viewOrderSourceModalComponent', { static: true }) viewOrderSourceModal: ViewOrderSourceModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    advancedFiltersAreShown = false;
    filterText = '';
    nameFilter = '';
    codeFilter : string;
    maxIsActiveFilter : number;
		maxIsActiveFilterEmpty : number;
		minIsActiveFilter : number;
		minIsActiveFilterEmpty : number;
    private hubConnection: signalR.HubConnection;
    maxTenantIdFilter : number;
		maxTenantIdFilterEmpty : number;
		minTenantIdFilter : number;
		minTenantIdFilterEmpty : number;
    maxOrderTypeIdFilter : number;
		maxOrderTypeIdFilterEmpty : number;
		minOrderTypeIdFilter : number;
        minOrderTypeIdFilterEmpty : number;
        uploadUrl 

        selectedId;
        selectedcode: string;
        header: string;
        key: string;
        file : File
		orderTypyeFilterId : number;

    _entityTypeFullName = 'SprintTek.Shipping.OrderSources.OrderSource';
    entityHistoryEnabled = false;
    orderTypeFilter : any;
    zipRegEx : any;
    constructor(
        injector: Injector,
        private _orderSourcesServiceProxy: OrderSourcesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _orderTypesServiceProxy: OrderTypesServiceProxy,
        private _router: Router,
        private _httpClient: HttpClient,
        private router: Router,
        private _progressBarService: ProgressBarService,
        private _orderServiceProxy: OrderServiceProxy,
    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/Import/ImportFromExcel';
    }

    ngOnInit(): void {
        this.backgroundProcessListener();
        $('.kt-select2').select2();
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
        this.getOrderTypes();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }
    getOrderTypes(){
        this._orderTypesServiceProxy.getAll(undefined,
            undefined,
            undefined,
            -1,
            undefined,
            undefined,
            undefined,
            1000
            ).subscribe(result => {						
            this.orderTypeFilter = result.items;
            console.log(this.orderTypeFilter);
        });
    }
    getOrderSources(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        this.orderTypyeFilterId = Number((<HTMLInputElement>document.getElementById('uotId')).value) ? Number((<HTMLInputElement>document.getElementById('uotId')).value) : undefined;
        this._orderSourcesServiceProxy.getAll(
            this.filterText,
            this.nameFilter,
            this.codeFilter,
            this.maxIsActiveFilter == null ? this.maxIsActiveFilterEmpty: this.maxIsActiveFilter,
            this.minIsActiveFilter == null ? this.minIsActiveFilterEmpty: this.minIsActiveFilter,
            this.maxTenantIdFilter == null ? this.maxTenantIdFilterEmpty: this.maxTenantIdFilter,
            this.minTenantIdFilter == null ? this.minTenantIdFilterEmpty: this.minTenantIdFilter,
            this.orderTypyeFilterId,
            this.zipRegEx,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    fileClick(code, id, header, key){
        this.selectedcode = code
        this.selectedId = id
        this.header = header;
        this.key = key;
        $('input[type="file"]').trigger('click')
    }


    importOrder( ){

        this.hubConnection
            .start()
            .then()
            .catch(err => {
                console.log('Error while starting connection: ' + err);
                this.toggle(false);
            });

        const formData: FormData = new FormData();
        
        formData.append('file', this.file, this.file.name);
        formData.append('key', this.key);
        formData.append('orderSourceCode',this.selectedcode);
        formData.append('orderSourceId',this.selectedId);
        formData.append('isHeaderFixed', this.header);
        formData.append('locale', abp.localization.currentLanguage.name);
        formData.append('loadDate', moment().tz(localStorage.getItem('timeZoneId')).format());
        this._httpClient
            .post<any>(this.uploadUrl, formData)
            .subscribe(response => {
                if (response.success) {

                    console.log(response);

                    let trackHangfireJobParam = new TrackImportInput({
                        jobId: response.result.jobId
                    });

                    let observable = interval(1500).subscribe((val) => {
                        this._orderServiceProxy.trackImport(trackHangfireJobParam).subscribe(watcher => {
                            
                            if (watcher.result == "DONE") { 
                                this.toggle(false);
                                this.notify.success('The import is complete.');
                                observable.unsubscribe();
                            }
                        });
                    });


                    this.notify.success('Import process is queued. Please wait');
                    this.toggle(true);
                } else if (response.error != null) {
                    this.notify.error(this.l('ImportOrderFailed'));
                    this.toggle(false);
                }
            });  
     }

    backgroundProcessListener() {
        this.hubConnection = new signalR.HubConnectionBuilder()
            .withUrl(AppConsts.remoteServiceBaseUrl + '/import')
            .build();

        this.hubConnection.on('BackgroundMessage', (message) => {
            
            if (message.match(/End Importing/)) {
                this.toggle(false);
                this.notify.success('Import process is complete');

                this.hubConnection.stop();
            } 
            
            this.sendMessage(message);
        });

    }
     
     onFileSelect(data: { files: File } ){
        this.file = data.files[0];
        this.importOrder();
     }    
     
     onUploadExcelError(): void {
        this.notify.error(this.l('ImportOrderFailed'));
    }
    

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }
    openOrderSource(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        this._router.navigateByUrl(myurl);

    }

    createOrderSource(): void {
        this.createOrEditOrderSourceModal.show();
    }

    showHistory(orderSource: OrderSourceDto): void {
        this.entityTypeHistoryModal.show({
            entityId: orderSource.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteOrderSource(orderSource: OrderSourceDto): void {
        this.message.confirm(
            this.l(''),
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderSourcesServiceProxy.delete(orderSource.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._orderSourcesServiceProxy.getOrderSourcesToExcel(
        this.filterText,
            this.nameFilter,
            this.codeFilter,
            this.maxIsActiveFilter == null ? this.maxIsActiveFilterEmpty: this.maxIsActiveFilter,
            this.minIsActiveFilter == null ? this.minIsActiveFilterEmpty: this.minIsActiveFilter,
            this.maxTenantIdFilter == null ? this.maxTenantIdFilterEmpty: this.maxTenantIdFilter,
            this.minTenantIdFilter == null ? this.minTenantIdFilterEmpty: this.minTenantIdFilter,
            this.maxOrderTypeIdFilter == null ? this.maxOrderTypeIdFilterEmpty: this.maxOrderTypeIdFilter,
            this.zipRegEx
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    toggle (toggle: boolean){
        this._progressBarService.toggleProgressBar(toggle);
    }

    sendMessage(message: string) {
        this._progressBarService.sendMessage(message);
    }

}
