import { Component, ViewChild, Injector, Output, EventEmitter } from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import {
    TimeZoneServiceProxy,
    OrderNotificationOrderStatusTypeLookupTableDto,
    UserListDto,
    OrganizationUnitDto,
    OrderNotificationSmsTemplateLookupTableDto,
    OrderNotificationOrderTypeLookupTableDto,
    OrderNotificationsServiceProxy, CreateOrEditOrderNotificationDto, OrderNotificationOrderNotificationTemplateLookupTableDto
    , OrganizationUnitServiceProxy, CreateOrEditOrganizationUnitOrderNotificationDto, OrganizationUnitOrderNotificationsServiceProxy, OrganizationUnitOrderNotificationDto, ReportScheduleServiceProxy, CreateOrEditReportScheduleDto, CreateOrEditReportRecipientDto, ReportSchedulerServiceProxy, LocationServiceProxy, LocationNameDto, ParameterValueNameDto, ReportScheduleParameterCheckerInput, CreateReportSchedulerFtpServersDto
} from '@shared/service-proxies/service-proxies';

import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
import { NUMBER_TYPE } from '@angular/compiler/src/output/output_ast';
import { ColorTransformCollection } from 'devexpress-richedit/lib/core/model/drawing/transform/color-transform-collection';
import { value } from 'devexpress-dashboard/model/index.metadata';
import { CommonLookupModalComponent } from '@app/shared/common/lookup/common-lookup-modal.component';
import { CommonLookupCheckboxModalComponent } from '@app/shared/common/lookup/common-lookup-checkbox-modal.component';
declare var $: any;

@Component({
    selector: 'createReportScheduleModalComponent',
    templateUrl: './create-report-schedule-modal.component.html',
    providers: [ReportScheduleServiceProxy, ReportSchedulerServiceProxy]
})
export class CreateReportScheduleModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @ViewChild('parameterModal', {static: true}) parameterModal: CommonLookupModalComponent;
    @ViewChild('parameterMultiModal', {static: true}) parameterMultiModal: CommonLookupCheckboxModalComponent;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;
    delete: any;
    form: any;
    isEmail: boolean;
    isSMS: boolean;
    organizationUnitId: any;
    // allOrganizationUnits: any;
    orderNotification: CreateOrEditOrderNotificationDto = new CreateOrEditOrderNotificationDto();
    activeTabIndex: number = 0;
    orderNotificationTemplateDescription = '';
    orderStatusTypeDescription = '';
    smsTemplateDescription = '';
    orderTypeDescription = '';
    OrganizationalUnitNotification: any;
    organizationUnitOrderNotification: any;
    orgUnits: CreateOrEditOrganizationUnitOrderNotificationDto = new CreateOrEditOrganizationUnitOrderNotificationDto();
    allOrderNotificationTemplates: OrderNotificationOrderNotificationTemplateLookupTableDto[];
    allOrderStatusTypes: OrderNotificationOrderStatusTypeLookupTableDto[];
    allSmsTemplates: OrderNotificationSmsTemplateLookupTableDto[];
    allOrderTypes: OrderNotificationOrderTypeLookupTableDto[];
    allUsers: UserListDto[];
    allOrganizationUnits: OrganizationUnitDto[];
    addOrg: any;

    input: any;
    // input: CreateOrEditReportScheduleDto= new CreateOrEditReportScheduleDto();
    reportId: number;
    schedulerDescription: string;
    schedulerStartDate: Date;
    schedulerStartTime: any;
    repeat: number;
    startDay : number;
    exportType: any;
    schedulerIsActive: boolean = true;
    ftpIsActive : boolean = false;
    recipientInput: any
    reportScheduleRecipient: any;
    exists: boolean = false
    params: any
    getLocationNames: LocationNameDto[];
    dropdownValidation : boolean = false
    repeatValidation : boolean = false
    startDayValidation : boolean = false
    exportTypeValidation : boolean = false
    startDayValue : any
    repeatValue : any
    parameterNameValue: ParameterValueNameDto = new ParameterValueNameDto();
    listParameterValueName : Array<{
        pname: string,
        pvalue: string
    }> = new Array();
    timeZones: any;
    defaultTz: any;
    ftpServerName: string;
    ftpUserName: string;
    ftpPassword: string;
    ftpFolderName: string;
    timeInterval : any;
    min : number
    max: number
    dynamicModel: any[] = []
    isSftp : boolean = false
    
    constructor(
        injector: Injector,
        private _orderNotificationsServiceProxy: OrderNotificationsServiceProxy,
        private _reportScheduleServiceProxy : ReportScheduleServiceProxy,
        private _organizationUnitService: OrganizationUnitServiceProxy,
        private _organizationUnitOrderNotification: OrganizationUnitOrderNotificationsServiceProxy,
        private _scheduler: ReportSchedulerServiceProxy,
        private _locationServiceProxy: LocationServiceProxy,
        private _timeZoneServiceProxy: TimeZoneServiceProxy
    ) {
        super(injector);
    }

    ngOnInit(): void {
        $('.kt-select2').select2();
        $('.organization-unit').select2();
        $('.user').select2();
        this.getParamInit();
        this.getParamMultiInit();
    }

    onShown(): void {
        let s = this;
        this.repeatValidation = false;
        this.startDayValidation = false
        this.exportTypeValidation = false
        $('.kt-select2').select2();
        $('.organization-unit').select2();
        $('.user').select2();
        
        var that = this
        that.repeatValue = 0
        $('#repeatInput').change(function() {
            that.repeatValidation=true
            that.repeatValue= $('#repeatInput').val();
            setTimeout(() => {
                $('#startDayInput').select2();
            }, 1000);
         });

         //startDateValidation
         $('#startDayInput').change(function() {
            that.startDayValidation=true
         });

         $('#exportTypeInput').change(function() {
            that.exportTypeValidation=true
         });
    }

    show(reportId: number): void {
        let s = this;
        this.reportId = reportId
        this.reportScheduleRecipient = []
        this.spinnerService.show();
        this.reportScheduleRecipient = []
        this.exportType = []
        this.organizationUnitId = "";
        this.reportId = reportId;
        this.schedulerIsActive = false
        this.repeat = null;
        this.schedulerDescription= ""
        this.schedulerStartDate = null;
        this.schedulerStartTime = null;
        this.active = true;
        this.getParams();
        this.modal.show();
        this.repeatValidation = false;                          
        this._orderNotificationsServiceProxy.getAllUserForTableDropdown().subscribe(result => {
            this.allUsers = result;
        });

        this._orderNotificationsServiceProxy.getAllOrgUnitsForTableDropdown().subscribe(result => {
            this.allOrganizationUnits = result;
        });

        this.defaultTz = 'UTC-11';
        this._timeZoneServiceProxy.getTimeZoneForDropDown(
        ).subscribe(result => {
            this.timeZones = result.items;
        });
    }

    

    updateUnits(id, data): void {
        this.reportScheduleRecipient[id].organizationUnitId = data;
        setTimeout(() => {
            $('.organization-unit').select2();
            $('.user').select2();
        }, 100);
    }

    updateUsers(id, data): void {
        this.reportScheduleRecipient[id].userId = data;

        setTimeout(() => {
            $('.organization-unit').select2();
            $('.user').select2();
        }, 100);

    }

    updateRecipientType(id, data): void {
        let s = this;
        // this.organizationUnitOrderNotification[id].code = data;
        this.reportScheduleRecipient[id].code = data;
        setTimeout(() => {
            $('.organization-unit').select2();
            $('.user').select2();

            $('.organization-unit').on('select2:select', function (e) {
                s.updateUnits($(e.currentTarget).attr('id'), $(e.currentTarget).val());
            });
            $('.user').on('select2:select', function (e) {
                s.updateUsers($(e.currentTarget).attr('id'), $(e.currentTarget).val());
            });
        }, 100);


    }


    updateIsActive(id, data): void {
        this.reportScheduleRecipient[id].isActive = data;
    }

    addRecipient(): void {
        let s = this;
        
        this.recipientInput = new CreateOrEditReportRecipientDto();
        this.recipientInput.emailAddress = ""
        this.recipientInput.isActive = false
        this.recipientInput.organizationUnitId = null
        this.recipientInput.userId = null
        this.recipientInput.code = null

        this.reportScheduleRecipient.push(this.recipientInput);
        setTimeout(() => {
            $('.organization-unit').select2();
            $('.user').select2();
        }, 10);
    }

    deleteRecipient(id: number): void {
        this.delete = this.reportScheduleRecipient[id];
        this._reportScheduleServiceProxy.deleteReportScheduleRecipient(this.delete.id)
                                        .subscribe(result => {
                                            this.notify.success(this.l('SuccessfullyDeleted'));
                                        });
        this.reportScheduleRecipient.splice(id, 1);
    }

    save(){
        // this.spinnerService.show()
        this.saving = true;
        this.input = new CreateOrEditReportScheduleDto();
        this.input.reportId = this.reportId
        this.input.description = this.schedulerDescription;
        this.input.startDate = this.schedulerStartDate;
        this.input.startDate = new Date(this.schedulerStartDate.setHours(this.schedulerStartDate.getHours() - this.schedulerStartDate.getTimezoneOffset() / 60));
        this.input.startTime = new Date(this.schedulerStartTime).toLocaleTimeString('en-US', { hour12: true, 
                                hour: "numeric", 
                                minute: "numeric",
                                second: "numeric"
                                }) as any;
                                
        this.input.repeat = $('#repeatInput').val();
        this.input.startDay = $('#startDayInput').val() ;
        this.exportType = $('#exportTypeInput').val();
        this.input.exportType = this.exportType.join(",").replace(/[^a-zA-Z,]/g,"");

        this.input.isActive = this.schedulerIsActive;
        this.input.ftpIsActive = this.ftpIsActive;

        this.input.recipients = new CreateOrEditReportRecipientDto();
        this.input.recipients = this.reportScheduleRecipient;
        this.input.parameterNameValue = []

        var tz = $('#timezone').val();
        this.input.timeZone = tz;

        this.input.timeInterval = this.timeInterval
            
        if(this.params != undefined) {
            for(var a = 0 ; a < this.params.length ; a++){
                // console.log("save")
                var parameterNameValue: ParameterValueNameDto = new ParameterValueNameDto();
                parameterNameValue.parameterName=this.params[a].name.toString();
                // parameterNameValue.parameterValue=$('#'+this.params[a].name).val().toString();
                parameterNameValue.parameterValue=this.dynamicModel[this.params[a].name] != undefined ? this.dynamicModel[this.params[a].name].toString() : '';
                
                parameterNameValue.multiValue = this.params[a].multiValue;
                
                this.input.parameterNameValue.push(parameterNameValue)
            }
        }
        var checkerInput : ReportScheduleParameterCheckerInput = new ReportScheduleParameterCheckerInput();
        checkerInput.reportId = this.reportId;
        checkerInput.parameterNameValue = this.input.parameterNameValue
        checkerInput.recipients = this.reportScheduleRecipient;
        checkerInput.repeat = $('#repeatInput').val();

        this.input.ftpServerName = this.ftpServerName;
        this.input.ftpUserName = this.ftpUserName;
        this.input.ftpPassword = this.ftpPassword;
        this.input.ftpFolderName = this.ftpFolderName;
        this.input.isSftp = this.isSftp;

        // console.log($('#repeatInput').val() + "test")

        // this._reportScheduleServiceProxy.reportScheduleParameterChecker(checkerInput)
        //                                 .subscribe(result=>{
        //                                     if(result){
        //                                         this.message.warn(
        //                                             this.l('SchedulerParameterWarning')
        //                                         );
        //                                         this.saving= false
        //                                     }
        //                                     else{
        //                                         // this.message.warn(
        //                                         //     'false'
        //                                         // );
                                                this._reportScheduleServiceProxy.createReportSchedule(this.input)
                                                    .subscribe(result=>{
                                                        this.notify.info(this.l('SavedSuccessfully'));
                                                        this.close();
                                                        this.modalSave.emit(null);
                                                        this.spinnerService.hide()
                                                },
                                                error => {
                                                    this.saving=false
                                                },
                                                ()=>{
                                                    this.saving=false
                                                })
                                                
                                        //         // console.log("false")
                                        //     }
                                           
                                        // })

            // this._reportScheduleServiceProxy.createReportSchedule(this.input)
            //                             .subscribe(result=>{
            //                                 this.notify.info(this.l('SavedSuccessfully'));
            //                                 this.close();
            //                                 this.modalSave.emit(null);
            //                             })
    }

    ngAfterViewInit(): void {
    }


    close(): void {
        this.active = false;
        this.modal.hide();
    }

    // get params
    getParams(){
        this._reportScheduleServiceProxy.getReportParameters(this.reportId)
                       .pipe()
                       .subscribe(result=>{
                           this.params = result
                           result.forEach(result => {
                               this.dynamicModel.push(result.name)
                           });
                           setTimeout(() => {
                                $('.kt-select2').select2();
                            }, 2000);
                            this.spinnerService.hide()
                       }, error => {
                           this.spinnerService.hide();
                       })
    }

    validateValue(event: number) {
        if(this.repeatValue == 6){
            if(event >=24){
                this.message.warn("Please enter value from 1 to 23")
                this.timeInterval = null
                
            }
        }
        if(this.repeatValue == 7){
            if(event >=60){
                this.message.warn("Please enter value from 1 to 59")
                this.timeInterval = null
            }
        }
    }


    // selectAll(name: any){

    //     if($('#'+name).val().length > 0){
    //         // Unselect all
    //         $('#'+name).select2('destroy').find('option').prop('selected', false).end().select2();
            
    //     }
    //     else{
    //         // Select all
    //         $('#'+name).select2('destroy').find('option').prop('selected', 'selected').end().select2();
    //     }
    // }

    getParameterList(parameterName : string, reportId : number, multiValue : boolean){
        if(!multiValue){
            this.parameterModal.parameterName = parameterName
            this.parameterModal.show();
        }
        else{
            this.parameterMultiModal.parameterName = parameterName
            this.parameterMultiModal.show();
        }
    }

    getParamInit() {
        this.parameterModal.configure({
            title: this.l('Select Parameter Value' ),
            dataSource: (skipCount: number, maxResultCount: number, filter: string, tenantId?: number) => {
                return this._reportScheduleServiceProxy.getReportDefaultParameterValues(this.reportId,this.parameterModal.parameterName,filter,false,undefined,skipCount,maxResultCount);
            }
        });
    }

    getParamMultiInit() {
        this.parameterMultiModal.configure({
            title: this.l('Select Parameter Value' ),
            dataSource: (skipCount: number, maxResultCount: number, filter: string, tenantId?: number) => {
                return this._reportScheduleServiceProxy.getReportDefaultParameterValues(this.reportId,this.parameterMultiModal.parameterName,filter,false,undefined,skipCount,maxResultCount);
            }
        });
    }

    selectParameter(item){
        this.dynamicModel[item.paramName] = item.value.toString()
        $('#'+item.paramName).val(item.name)
        
    }

    selectMultiParameter(item){
        this.dynamicModel[item[0].paramName] = item.map(o=> o.value).toString()
        $('#'+item[0].paramName).val(item.map(o=> o.name).toString())

        // this.dynamicModel[item[0].paramName] = item.map(o=> o.value)
        // // $('#'+item[0].paramName).val(item.map(o=> o.name))

        // console.log(item)

        // item.map(c=>{
        //     this.dynamicModel[item[0].paramName].push(c.value)
        // })
        // // this.dynamicModel[item[0].paramName].push(c.value)
    }

    clearInput(paramName){
        this.dynamicModel[paramName] =undefined
        $('#'+paramName).val('')
        console.log(paramName)
    }

    // addCategory(categories: []){
    //     categories.map(c => {
    //         this.categoryType.push(c)
    //     })
    //     this.categoryTypeArray = this.categoryType.map(c => c.value);
    // }

    removeLocation(item){
        // this.locations = this.locations.filter(c => c.name != item.value);
        // this.routes = this.routes.filter(c => c.locationId != item.id);
        
    }

    testConnection(){
        let test = new CreateReportSchedulerFtpServersDto();

        test.ftpServerName = this.ftpServerName;
        test.ftpUserName = this.ftpUserName;
        test.ftpPassword = this.ftpPassword;
        test.ftpIsSftp = this.isSftp;

        this._reportScheduleServiceProxy.testFtpConnection(test).subscribe(result =>{
            if(result){
                this.message.success("Connection Success","Test FTP Connection")
            }
            else{
                this.message.error("Please check input", "Test FTP Connection")
            }
        })
    }
}


