﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { Route4MeSettingsServiceProxy, CreateOrEditRoute4MeSettingDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditRoute4MeSettingModal',
    templateUrl: './create-or-edit-route4MeSetting-modal.component.html'
})
export class CreateOrEditRoute4MeSettingModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    route4MeSetting: CreateOrEditRoute4MeSettingDto = new CreateOrEditRoute4MeSettingDto();



    constructor(
        injector: Injector,
        private _route4MeSettingsServiceProxy: Route4MeSettingsServiceProxy
    ) {
        super(injector);
    }

    show(route4MeSettingId?: number): void {

        if (!route4MeSettingId) {
            this.route4MeSetting = new CreateOrEditRoute4MeSettingDto();
            this.route4MeSetting.id = route4MeSettingId;

            this.active = true;
            this.modal.show();
        } else {
            this._route4MeSettingsServiceProxy.getRoute4MeSettingForEdit(route4MeSettingId).subscribe(result => {
                this.route4MeSetting = result.route4MeSetting;


                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._route4MeSettingsServiceProxy.createOrEdit(this.route4MeSetting)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
