import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Route4MeSettingsServiceProxy, Route4MeSettingDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRoute4MeSettingModalComponent } from './create-or-edit-route4MeSetting-modal.component';
import { ViewRoute4MeSettingModalComponent } from './view-route4MeSetting-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './route4MeSettings.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class Route4MeSettingsComponent extends AppComponentBase {

    @ViewChild('createOrEditRoute4MeSettingModal', { static: true }) createOrEditRoute4MeSettingModal: CreateOrEditRoute4MeSettingModalComponent;
    @ViewChild('viewRoute4MeSettingModalComponent', { static: true }) viewRoute4MeSettingModal: ViewRoute4MeSettingModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    valueFilter = '';




    constructor(
        injector: Injector,
        private _route4MeSettingsServiceProxy: Route4MeSettingsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getRoute4MeSettings(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._route4MeSettingsServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.valueFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createRoute4MeSetting(): void {
        this.createOrEditRoute4MeSettingModal.show();
    }

    deleteRoute4MeSetting(route4MeSetting: Route4MeSettingDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._route4MeSettingsServiceProxy.delete(route4MeSetting.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._route4MeSettingsServiceProxy.getRoute4MeSettingsToExcel(
        this.filterText,
            this.codeFilter,
            this.valueFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
