﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { RouteTemplateOrderExtrasServiceProxy, RouteTemplateOrderExtrasDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRouteTemplateOrderExtrasModalComponent } from './create-or-edit-routeTemplateOrderExtras-modal.component';
import { ViewRouteTemplateOrderExtrasModalComponent } from './view-routeTemplateOrderExtras-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './routeTemplateOrderExtras.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RouteTemplateOrderExtrasComponent extends AppComponentBase {

    @ViewChild('createOrEditRouteTemplateOrderExtrasModal', { static: true }) createOrEditRouteTemplateOrderExtrasModal: CreateOrEditRouteTemplateOrderExtrasModalComponent;
    @ViewChild('viewRouteTemplateOrderExtrasModalComponent', { static: true }) viewRouteTemplateOrderExtrasModal: ViewRouteTemplateOrderExtrasModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    orderExtraKeyFilter = '';
    orderExtraValueFilter = '';




    constructor(
        injector: Injector,
        private _routeTemplateOrderExtrasServiceProxy: RouteTemplateOrderExtrasServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getRouteTemplateOrderExtras(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._routeTemplateOrderExtrasServiceProxy.getAll(
            this.filterText,
            this.orderExtraKeyFilter,
            this.orderExtraValueFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createRouteTemplateOrderExtras(): void {
        this.createOrEditRouteTemplateOrderExtrasModal.show();
    }


}
