import { Injector, Component, ViewChild, AfterViewInit, HostBinding, ElementRef, OnInit, Input, ViewContainerRef } from "@angular/core";
import { AppComponentBase } from "@shared/common/app-component-base";
import { Router } from "@angular/router";
import { TabsetComponent } from "ngx-bootstrap";
import * as moment from 'moment-timezone';
import {RouteAnalyzerServiceProxy,RouteCodeListInput } from '@shared/service-proxies/service-proxies';
import { XmlHttpRequestHelper } from '@shared/helpers/XmlHttpRequestHelper';
import { AppConsts } from '@shared/AppConsts';
import { AppSessionService } from "@shared/common/session/app-session.service";
@Component({
    selector: 'deliveryRouteAnalyzerFlyoutComponent',
    templateUrl: 'route-analyzer-flyout.component.html',
    // animations: [appModuleAnimation()],
    styleUrls: ['route-analyzer-flyout.component.less'],
})

export class DeliveryRouteAnalyzerFlyoutComponent extends AppComponentBase implements OnInit, AfterViewInit {

    @HostBinding('id') id = 'kt_quick_sidebar';
    @HostBinding('class')
    classes = 'kt-quick-panel';
    @HostBinding('attr.mQuickSidebarOffcanvas')

    @HostBinding('style.overflow') styleOverflow: any = 'hidden';
    public dateRange: moment.Moment[] = [moment().startOf('day'), moment().endOf('day')];
    mQuickSidebarOffcanvas: any;
    constructor(

        injector: Injector,
        private el: ElementRef,
        private router: Router,
        private viewContainerRef: ViewContainerRef,
        private _routeAnalyzerService: RouteAnalyzerServiceProxy,
    ) {
        super(injector);
          

    }
    locationName:any;
    _isOpen: boolean;
    _pinned = false;
    flyoutDate: Date[];
    locations:any;
    locationId: number;
    routeCodes:any;
    routeCode:any;
    dayInput:string;
    disabledDates = [];
    ngOnInit(): void{
        this._isOpen = true;
        $('.kt-select2').select2();
        // this.flyoutDate = moment(Date.now()).toDate();
        if (localStorage.getItem('analyzerLocationId') != null && localStorage.getItem('analyzerLocationId') != "") {
            this.locationId = parseInt(localStorage.getItem('analyzerLocationId'));
        }

        if (localStorage.getItem('deliveryAnalyzerEndDate') != null) {
            this.dateRange.push(moment(localStorage.getItem('deliveryAnalyzerStartDate')));
            this.dateRange.push(moment(localStorage.getItem('deliveryAnalyzerEndDate')));
        } else {
            this.flyoutDate = [moment(localStorage.getItem('deliveryAnalyzerStartDate')).toDate(), moment(localStorage.getItem('deliveryAnalyzerEndDate')).toDate()];
        }

        if(localStorage.getItem('deliveryAnalyzerDayFilter') == null){
            this.dayInput = 'ALL';
        } else {
            this.dayInput = localStorage.getItem('deliveryAnalyzerDayFilter');
        }

    }

    assignLocationModal(): void {
        this.getParentComponent().assignLocationModal();
    }

    deleteLocation(): void {
        this.locationId = null;
        this.locationName = '';
        this.routeCode = '';
        this.getParentComponent().deleteLocation();
        this.getParentComponent().deleteRoute();
    }

    selectLocation(): void {
        this.getParentComponent().selectLocation();
        this.getParentComponent().deleteRoute();
        this.routeCode = '';
        localStorage.removeItem('deliveryAnalyzerRouteCode');
        $('#routeCodeInput').val("");
    }

    selectRouteModal(): void {
        this.getParentComponent().selectRouteModal();
    }

    deleteRoute(): void {
        this.routeCode = '';
        this.getParentComponent().deleteRoute();
    }


    getParentComponent() {
        return this.viewContainerRef['_data'].componentView.component.viewContainerRef['_view'].component
    }

    ngAfterViewInit(): void{
        $(".select2-container").css("width", "100%");
        $(".form-control").css("height", "38px");
        $(".input-group-append").css("display", "none");

        
        this.mQuickSidebarOffcanvas = new KTOffcanvas(this.el.nativeElement, {
            overlay: true,
            baseClass: 'kt-quick-panel',
            closeBy: 'controller-header-close',
            toggleBy: 'controller_header_settings_toggle'
        });

        this.mQuickSidebarOffcanvas.events.push({
            name: 'afterHide',
            handler: () => {
                if (this._pinned) {
                    this.mQuickSidebarOffcanvas.show();
                } else {
                    this.isOpen = false;
                }
            }
        }, {
            name: 'afterShow',
            handler: () => {
                this.isOpen = true;
            }
        });


        // change location
        let that = this;
        $("#locationSelectInput").on('change',function(e){
            localStorage.setItem('analyzerLocationId', $("#locationSelectInput").val().toString());
            that.getRoutes();
        });
    }

    getRoutes(){
        this.spinnerService.show();
        var data = new RouteCodeListInput();
        this.locationId = parseInt(localStorage.getItem('analyzerLocationId'));
        data.locationId = this.locationId;
        if(localStorage.getItem('analyzerLocationId') != null && localStorage.getItem('analyzerLocationId') != ""){
            this._routeAnalyzerService.routeCodeList(data).subscribe(result => {

                this.routeCodes = result;
                this.spinnerService.hide();
          });
        } else {
            this.spinnerService.hide();
        }
        
    }

    set pinned(newValue: boolean) {
        if (newValue === this._pinned) {
            return;
        }
        this._pinned = newValue;
    }
    get pinned(): boolean {
        return this._pinned;
    }
    reversePinned(): void {
        this.pinned = !this.pinned;
    }
    set isOpen(newValue: boolean) {
        if (newValue === this._isOpen) {
            return;
        }
        this._isOpen = newValue;
    }
    get isOpen(): boolean {
        return this._isOpen;
    }

    showFlyout(location:any):void{
        let that = this;
        
        this.getRoutes();
        this.locations = location;
        this.flyoutDate = [moment(localStorage.getItem('deliveryAnalyzerStartDate')).toDate(), moment(localStorage.getItem('deliveryAnalyzerEndDate')).toDate()];
       
        $('#dayInput').on('select2:select', function (e){
           
            var data = e.params.data;
            that.dayInput = data.id;
            that.disabledDates.length = 0;
            var day = moment().subtract(60, 'days').toDate();

            if(data.id == "WEEKDAY"){
                //0,6
                for (let index = 0; index < 60; index++) {
                    
                    if(moment(day).add(index, 'days').day() == 0 || moment(day).add(index, 'days').day() == 6){
                        that.disabledDates.push(new Date(moment(day).add(index, 'days').format("YYYY-MM-DD")));
                    } 
                }
                
            } else if (data.id == "WEEKEND"){
                //1,2,3,4,5

                for (let index = 0; index < 60; index++) {
                    if(moment(day).add(index, 'days').day() == 1 || moment(day).add(index, 'days').day() == 2 || moment(day).add(index, 'days').day() == 3 || moment(day).add(index, 'days').day() == 4 || moment(day).add(index, 'days').day() == 5){
                        that.disabledDates.push(new Date(moment(day).add(index, 'days').format("YYYY-MM-DD")));
                    } 
                }
            } else {
                that.disabledDates.length = 0;
            }
        });


        if(localStorage.getItem('deliveryAnalyzerLocationString') != null){
            this.locationName =  localStorage.getItem('deliveryAnalyzerLocationString');
            $('#locationSelectInput').val(localStorage.getItem('deliveryAnalyzerLocationString'));
        } else {
            this.locationName =  $('#locationSelectInput').val();
        }

        if(localStorage.getItem('deliveryAnalyzerRouteCode') != null){
            this.routeCode =  localStorage.getItem('deliveryAnalyzerRouteCode');
            $('#routeCodeInput').val(localStorage.getItem('deliveryAnalyzerRouteCode'));
        } else {
            this.routeCode = $('#routeCodeInput').val();
        }

    }

    filterRouteAnalyzer(){
        
        // localStorage.removeItem('deliveryAnalyzerStartDate');
        // localStorage.removeItem('deliveryAnalyzerLocationId');
        // localStorage.removeItem('deliveryAnalyzerRouteCode');
        // localStorage.setItem('deliveryAnalyzerStartDate', this.dateRange[0].format("L"));
        // localStorage.setItem('deliveryAnalyzerEndDate', this.dateRange[1].format("L"));

        localStorage.setItem('deliveryAnalyzerStartDate', this.dateRange[0].toDate().getMonth() + 1 +"/"+this.dateRange[0].toDate().getDate()+"/"+this.dateRange[0].toDate().getFullYear());
        localStorage.setItem('deliveryAnalyzerEndDate', this.dateRange[1].toDate().getMonth() + 1 +"/"+this.dateRange[1].toDate().getDate()+"/"+this.dateRange[1].toDate().getFullYear());

        
        localStorage.setItem('deliveryAnalyzerDayFilter', this.dayInput);

        // if($("#locationSelectInput").val().toString() != ""){
        //     var location = $('#locationSelectInput').select2('data');
        //     localStorage.setItem('deliveryAnalyzerLocationId', $("#locationSelectInput").val().toString());
        //     if(location[0] != null || undefined){
        //         localStorage.setItem('deliveryAnalyzerLocationString', location[0].text);
        //     }
            
        // }
        
        // if($("#routeCodeInput").val().toString() != ""){
        //     localStorage.setItem('deliveryAnalyzerRouteCode', $("#routeCodeInput").val().toString());
        // }
        // var url = this.router.url;
        // var path = url.split("/")[3];
        // if( path != null || path != undefined){
        //     if(path == "route-analyzer-pickup"){
        //         this.getParentComponent().getOrdersByLocation();
        //     }

        //     if(path == "route-analyzer-delivery"){
                
        //     }
        // }

        // if (this.dayInput == 'WEEKEND') {
        //     if (moment($('.analyzerDatePicker').val().toString(), "MM-DD-YYYY").day() == 0) {
        //         this.getParentComponent().getStopsByLocation();
        //         this.mQuickSidebarOffcanvas.hide();

        //     } else if (moment($('.analyzerDatePicker').val().toString(), "MM-DD-YYYY").day() == 6) {
        //         this.getParentComponent().getStopsByLocation();
        //         this.mQuickSidebarOffcanvas.hide();
        //     } else {
        //         this.message.info("", "Selected date is not on weekend");
        //     }
        // } else if (this.dayInput == 'WEEKDAY') {
        //     if (moment($('.analyzerDatePicker').val().toString(), "MM-DD-YYYY").day() == 6) {
        //         this.message.info("", "Selected date is not a weekday");
        //     } else if (moment($('.analyzerDatePicker').val().toString(), "MM-DD-YYYY").day() == 0) {
        //         this.message.info("", "Selected date is not a weekday");
        //     } else {
        //         this.getParentComponent().getStopsByLocation();
        //         this.mQuickSidebarOffcanvas.hide();
        //     }
        // } else {
        //     this.getParentComponent().getStopsByLocation();
        //     this.mQuickSidebarOffcanvas.hide();
        // }
        this.getParentComponent().getStopsByLocation();
        this.mQuickSidebarOffcanvas.hide();
    }
    getOrdersByLocation(){
        this.getParentComponent().getOrdersByLocation();
    }

    getStopsByLocation(){
        this.getParentComponent().getStopsByLocation();
    }

}
