import { Injector, Component, ViewChild, AfterViewInit, HostBinding, ElementRef, OnInit, Directive, HostListener, Input, Output, EventEmitter, ViewContainerRef } from "@angular/core";
import { AppComponentBase } from "@shared/common/app-component-base";
import { appModuleAnimation } from "@shared/animations/routerTransition";
import { LocalStorageService } from "@shared/utils/local-storage.service";
import { Router } from "@angular/router";
import { ModalDirective } from "ngx-bootstrap";
import { LazyLoadEvent } from "primeng/api";
import * as moment from 'moment';
import { ProfileServiceProxy, CurrentUserProfileEditDto } from '@shared/service-proxies/service-proxies';
@Component({

    selector: 'driverProximityFlyOutComponent',
    templateUrl: 'driver-proximity-filter-component.html',
    animations: [appModuleAnimation()]

})

export class DriverProximityFlyOutComponent extends AppComponentBase implements OnInit, AfterViewInit {

    @HostBinding('id') id = 'kt_quick_sidebar';
    @HostBinding('class')
    classes = 'kt-quick-panel';
    @HostBinding('attr.mQuickSidebarOffcanvas')

    @HostBinding('style.overflow') styleOverflow: any = 'hidden';

    mQuickSidebarOffcanvas: any;
    showLocationIcon: boolean = false;
    showPackageTypeIcon: boolean = false;
    showFleetIcon: boolean = false
    showDriverIcon: boolean = false
    showContactsIcon: boolean = false
    showCorporateIdentity: boolean = false
    showAccountsIcon = false

    hideEmpty: boolean = false;
    hideCompleted: boolean = true;
    hidePickedUp: boolean = true;
    rowsToShow: number = 50;
    todaysDate: Date;
    locationId: number;
    locations: any;
    filters: {
        locationFilter: number,
        statusFilter: number,
        typeFilter: number,
        startDate: Date,
        endDate: Date,
        orderStatusCode: string
    } = <any>{};
    timezone: string;
    currentRoutes: any;
    public userProfile: CurrentUserProfileEditDto;
    flyoutDate: Date;
    timeId: any;
    constructor(

        injector: Injector,
        private _localStorageService: LocalStorageService,
        private el: ElementRef,
        private router: Router,
        private viewContainerRef: ViewContainerRef,
        private _profileService: ProfileServiceProxy
    ) {
        super(injector);
        this.todaysDate = new Date(Date.now());
        this.currentRoutes = this.router.url;
        this.currentRoutes = this.currentRoutes.toString().split('/');
        this.currentRoutes = this.currentRoutes[this.currentRoutes.length - 1];
    }
    _isOpen: boolean;
    _pinned = false;
    getParentComponent() {
        return this.viewContainerRef['_data'].componentView.component.viewContainerRef['_view'].component
    }
    ngOnInit(): void {
        this.timeId = localStorage.getItem('timeZoneId');
        if (localStorage.getItem('loadDate') == null) {
            $(".controllerDatePicker1").val(moment(this.todaysDate).format('MM/DD/YYYY'));  
            this.flyoutDate = moment(Date.now()).toDate();
        }
        else {
            // $(".controllerDatePicker1").val(localStorage.getItem('dateInDriverproximity'));
            if(localStorage.getItem('loadDate').search('-') > 0)
            {
                var str = localStorage.getItem('loadDate').split('-');
                var str1 = str[1] + '/' + str[2] + '/' + str[0];
                localStorage.setItem('loadDate',str1);
            }
            $(".controllerDatePicker1").val(localStorage.getItem('loadDate'));
            this.flyoutDate = moment.tz(localStorage.getItem('loadDate'), this.timeId).toDate()
        }
        if (localStorage.getItem('operatingLocationId') != null) {
            this.locationId = parseInt(localStorage.getItem('operatingLocationId'));
        }

    }

    ngAfterViewInit(): void {

        this.mQuickSidebarOffcanvas = new KTOffcanvas(this.el.nativeElement, {
            overlay: true,
            baseClass: 'kt-quick-panel',
            closeBy: 'driver-proximity-close',
            toggleBy: 'driver-proximity-toggle'
        });

        this.mQuickSidebarOffcanvas.events.push({
            name: 'afterHide',
            handler: () => {
                if (this._pinned) {
                    this.mQuickSidebarOffcanvas.show();
                } else {
                    this.isOpen = false;
                }
            }
        }, {
            name: 'afterShow',
            handler: () => {
                this.isOpen = true;
            }
        });
    }
    set pinned(newValue: boolean) {
        if (newValue === this._pinned) {
            return;
        }
        this._pinned = newValue;
    }
    get pinned(): boolean {
        return this._pinned;
    }
    reversePinned(): void {
        this.pinned = !this.pinned;
    }
    set isOpen(newValue: boolean) {
        if (newValue === this._isOpen) {
            return;
        }
        this._isOpen = newValue;
    }
    get isOpen(): boolean {
        return this._isOpen;
    }


    switchHideEmpty(event) {
        console.log(event.target.checked);
        this.hideEmpty = event.target.checked;
    }
    switchHideCompleted(event) {
        console.log(event.target.checked);
        this.hideCompleted = event.target.checked;
    }
    switchHidePickedUp(event) {
        console.log(event.target.checked);
        this.hidePickedUp = event.target.checked;
    }

    getDataFilter(event?: LazyLoadEvent) {
        // console.log(this.hideEmpty);
        // console.log(this.hideCompleted);
        // console.log(this.hidePickedUp);
        // console.log(this.rowsToShow);
    }

    showFlyout(x: any): void {
        $('.kt-select2').select2();
        
        // var locationss = this.getParentComponent().locations;
        // console.log(x);
        let that = this;
        this.locations = x;
        $("#locationSelectInput").val(x["0"].id);
        $('#locationSelectInput').on('change', function () {
            $("#locationSelectInput").val($(this).val());
            that.getParentComponent().filters.locationFilter = $("#locationSelectInput").val();
        });
        if (localStorage.getItem('operatingLocationId') != null) {
            this.locationId = parseInt(localStorage.getItem('operatingLocationId'));
        }
        // this.filters.locationFilter = x["0"].id;
        // this.userProfile = data;
    }

    filterDriverProximity() {

        let that = this;
        var lang = abp.localization.currentLanguage.name;
        console.log(lang);
        if(lang=="en-GB"){
            var str = $('.controllerDatePicker1').val().toString().split('/');
            var str1 = str[1] + '/' + str[0] + '/' + str[2];
            localStorage.setItem('loadDate', str1);
        }else{
            var str = $('.controllerDatePicker1').val().toString().split('/');
            var str1 = str[0] + '/' + str[1] + '/' + str[2];
            localStorage.setItem('loadDate', str1);
        }
        // that.getParentComponent().filters.startDate = $(".controllerDatePicker1").val();
        // that.getParentComponent().filters.endDate = $(".controllerDatePicker1").val();

        that.getParentComponent().filters.startDate =  moment(localStorage.getItem('loadDate'));
        that.getParentComponent().filters.endDate =  moment(localStorage.getItem('loadDate'));
        that.getParentComponent().filters.locationFilter = $("#locationSelectInput").val();
        localStorage.setItem('operatingLocationId', $("#locationSelectInput").val().toString());  

        that.getParentComponent().filter();
        // this.locations.forEach(element => {
        //     if (element.id == $("#locationSelectInput").val()) {
        //         this.timezone = element.timeZone.value;
        //     }
        // });
        // this.userProfile.timezone = this.timezone;
        // this._profileService.updateCurrentUserProfile(this.userProfile).subscribe(() => {
        //     localStorage.setItem('timeZone', this.timezone);
        // });
    }
}