import { Component, ViewChild, Injector, Output, EventEmitter } from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { AddressTypesServiceProxy, CreateOrEditAddressTypeDto, ProximityServiceProxy, CreateProximityInput, UpdateProximityInput } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';


@Component({
    selector: 'createOrEditAddressTypeModal',
    templateUrl: './create-or-edit-addressType-modal.component.html'
})
export class CreateOrEditAddressTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    addressType: CreateOrEditAddressTypeDto = new CreateOrEditAddressTypeDto();
    proximityInput: CreateProximityInput = new CreateProximityInput();
    proximityEditInput: UpdateProximityInput = new UpdateProximityInput();
    proxValue: any;
    
    addressTypeId: number;



    constructor(
        injector: Injector,
        private _addressTypesServiceProxy: AddressTypesServiceProxy,
        private _proximityServiceProxy: ProximityServiceProxy,

    ) {
        super(injector);
    }

    show(addressTypeId?: number): void {
        this.addressTypeId = addressTypeId;
        if (!addressTypeId) {
            this.addressType = new CreateOrEditAddressTypeDto();
            this.proximityInput = new CreateProximityInput();
            this.addressType.id = addressTypeId;
            this.addressType.color = '#000000';
            // this.addressType.icon =  "fas fa-address-card"
            this.active = true;
            this.modal.show();
            console.log(this.addressTypeId);
        } else {
            this._addressTypesServiceProxy.getAddressTypeForEdit(addressTypeId).subscribe(result => {
                this.addressType = result.addressType;
                this.addressType.name = result.addressType.name;
                this.addressType.proximityValue = result.addressType.proximityValue;
                this.addressType.color = result.addressType.color==null? "#333333" : result.addressType.color
                this.addressType.icon = result.addressType.icon
                this.proximityInput.value = result.addressType.proximityValue;
                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
        this.saving = true;
        if(this.addressType.icon != undefined){
            if(this.addressType.icon.indexOf('fas fa') >=0){
                this.addressType.icon=this.addressType.icon
            }
            else if(this.addressType.icon.indexOf('fas fa') < 0 && this.addressType.icon != ''){
                this.addressType.icon='fas fa-'+this.addressType.icon
            }
        }
        else{
            this.addressType.icon = 'fas fa-address-card'
        }
        if(this.addressTypeId==undefined || this.addressTypeId==null){
            this._addressTypesServiceProxy.create(this.addressType)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(addressTypeId => {
                this.proximityInput.addressTypeId = addressTypeId;
                this.proximityInput.locationId = parseInt(localStorage.getItem('operatingLocationId'));
                this.proximityInput.value = this.addressType.proximityValue
                this._proximityServiceProxy.createProximity(this.proximityInput)
                    .pipe(finalize(() => { this.saving = false; }))
                    .subscribe(() => {
                        this.notify.info(this.l('SavedSuccessfully'));
                        this.close();
                        this.modalSave.emit(null);
                    });
            });
        }else{
            this._addressTypesServiceProxy.update(this.addressType)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(addressTypeId => {
                this.proximityEditInput.addressTypeId = addressTypeId;
                this.proximityEditInput.locationId = parseInt(localStorage.getItem('operatingLocationId'));
                this.proximityEditInput.value = this.addressType.proximityValue;
                this._proximityServiceProxy.updateProximity(this.proximityEditInput)
                    .pipe(finalize(() => { this.saving = false; }))
                    .subscribe(() => {
                        this.notify.info(this.l('SavedSuccessfully'));
                        this.close();
                        this.modalSave.emit(null);
                    });
            });
        }
       
    }







    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
