import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { AddressesServiceProxy, CreateOrEditAddressDto, UpdateAddressInput, PostalCodeServiceProxy, CreatePostalCodeInput } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

import { AddressFormComponent } from '@app/shared/layout/form/address-form.component';


@Component({
    selector: 'createOrEditAddressModal',
    templateUrl: './create-or-edit-address-modal.component.html'
})
export class CreateOrEditAddressModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @ViewChild(AddressFormComponent, {static:false}) addressForm;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    address: CreateOrEditAddressDto = new CreateOrEditAddressDto();
    postalCodeInput: CreatePostalCodeInput = new CreatePostalCodeInput();
    address2: UpdateAddressInput = new UpdateAddressInput();
    
    postalCodeValue = '';
    geocodeLatitude = '';
    formattedAddress = '';
    geocodeLongitude = '';
    addressTypeName : any;
    postalCodeId : number;
    geocodeId : number;
    postalCodeCountry : string;
    stateFeature: boolean

    isLoadingData: boolean;

    constructor(
        injector: Injector,
        private _addressesServiceProxy: AddressesServiceProxy,
        private _postalCodeServiceProxy: PostalCodeServiceProxy
    ) {
        super(injector);
    }

    ngOnInit(){
        // $('.kt-select2').select2();
        this.stateFeature = abp.features.isEnabled('App.StateFeature');
    }

    onShown(){
        $('.kt-select2').select2();
    }

    show(addressId?: number): void {
        
        this.isLoadingData = true;

        if (!addressId) {
            this.address = new CreateOrEditAddressDto();
            this.address.id = addressId;
            this.postalCodeValue = '';
            this.geocodeLatitude = '';
            this.geocodeLongitude = '';
            this.addressTypeName = '';
            this.formattedAddress = '';
            this.address.addressTypeId = null;
            this.active = true;
            this.isLoadingData = false;
            this.modal.show();
        } else {

            this._addressesServiceProxy.getAddressForEdit(addressId).subscribe( result => {
                
                this.address = result.address;

                this.postalCodeValue = result.postalCodeValue;
                this.geocodeId = result.geocodeId;
                this.addressTypeName = result.addressTypeName;

                this._postalCodeServiceProxy.getPostalCodeEdit(this.address.postalCodeId)
                    .pipe()
                    .subscribe( postalcodeResult => {

                        this.addressForm.formattedAddress = result.address.formattedAddress;
                        this.addressForm.city = result.city;
                        this.addressForm.state = result.state;
                        this.addressForm.value = postalcodeResult.value;
                        this.addressForm.country = result.country;
                        this.addressForm.latitude = result.geocodeLatitude;
                        this.addressForm.longitude = result.geocodeLongitude;
                        this.addressForm.overridableAddressType = result.address.overridableAddressType;
                        this.postalCodeValue = postalcodeResult.value;

                        this.isLoadingData = false;
                    });

                this.active = true;

                this.modal.show();
            });
        }
    }

    save(): void {

        this.saving = true;

        this.address = this.addressForm.address;

        this.address.addressTypeId = Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value) == 0 
            ? null 
            : Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value);


        this.postalCodeInput.city = this.addressForm.city;
        this.postalCodeInput.state = this.addressForm.state;
        this.postalCodeInput.value = this.addressForm.value;
        this.postalCodeInput.country = this.addressForm.country;

        
        this.address.latitude = this.addressForm.latitude;
        this.address.longitude = this.addressForm.longitude;


        this._postalCodeServiceProxy.createPostalCode(this.postalCodeInput)
            .pipe()
            .subscribe(postalCodeResult => {
                
                this.address.postalCodeId = postalCodeResult;

                this._addressesServiceProxy.createOrEdit(this.address)
                    .pipe(finalize(() => { this.saving = false;}))
                    .subscribe((
                        ) => {
                        
                        this.notify.info(this.l('SavedSuccessfully'));
                        this.close();
                        this.modalSave.emit(null);
                    },
                    error => {
                        this.saving = false;
                    });
            },
            error => {
                this.saving = false;
            });
            
    }

    setPostalCodeIdNull() {
        this.address.postalCodeId = null;
        this.postalCodeValue = '';
    }

    setGeocodeIdNull() {
        this.address.geocodeId = null;
        this.geocodeLatitude = '';
    }
        setAddressTypeIdNull() {
        this.address.addressTypeId = null;
        this.addressTypeName = '';
    }


    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
