import { Component, ViewEncapsulation, Input, ViewChild, Injector, EventEmitter ,Output } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AddressesServiceProxy, AddressTypesServiceProxy, NameValueDto, SetOverridableAddressTypeInput } from '@shared/service-proxies/service-proxies';
import { CommonLookupModalComponent } from '@app/shared/common/lookup/common-lookup-modal.component';
@Component({
    encapsulation: ViewEncapsulation.None,
    selector: 'setOverridableAddressTypeModal',
    templateUrl: './set-overridable-address-type-modal.component.html',
    animations: [appModuleAnimation()],
    // styleUrls: ['./addresses.component.less']
})
export class SetOverridableAddressTypeModalComponent extends AppComponentBase {

    @ViewChild('overridableAddressTypeModal', { static: true }) modal: ModalDirective;
    @Input() addressIds = [] as any;
    @Input() addressTypeNames = [] as any;
    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();
    
    @ViewChild("addressTypeModal", { static: true })
    addressTypeModal: CommonLookupModalComponent;
    
    overridableInput: SetOverridableAddressTypeInput = new SetOverridableAddressTypeInput();
    overridableAddressTypeInput: boolean = false;
    saving = false;
    addressTypeName: string;
    selectedAddressTypeId;

    constructor(
        injector: Injector,
        private _address: AddressesServiceProxy,
        private _addressTypes: AddressTypesServiceProxy,
    ) {
        super(injector);
    }

    save(): void {

        this.saving = true;

        this.overridableInput.addressIds = this.addressIds;
        this.overridableInput.value = this.overridableAddressTypeInput;
        this.overridableInput.addressTypeId = this.selectedAddressTypeId;

        this._address.setOverridableAddressType(this.overridableInput)
            .pipe(finalize(() => { this.saving = false;}))
            .subscribe(result => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
            });
    }

    show(): void {

        this.modal.show();
        
        this.addressTypeModal.configure({
            title: this.l("Set Address Type"),
            dataSource: (
                skipCount: number, 
                maxResultCount: number, 
                filter: string, 
                tenantId?: number, 
                locationId?: number
            ) => {
                return this._address.getAddressTypesForSelect(
                    undefined,
                    filter,
                    filter,
                    undefined,
                    maxResultCount,
                    skipCount);
            },
        });

        if(this.addressTypeNames.length == 1){
            this.addressTypeName = this.addressTypeNames[0];
        }
    }

    assignAddressType(): void {
        this.addressTypeModal.show();
    }

    selectAddressType(item: NameValueDto): void {
        this.selectedAddressTypeId = parseInt(item.value);
        this.addressTypeName = item.name;
    }

    clearAddressType(){
        this.addressTypeName = "";
        this.selectedAddressTypeId = undefined;
    }


    close(): void {

        this.addressTypeName = "";
        this.selectedAddressTypeId = undefined;
        this.modal.hide();
    }

}
