import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { PostalCodesServiceProxy, PostalCodeDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditPostalCodeModalComponent } from './create-or-edit-postalCode-modal.component';
import { ViewPostalCodeModalComponent } from './view-postalCode-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './postalCodes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class PostalCodesComponent extends AppComponentBase {

    @ViewChild('createOrEditPostalCodeModal', { static: true }) createOrEditPostalCodeModal: CreateOrEditPostalCodeModalComponent;
    @ViewChild('viewPostalCodeModalComponent', { static: true }) viewPostalCodeModal: ViewPostalCodeModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    countryFilter = '';
    cityFilter = '';
    stateFilter = '';
    valueFilter = '';




    constructor(
        injector: Injector,
        private _postalCodesServiceProxy: PostalCodesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getPostalCodes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show()

        this._postalCodesServiceProxy.getAll(
            this.filterText,
            this.countryFilter,
            this.cityFilter,
            this.stateFilter,
            this.valueFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createPostalCode(): void {
        this.createOrEditPostalCodeModal.show();
    }

    deletePostalCode(postalCode: PostalCodeDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._postalCodesServiceProxy.delete(postalCode.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._postalCodesServiceProxy.getPostalCodesToExcel(
        this.filterText,
            this.countryFilter,
            this.cityFilter,
            this.stateFilter,
            this.valueFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
