import {
    Component,
    Injector,
    OnInit,
    ViewChild,
    ViewEncapsulation,
    Input,
    OnDestroy,
    ElementRef,
    Renderer,
    QueryList,
    ViewChildren,
    ViewContainerRef,
    Output,
    EventEmitter,
    ChangeDetectionStrategy,
    ChangeDetectorRef
} from '@angular/core';
import {
    ActivatedRoute,
    Router
} from '@angular/router';
import {
    AppConsts
} from '@shared/AppConsts';
import {
    appModuleAnimation
} from '@shared/animations/routerTransition';
import {
    AppComponentBase
} from '@shared/common/app-component-base';
import {
    RouteListDto,
    RouteServiceProxy,
    ControllerRouteServiceProxy,
    ControllerKpiCountServiceProxy,
    UpdateRouteInput,
    UpdateCompleteRouteInput,
    VehicleLogServiceProxy,
    ShippingPackageServiceProxy,
    RealTimeEtaServiceProxy,
    GetControllerRouteStatusInputDto,
    UpdateLockedRouteInput,
    ShowBlindCountCodeInput,
    UpdateDepartureRouteInput,
    ControllerRouteDetailServiceProxy,
    DriverListInput,
    NameValueDto,
    VehicleInput,
    UpdateRouteByStopInput,
    ControllerRouteListDto,
    UpdateRouteId,
    ContainersServiceProxy,
    DriverClientServiceProxy,
    LoginUnlockByIdInputDto,
    ControllerRouteTypeListGroupDto,
    LocationServiceProxy,
    LogOffClockOutInput,
    EmployeesServiceProxy,
    EmployeeTimeSheetServiceProxy

} from '@shared/service-proxies/service-proxies';

import * as moment from 'moment-timezone';
import {
    LazyLoadEvent
} from 'primeng/components/common/lazyloadevent';
import {
    Paginator
} from 'primeng/components/paginator/paginator';
import {
    Table
} from 'primeng/components/table/table';
import {
    HttpClient
} from '@angular/common/http';
import {
    finalize
} from 'rxjs/operators';
import {
    Subscription
} from 'rxjs';
import {
    CheckBoxComponent
} from '@app/sprintship/core-components/checkbox/checkbox.component';
import {
    StatusComponent
} from '@app/sprintship/controller/status/status.component';

import {
    ControllerRoutesService
} from '@app/sprintship/controller/routes/croutes.service';
import * as _ from 'lodash';
import * as status_icon from '@app/sprintship/controller/status/status_icon.json';

import {
    KPIBoxComponent
} from '@app/sprintship/controller/kpi-box/kpi-box.component';
import {
    TimerService
} from '@app/sprintship/core-components/timer/timer.service';
import {
    RouteDetailModal
} from '@app/sprintship/controller/route-detail/route-detail-modal.component';
import { DeleteRoutesComponent } from './delete-routes.component';

import {
    MarkAsCompleteModalComponent
} from '../routes/mark-as-complete-modal.component';
import { analyzeAndValidateNgModules } from '@angular/compiler';
import { CommonLookupModalComponent } from '@app/shared/common/lookup/common-lookup-modal.component';
import { AssignStopModalComponent } from '@app/shared/common/lookup/assignStop-modal.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import {
    FreezeModalComponent
} from '@app/sprintship/controller/modals/freeze-modal.component';

import {
    UnfreezeModalComponent
} from '@app/sprintship/controller/modals/unfreeze-modal.component';
import { TabsetComponent } from 'ngx-bootstrap';
declare var $: any;
declare var jquery: any;
@Component({
    selector: 'dispatch-group-details',
    templateUrl: 'dispatch-details.group.component.html',
    encapsulation: ViewEncapsulation.None,
    styleUrls: ['./dispatch-details.group.component.less', './dispatch-details.general.component.less'],
    animations: [appModuleAnimation()],
    providers: [VehicleLogServiceProxy, RealTimeEtaServiceProxy,EmployeeTimeSheetServiceProxy],
    changeDetection: ChangeDetectionStrategy.OnPush,
})

export class DispatchDetailsGroupComponents extends AppComponentBase implements OnInit {
    @ViewChild('stopList', { static: true }) stopList: AssignStopModalComponent;
    @ViewChild('driversList', { static: true }) driversList: CommonLookupModalComponent;
    @ViewChild('deleteRoutes', { static: false }) deleteRoutes: DeleteRoutesComponent;
    @ViewChild('dataTable', {
        static: true
    }) dataTable: Table;
    @ViewChild('paginator', {
        static: true
    }) paginator: Paginator;
    @ViewChildren('CheckBoxComponent') chkBoxList: QueryList<any>;
    @ViewChildren(KPIBoxComponent) tKpiBoxComponents: KPIBoxComponent;
    @ViewChild(StatusComponent, {
        static: false
    }) statusDiv: StatusComponent;
    @ViewChild('routeDetailModal', {
        static: true
    }) routeDetailModal: RouteDetailModal;
    @ViewChild('markAsCompleteModalComponent', {
        static: true
    }) markAsCompleteModalComponent: MarkAsCompleteModalComponent;
    @ViewChild('freezeModal', {
        static: true
    }) freezeModal: FreezeModalComponent;

    @ViewChild('unfreezeModal', {
        static: true
    }) unfreezeModal: UnfreezeModalComponent;


    /**** ELEMENTS ****/
    @ViewChild('dataTable', {
        read: ElementRef,
        static: true
    }) dTable: ElementRef;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Routes.Route';
    entityHistoryEnabled = false;
    @ViewChild('locationTab', {
        static: true
    }) staticTabs: TabsetComponent;
    @Input() filters: {
        locationFilter: number;
        locationIds: number[];
        statusFilter: number;
        typeFilter: number;
        startDate: Date;
        endDate: Date;
        datePicked: string;
        filterTxt: string,
        routeId: string,
        routeName: string,
        vehicleName: string,
        driverName: string,
        completed: boolean,
        orderType: Array<string>,
        osName: string,
        routeIds: Array<any>,
        managerIds: Array<any>,
        routeTypeIds: Array<any>,
        isLineHaul: boolean
    } = <any>{};
    device_type: string;
    routeIds: any;
    routeIdsStops: GetControllerRouteStatusInputDto = new GetControllerRouteStatusInputDto();
    attr: any;
    respkey: any;
    saving = false;
    completion: any;
    route: UpdateRouteInput = new UpdateRouteInput();
    updateCompletedInput = new UpdateCompleteRouteInput();
    updateDepartureRouteInput = new UpdateDepartureRouteInput();
    updateLockedInput = new UpdateLockedRouteInput();
    showBlindCountCodes = new ShowBlindCountCodeInput()
    totalMileage: number;
    given: number;
    convert: any;
    hours: number;
    etaHours: any;
    minutes: number;
    routeIdStatus: any;
    etaMinutes: any;
    routeIdsForEta: any;
    routeIdsToLab: any;
    routeIdsToClinics: any;
    totalGroup: number;
    key2: any;
    driverStatusDisplay: boolean;
    packages: Array<{
        id: number,
        name: string,
        color: string
    }> = new Array();
    containers: Array<{
        id: number,
        name: string,
        color: string
    }> = new Array();
    filterFlyout: {};
    filterFlyoutHolder = [];
    finalFilterResult = [];
    routeGroupDetails = [];
    sprintIds: Array<number> = new Array<number>();
    sprintIdsList = [];
    selectAll: boolean = false;
    checkboxes_: boolean = false;
    @Input('search_string') search: string;
    @Input('edition') edition: string;
    @Input('firstload') firstload: boolean;
    subscription: Subscription;
    kpiClick: boolean;
    routeTypeIds: any;
    managerIds: any;
    groupGridBy: any;
    totalNoStatus: number;
    private cBox: CheckBoxComponent;
    noResultFound: boolean = false;
    selectedRouteId: any;
    localRouteIds = [];
    routeBuilder: boolean = false;
    routeId: number = null;
    lateRouteIds: any;
    routeEtaList: any;
    etaTime: any;
    hideEmptyRoute: any;
    lineHaulIds = [];
    driverThreshold: any;
    completedRoutes: number = 0;
    locationTabs: any;
    isMultipleLocation: any;
    isShowAllLocation:boolean = false;
    showCompletedRoute: boolean;
    constructor(
        injector: Injector,
        private _routeService: RouteServiceProxy,
        private _controllerRouteService: ControllerRouteServiceProxy,
        private _cRoutesService: ControllerRoutesService,
        private _cKpiService: ControllerKpiCountServiceProxy,
        private _vehicleLogsService: VehicleLogServiceProxy,
        private _controllerRoutePackages: ShippingPackageServiceProxy,
        private _timerService: TimerService,
        private router: Router,
        private renderer: Renderer,
        private elem: ElementRef,
        private _realTimeEtaService: RealTimeEtaServiceProxy,
        private viewContainerRef: ViewContainerRef,
        private _controllerRouteDetails: ControllerRouteDetailServiceProxy,
        private ref: ChangeDetectorRef,
        private _controllerRouteContainers: ContainersServiceProxy,
        private _driverClientService: DriverClientServiceProxy,
        private _locationServiceProxy: LocationServiceProxy,
        private _employeeTimeSheetServiceProxy: EmployeeTimeSheetServiceProxy
    ) {
        super(injector);
        setInterval(() => {
            this.ref.markForCheck();
        }, 1000);
        this.kpiClick = false;
    }

    ngOnInit(): void {
        this.routeBuilder = abp.features.isEnabled('App.RouteBuilder');
        this.setIsEntityHistoryEnabled();
        this.sprintIdsList = [];
        localStorage.setItem('routeIds', JSON.stringify(this.sprintIdsList));
        this.groupGridBy = localStorage.getItem('groupGridBy');
        this.isShowAllLocation = localStorage.getItem('showAllLocation')==null ? false : (localStorage.getItem('showAllLocation')=="true");
        this.driverStatusDisplay = false;
        //assign driver
        this.driversList.configure({
            title: this.l('Assign Driver'),
            type: 'assignDriver',
            dataSource: (skipCount: number, maxResultCount: number, filter: string, tenantId?: number, locationId?: number) => {
                let input = new DriverListInput();
                input.filter = filter;
                input.maxResultCount = maxResultCount;
                input.skipCount = skipCount;
                input.tenantId = tenantId;
                input.locationId = parseInt(localStorage.getItem('operatingLocationId'));
                return this._controllerRouteDetails.assignDriversList(input);
            }
        });
        // assign stop
        this.stopList.configure({
            title: this.l('Select Delivery Address'),
            dataSource: (skipCount: number, maxResultCount: number, filter: string, tenantId?: number, locationId?: number) => {
                return this._controllerRouteDetails.getAllStopsList(tenantId, localStorage.getItem('operatingLocationId').split(',').map(Number), maxResultCount, skipCount, filter);
            }
        });
        this.packages = this.getParentComponent().packages;
        this.containers = this.getParentComponent().containers;
        this.hideEmptyRoute = localStorage.getItem("hideEmptyRoute") == null ? false : JSON.parse(localStorage.getItem("hideEmptyRoute"));
        this.showCompletedRoute = localStorage.getItem("showCompleted") == null ? false : JSON.parse(localStorage.getItem("showCompleted"));
    }
    setFilter(value: string) {
        this.search = value;
    }
    populateTable(event?: LazyLoadEvent): void {
        this.spinnerService.show();
        let timeId = localStorage.getItem('timeZoneId');
        this.routeTypeIds = JSON.parse(localStorage.getItem('selectedRouteTypeIds')) == null ? undefined : JSON.parse(localStorage.getItem('selectedRouteTypeIds'));
        this.managerIds = JSON.parse(localStorage.getItem('selectedManagerIds')) == null ? undefined : JSON.parse(localStorage.getItem('selectedManagerIds'));
        this.driverThreshold = localStorage.getItem('THRESHOLD') == null ? undefined : this.isNumeric(localStorage.getItem('THRESHOLD')) ? parseInt(localStorage.getItem('THRESHOLD')) : undefined;
        this.filters.locationIds = localStorage.getItem('operatingLocationId') == null ? undefined : localStorage.getItem('operatingLocationId').split(',').map(Number);
        this.showCompletedRoute = localStorage.getItem("showCompleted") == null ? false : JSON.parse(localStorage.getItem("showCompleted"));
        var that = this;
        that.routeIds = [];
        that.lineHaulIds = [];
        this.filterFlyoutHolder = [];
        this.finalFilterResult = [];
        this.completedRoutes = 0;
        this.isShowAllLocation = localStorage.getItem('showAllLocation')==null ? false : (localStorage.getItem('showAllLocation')=="true");
        if (this.filters.locationFilter !== undefined) {
            if (this.primengTableHelper.shouldResetPaging(event)) {
                if(this.paginator!=undefined){
                    this.paginator.changePage(0);
                    return;
                }
            }
            this.hideEmptyRoute = localStorage.getItem("hideEmptyRoute") == null ? false : JSON.parse(localStorage.getItem("hideEmptyRoute"));
            console.log(this.kpiClick);
            if (this.kpiClick == false && this.edition === 'LABORATORY') {
                this._cKpiService.getKPICountLab(undefined, this.filters.locationFilter, undefined, this.filters.routeId, this.filters.vehicleName, this.filters.routeName, this.filters.driverName,
                    moment.tz(localStorage.getItem('loadDate'), timeId),
                    moment.tz(localStorage.getItem('loadDate'), timeId).endOf('day'), this.filters.filterTxt, undefined, this.showCompletedRoute, undefined, this.filters.orderType, this.edition, this.routeTypeIds,
                    this.managerIds, undefined, this.filters.locationIds, false, undefined, localStorage.getItem('loadDate'),  this.filters.isLineHaul, this.isShowAllLocation, undefined, this.paginator==undefined ? 1000 : this.primengTableHelper.getMaxResultCount(this.paginator, event), this.paginator==undefined ? 0 : this.primengTableHelper.getSkipCount(this.paginator, event))
                    .subscribe(resultCount => {
                        that._cRoutesService.setStatistics({
                            'total_stops': resultCount.totalStops,
                            'total_callins': resultCount.totalWillCalls,
                            'total_routes': resultCount.totalRoutes,
                            'completed_stops': resultCount.totalCompletedStops,
                            'completed_callins': resultCount.totalCompletedWillCalls,
                            'total_completed_routes': resultCount.totalCompletedRoutes,
                            'total_unassigned_routes': resultCount.totalUnassignedRoutes,
                            'total_unassigned_stops': resultCount.totalUnassignedStops,
                            'total_not_checkin': resultCount.totalNotCheckedIn,
                            'total_twenty_minutes': resultCount.kpiCountLates[0].total20Minutes,
                            'total_thirty_minutes': resultCount.kpiCountLates[0].total30Minutes,
                            'unassigned_routes_ids': resultCount.unassignedRoutesIds,
                            'not_check_in_routes_ids': resultCount.notCheckedInRouteIds,
                            'completed_route_ids': resultCount.completedRouteIds,
                            'late20_route_ids': resultCount.kpiCountLates[0].total20MinutesList,
                            'late30_route_ids': resultCount.kpiCountLates[0].total30MinutesList,
                            'unassigned_willcall_ids': resultCount.unassignedWillCallIds,
                            'completed_stop_ids': resultCount.completedStopsIds,
                            'completed_willcalls_ids': resultCount.completedWillCallIds,
                            'incident': resultCount.incidentIds,
                            'total_unassigned_willcalls': resultCount.totalUnassignedWillCalls,
                            'total_locked_pickup': resultCount.totalLockPickedUp,
                            'total_missed_window': resultCount.totalMissedWindow,
                            'locked_pickup_ids': resultCount.lockPickedUpIds,
                            'missed_window_ids': resultCount.missedWindowIds,
                            'total_locked_clinics': resultCount.totalLockClinics
                        });
                        if (resultCount != null) {
                            // this.lateRouteIds=resultCount.lateRouteIds;
                            this.etaTime = resultCount.routeDestination;
                        }
                    });
            }
            if (!isNaN(Date.parse(localStorage.getItem('loadDate')))) {
                if (localStorage.getItem('loadDate').search('-') > 0) {
                    var str = localStorage.getItem('loadDate').split('-');
                    var str1 = str[1] + '/' + str[2] + '/' + str[0];
                    localStorage.setItem('loadDate', str1);
                }
            }

            this._controllerRouteService.getDispatchGroupList(undefined, this.filters.locationFilter, undefined, this.filters.routeId, this.filters.vehicleName, this.filters.routeName, this.filters.driverName,
                moment.tz(localStorage.getItem('loadDate'), timeId), moment.tz(localStorage.getItem('loadDate'), timeId).endOf('day'),
                this.filters.filterTxt, (this.filters.osName == "null") ? undefined : this.filters.osName, this.showCompletedRoute, this.filters.routeIds, this.filters.orderType, this.edition, this.routeTypeIds,
                this.managerIds, undefined, this.filters.locationIds, this.hideEmptyRoute, this.driverThreshold, localStorage.getItem('loadDate'), this.filters.isLineHaul, this.isShowAllLocation, undefined, this.paginator==undefined ? 1000 : this.primengTableHelper.getMaxResultCount(this.paginator, event), this.paginator==undefined ? 0 : this.primengTableHelper.getSkipCount(this.paginator, event))
                .subscribe(result => {
                    var noresult = result == null || result.length == 0 ? false : result[0].routeListDetails[0].routeNull;
                    if (noresult) {
                        var result2 = [];
                        this.primengTableHelper.totalRecordsCount = result2.length;
                        this.primengTableHelper.records = result2;
                        this.noResultFound = true;
                        this.getParentComponent().firstload = false;
                        // that.getParentComponent().disableAll();
                        this.spinnerService.hide();
                    } else {
                        if (result == null) {
                            this.noResultFound = false;
                            this.getParentComponent().lastRefresh = sessionStorage.getItem('last-refresh');
                            var result2 = [];
                            this.primengTableHelper.totalRecordsCount = result2.length;
                            this.primengTableHelper.records = result2;
                            this.getParentComponent().firstload = false;
                            that.getParentComponent().disableAll();
                            this.spinnerService.hide();
                        }
                        else if (result.length > 0) {
                            this.noResultFound = false;
                            that.getParentComponent().enableAll();
                            this.primengTableHelper.records = result;
                            sessionStorage.setItem('last-refresh', result[0].routeListDetails[0].currentDateTime);
                            this.getParentComponent().lastRefresh = sessionStorage.getItem('last-refresh');
                            var increment = 1;
                            var temp = "";
                            for (var key in result) {
                                for (var key2 in result[key].routeListDetails) {
                                    that.routeIds.push(result[key].routeListDetails[key2].routeId);
                                    let fullName = result[key].routeListDetails[key2].fullName ? "/" + result[key].routeListDetails[key2].fullName : "";
                                    this.filterFlyout = {
                                        'routeId': result[key].routeListDetails[key2].routeId,
                                        'searchTerm': result[key].routeListDetails[key2].routeTemplateName + fullName,
                                    }
                                    if (result[key].routeListDetails[key2].completion != null) {
                                        this.completedRoutes += 1;
                                    }
                                    this.filterFlyoutHolder.push(this.filterFlyout);

                                    if (this.filters.isLineHaul) {
                                        that.lineHaulIds.push(result[key].routeListDetails[key2].routeId);
                                    } else {
                                        if (result[key].routeListDetails[key2].parentRoute != null) {
                                            that.lineHaulIds.push(result[key].routeListDetails[key2].routeId);
                                        }
                                    }
                                    
                                    // if (result[key].loadLimit != null) {
                                    //     if (temp != "" && temp != result[key].routeTemplateName) {
                                    //         increment = 1;
                                    //     }
                                    //     if (result[key]['routeTemplateName'] == result[key].routeTemplateName) {
                                    //         temp = result[key].routeTemplateName;
                                    //         result[key]['routeTemplateName'] = result[key].routeTemplateName + "-" + increment;
                                    //     }
                                    //     increment++;
                                    // }   
                                }
                            }
                            this.getParentComponent().getRouteAndDriver(this.filterFlyoutHolder);
                            this.routeIdsStops.routeIds = that.routeIds;
                            this.routeIdsStops.editions = that.edition;
                            that._controllerRouteService.stopsStats(that.routeIdsStops)
                                .subscribe(resultStatus => {
                                    var totalStandardPackages = 0, totalStandardDeliveredPackages = 0;
                                    for (var data in this.primengTableHelper.records) {
                                        for (var data2 in this.primengTableHelper.records[data].routeListDetails) {
                                            for (var data1 in resultStatus) {
                                                if (this.primengTableHelper.records[data].routeListDetails[data2].routeId == resultStatus[data1].routeId) {
                                                    this.primengTableHelper.records[data].routeListDetails[data2].statusDescription = resultStatus[data1].statusDescription;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].statusIcon = resultStatus[data1].icon;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].statusColor = resultStatus[data1].color;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].totalStops = resultStatus[data1].totalPackages;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].totalPickedUp = resultStatus[data1].totalPickedUp;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].totalCallIn = resultStatus[data1].totalCallIn;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].totalCallInPickedUp = resultStatus[data1].totalCallInPickedUp;
                                                    this.primengTableHelper.records[data].routeListDetails[data2].completePercentage = resultStatus[data1].completeRate;
                                                    // this.primengTableHelper.records[data].routeListDetails[data2].eta = resultStatus[data1].eta;
                                                    break;
                                                }
                                                totalStandardDeliveredPackages = totalStandardPackages + resultStatus[data1].totalPickedUp;
                                            }
                                        }
                                    }
                                    this.highlightLateRoutes(this.lateRouteIds);
                                    // if (this.edition === 'STANDARD') {
                                    //     that._cRoutesService.setStatistics({
                                    //         'total_stops': totalStandardPackages,
                                    //         'total_delivered': totalStandardDeliveredPackages
                                    //     });
                                    // }
                                });
                            that._controllerRouteService.getProximityCodes(that.routeIds, that.edition, undefined)
                                .subscribe(resultProximity => {
                                    for (var data in this.primengTableHelper.records) {
                                        for (var data2 in this.primengTableHelper.records[data].routeListDetails) {
                                            for (var data1 in resultProximity.proximity) {
                                                if (this.primengTableHelper.records[data].routeListDetails[data2].routeId == resultProximity.proximity[data1].routeId) {
                                                    //this.primengTableHelper.records[data].routeListDetails[data2].proximity = resultProximity[data1].proximityCode;
                                                    if(resultProximity.proximity[data1].proximityCode.includes("Pickup")){
                                                        if(JSON.parse(resultProximity.proximity[data1].proximityCode).Delivery != null){
                                                            this.primengTableHelper.records[data].routeListDetails[data2].proximity = JSON.parse(resultProximity.proximity[data1].proximityCode).Delivery;
                                                        } else {
                                                            this.primengTableHelper.records[data].routeListDetails[data2].proximity = JSON.parse(resultProximity.proximity[data1].proximityCode).Pickup;
                                                        }
                                                    } else {
                                                        this.primengTableHelper.records[data].routeListDetails[data2].proximity = resultProximity.proximity[data1].proximityCode;
                                                    }
                                                    break;
                                                }
                                            }
                                            console.log(this.primengTableHelper.records[data].routeListDetails[data2].routeId)
                                            for (var data1 in resultProximity.zeroCountCode) {
                                                if (this.primengTableHelper.records[data].routeListDetails[data2].routeId == resultProximity.zeroCountCode[data1].routeId) {
                                                    console.log(resultProximity.zeroCountCode[data1].zeroCountCode);
                                                    this.primengTableHelper.records[data].routeListDetails[data2].zeroCountCode = resultProximity.zeroCountCode[data1].zeroCountCode;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                });
                            // that._vehicleLogsService.getTotalMileage(that.routeIds).subscribe(result => {
                            //     for (var data in this.primengTableHelper.records) {
                            //         for (var data2 in this.primengTableHelper.records[data].routeListDetails) {
                            //             for (var data1 in result) {
                            //                 if (this.primengTableHelper.records[data].routeListDetails[data2].routeId == result[data1].routeId) {
                            //                     this.primengTableHelper.records[data].routeListDetails[data2].totalMileage =
                            //                         result[data1].totalMileage;
                            //                     break;
                            //                 }
                            //             }
                            //         }
                            //     }
                            // });
                            if (this.edition === 'LABORATORY') {
                                that.getParentComponent().enableAll();
                                this.primengTableHelper.totalRecordsCount = that.routeIds.length - (that.routeIds.length == this.completedRoutes ? 0 : this.completedRoutes);
                                that._controllerRouteService.getCountRouteDetails(that.routeIds, that.edition, undefined)
                                    .subscribe(resultCount => {
                                        this.primengTableHelper.records.forEach(val => { val.countPerPackage = [] });
                                        for (var data in this.primengTableHelper.records) {
                                            this.primengTableHelper.records[data].routeListDetails.forEach(val => { val.countPerPackage = [] });
                                            for (var data1 in this.primengTableHelper.records[data].routeListDetails) {
                                                var sum = 0, arrayHolder = [];
                                                for (var data2 in resultCount) {
                                                    if (this.primengTableHelper.records[data].routeListDetails[data1].routeId == resultCount[data2].routeId) {
                                                        this.primengTableHelper.records[data].routeListDetails[data1].countPerPackage.push(resultCount[data2]);
                                                        arrayHolder.push(resultCount[data2].totalPackages)
                                                    }
                                                }
                                                sum = arrayHolder.reduce((acc, cur) => acc + cur, 0);
                                                this.primengTableHelper.records[data].routeListDetails[data1].totalCountPackages = sum;
                                            }
                                        }
                                    });

                                if (this.lineHaulIds.length > 0) {
                                    that._controllerRouteService.getCountLineHaulRouteDetails(this.lineHaulIds, that.edition, undefined)
                                        .subscribe(resultCount => {
                                            this.primengTableHelper.records.forEach(val => { val.countLineHaulPerPackage = [] });
                                            for (var data in this.primengTableHelper.records) {
                                                this.primengTableHelper.records[data].routeListDetails.forEach(val => { val.countLineHaulPerPackage = [] });
                                                for (var data1 in this.primengTableHelper.records[data].routeListDetails) {
                                                    var sum = 0, arrayHolder = [];
                                                    for (var data2 in resultCount) {
                                                        if (this.primengTableHelper.records[data].routeListDetails[data1].routeId == resultCount[data2].routeId) {
                                                            this.primengTableHelper.records[data].routeListDetails[data1].countLineHaulPerPackage.push(resultCount[data2]);
                                                            arrayHolder.push(resultCount[data2].quantity);
                                                        }
                                                    }
                                                    sum = arrayHolder.reduce((acc, cur) => acc + cur, 0);

                                                    this.primengTableHelper.records[data].routeListDetails[data1].totalLineHaulPickedupContainers = sum;
                                                }
                                            }
                                        });
                                }

                            }

                        } else {
                        }
                        this.spinnerService.hide();
                    }
                }); // END FULL LIST
        }
    }
    isNumeric = (val: string): boolean => {
        return !isNaN(Number(val));
    }
    tickBox() {
        this.cBox.tickBox();
    }
    filterArray(array: any, key: any, value: any) {
        var i, arrList = [];
        for (i in array) {

            if (array[i][key] == value) {
                arrList.push(array[i]);
            }
        }
        return arrList;
    }
    filterArrayKeys(array: any, key: any, value: any) {
        var i, keys = 0;
        for (i in array) {
            if (array[i][key] == value) {
                keys = i;
            }
        }
        return keys;
    }
    openMap(userId, routeId) {
        // window.open('/app/sprintship/locator/'+routeId+'/'+this.filters);
        this.router.navigate([]).then(result => {
            window.open('/app/sprintship/locator/' + userId + '/' + routeId, '_blank');
        });
    }

    navigateDetail(id) {
        this.router.navigate(['/app/sprintship/controller-routes-details/', id]);
    }
    updateRoute() {
        this._routeService.updateRoute(this.route)
            .pipe(finalize(() => this.saving = false))
            .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
            });
    }

    freezeRoute(routeId, routeCode): void {
        this.freezeModal.show(routeId, routeCode);
    }

    unfreezeRoute(routeId, routeCode): void {
        this.unfreezeModal.show(routeId, routeCode);
    }
    setDepature(record) {
        this.saving = true;
        if (!record.locked) {
            this.updateDepartureRouteInput.id = record.routeId;
            if (record.departure) {
                this.updateDepartureRouteInput.departure = false;
                this._controllerRouteService.updateDepartureRoute(this.updateDepartureRouteInput)
                    .pipe(finalize(() => {
                        this.saving = false;
                    }))
                    .subscribe(() => {
                        this.populateTable();
                        this.notify.info(this.l('Saved Successfully'));
                        record.departure = false;
                    });
            } else {
                this.updateDepartureRouteInput.departure = true;
                this._controllerRouteService.updateDepartureRoute(this.updateDepartureRouteInput)
                    .pipe(finalize(() => {
                        this.saving = false;
                    }))
                    .subscribe(() => {
                        this.populateTable();
                        this.notify.info(this.l('Saved Successfully'));
                        record.departure = true;

                    });
            }
        } else {
            this.notify.info(this.l('Unlocked Loadlist First'));
        }
    }
    setCompleted(record) {
        this.saving = true;
        if (!record.locked) {
            this.updateCompletedInput.id = record.routeId;
            if (record.completion) {
                this.updateCompletedInput.completion = false;
                this._controllerRouteService.updateCompletedRoute(this.updateCompletedInput)
                    .pipe(finalize(() => {
                        this.saving = false;
                    }))
                    .subscribe(() => {
                        // this.populateTable();
                        this.getParentComponent().filterRoutes();
                        this.notify.info(this.l('Saved Successfully'));
                        record.completion = false;

                    });
            } else {
                this.updateCompletedInput.completion = true;
                this._controllerRouteService.updateCompletedRoute(this.updateCompletedInput)
                    .pipe(finalize(() => {
                        this.saving = false;
                    }))
                    .subscribe(() => {
                        // this.populateTable();
                        this.getParentComponent().filterRoutes();
                        this.notify.info(this.l('Saved Successfully'));
                        record.completion = true;

                    });
            }
        } else {
            this.notify.info(this.l('Unlocked Loadlist First'));
        }
    }
    setLocked(record) {
        this.updateLockedInput.routeId = record.routeId;
        if (record.locked == true) {
            this.updateLockedInput.locked = false;
            this._controllerRouteService.updateLockedRoute(this.updateLockedInput)
                .pipe(finalize(() => {
                    this.saving = false;
                }))
                .subscribe(() => {
                    this.notify.info(this.l('Route Unlocked Successfully'));
                });
            record.locked = false;
        } else {
            this.updateLockedInput.locked = true;
            this._controllerRouteService.updateLockedRoute(this.updateLockedInput)
                .pipe(finalize(() => {
                    this.saving = false;
                }))
                .subscribe(() => {
                    this.notify.info(this.l('Route Locked Successfully'));
                });
            record.locked = true;
        }
    }
    getParentComponent() {
        return this.viewContainerRef['_data'].componentView.component.viewContainerRef['_view'].component
    }
    selectRoute(id: number) {
        if (this.sprintIdsList.indexOf(id) == -1) {
            this.sprintIdsList.push(id);
        } else {
            this.sprintIdsList.splice(this.sprintIdsList.indexOf(id), 1);
        }
        localStorage.setItem('routeIds', JSON.stringify(this.sprintIdsList));
        if (this.sprintIdsList.length == this.routeIds.length) {
            this.selectAll = true;
        } else {
            this.selectAll = false;
        }
    }

    CheckAllOptions() {
        if (!this.selectAll) {
            for (var data in this.primengTableHelper.records) {
                this.primengTableHelper.records[data].routeListDetails.forEach(val => { val.selectedRoute = false });
            }
            this.sprintIdsList = [];
            localStorage.setItem('routeIds', JSON.stringify(this.sprintIdsList));
        } else
            for (var data in this.primengTableHelper.records) {
                this.primengTableHelper.records[data].routeListDetails.forEach(val => { val.selectedRoute = true; this.sprintIdsList.push(val.routeId) });
            }
        localStorage.setItem('routeIds', JSON.stringify(this.sprintIdsList));
    }

    showBlindCountCode(routeId) {
        this.showBlindCountCodes.routeId = routeId;
        this._controllerRouteService.showBlindcode(this.showBlindCountCodes).subscribe(result => {
            $("#showBlindCount" + routeId).show();
            $("#hideCount" + routeId).hide();
        });
    }
    disableAll() {
        this.getParentComponent().disableAll();
    }
    generateRoutes() {
        
        this.getParentComponent().generateRoute();

    }
    //asign driver
    assignDriverModal(record: any): void {
        this.driversList.tenantId = record.id;
        this.driversList.show();

        this.selectedRouteId = record.routeId;
    }

    updateRouteByVehicle(item: NameValueDto): void {
        this.spinnerService.show();
        let input = new VehicleInput();
        input.driverId = parseInt(item.value);
        input.routeId = this.selectedRouteId;
        this._controllerRouteDetails.updateRouteByVehicleId(input).subscribe(result => {
            $('.dispatch-refresh').trigger('click');
        });
    }

    printRouteSheet(id: number) {
        this.routeId = id;
        if (localStorage.getItem('routeIds') == null) {
            this.localRouteIds = [];
            this.localRouteIds.push(this.routeId)
            localStorage.setItem('routeIds', JSON.stringify(this.localRouteIds));
        } else {
            localStorage.removeItem('routeIds');
            this.localRouteIds = [];
            this.localRouteIds.push(this.routeId)
            localStorage.setItem('routeIds', JSON.stringify(this.localRouteIds));
        }
        if (this.edition === 'LABORATORY') {
            window.open('/app/sprintship/print-route-sheet', '_blank');
        } else {
            window.open('/app/sprintship/route-print', '_blank');
        }
    }
    removeRoutes(id: number, routeCode: string) {
        this.message.confirm(
            this.l('Are you sure you want to delete ' + routeCode + '?'),
            '',
            isConfirmed => {
                if (isConfirmed) {
                    let input: any;
                    input = { 'RouteId': id };
                    this._controllerRouteService.closedRoutes(input).pipe().subscribe(result => {
                        this.notify.info(this.l('Deleted Successfully'));
                        this.populateTable(null);
                    });
                }
            }
        );
    }

    markAsDeliveredModal(record: any): void {
        if(!record.completion){
            this.stopList.tenantId = record.id;
            this.selectedRouteId = record.routeId;
            this.stopList.show();
        }
        else{
            let input = new UpdateRouteByStopInput()
            input.contactId = undefined,
            input.routeId = record.routeId,
            input.routeComplete = true,
            input.loadDate = undefined
       
            this._controllerRouteDetails.updateRouteByStop(input).subscribe(result=>{
                this.notify.info(this.l('UnmarkDelivered'));
                $('.dispatch-refresh').trigger('click');
            });
        }
    }

    updateRouteByStop(item: NameValueDto): void {
        let timeId = localStorage.getItem('timeZoneId');
        let input = new UpdateRouteByStopInput();
        input.contactId = parseInt(item.value);
        input.routeId = this.selectedRouteId;
        input.loadDate = moment.utc(this.stopList.completionDate);
        this._controllerRouteDetails.updateRouteByStop(input).subscribe(result => {
            $('.dispatch-refresh').trigger('click');
        });
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(controller: ControllerRouteListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: controller.routeId.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: controller.routeTemplateName
        });
    }

    expandParentRoute(obj: string, hoIds: Array<number>, routeDetails: any, isParent: boolean, parentRoute: any, color: string) {
        if (hoIds.length > 0) {
            if ($('.tr-' + hoIds[0]).css('display') == 'none') {
                $('.fa-' + obj).html('<i class="fas fa-chevron-down"></i>');
                let level = 0
                if (parentRoute != null) {
                    level = routeDetails.find(o => o.routeId == obj).layer;
                }
                let childRoutes = [];
                hoIds.forEach(r => {
                    var child = routeDetails.find(o => o.routeId == r);
                    var childIndex = routeDetails.findIndex(o => o.routeId == r);
                    routeDetails.splice(childIndex, 1);
                    childRoutes.push(child);
                    $('.tr-' + r).removeClass('handoff-parent');
                    $('.tr-' + r).css('display', 'table-row');
                    $('.tr-' + r).css('background', color);
                    child.layer = level + 1;
                });
                var newParentIndex = routeDetails.findIndex(o => o.routeId == obj);
                Array.prototype.splice.apply(routeDetails, [newParentIndex + 1, 0].concat(childRoutes));
            } else {
                let childIndices = [];
                var parentIndex = routeDetails.findIndex(o => o.routeId == obj);
                var getRouteLayer = routeDetails.find(o => o.routeId == obj).layer;

                for (var i = parentIndex + 1; i <= routeDetails.length - 1; i++) {
                    var element = routeDetails[i];
                    if (element.layer <= getRouteLayer) {
                        break;
                    } else {
                        childIndices.push(routeDetails.findIndex(o => o.routeId == element.routeId));
                    }
                }
                var getChildRoutes = routeDetails.slice(parentIndex + 1, Math.max(...childIndices) + 1);
                getChildRoutes.forEach(r => {
                    if (r.isParentHandOff) {
                        $('.fa-' + r.routeId).html('<i class="fas fa-chevron-right"></i>');
                    }
                    $('.tr-' + r.routeId).css('display', 'none');
                });
                $('.fa-' + obj).html('<i class="fas fa-chevron-right"></i>');
            }
        }
    }

    openBuilder(id) {
        this.router.navigate([]).then(result => {
            window.open('/app/sprintship/route-builder/' + id, '_blank');
        });
    }
    isEmpty(element: HTMLElement): boolean {
        const nodes = element.childNodes;
        for (let i = 0; i < nodes.length; i++) {
            const node = nodes.item(i);
            if (node.nodeType !== 8 && nodes.item(i).textContent.trim().length !== 0) {
                return false;
            }
        }
        return true;
    }

    getPackages(): void {
        this._controllerRoutePackages.getShippingPackageListOptions(undefined, undefined, undefined, undefined, undefined, undefined)
            .subscribe(resultPackages => {
                resultPackages.forEach(element => {
                    if (element.name !== 'Default') {
                        if (element.isHiddenPackages === '0') {
                            this.packages.push(element);
                        }
                    }
                });
            });
    }

    getContainers(): void {
        this.containers.length = 0;
        this._controllerRouteContainers.getAll().subscribe(result => {
            result.items.forEach(element => {
                this.containers.push(element);
            });
        });
    }

    expandDetails(obj: string) {
        obj = obj == 'Line Haul' ? 'LineHaul' : obj;
        if ($('.tr-' + obj).css('display') == 'none') {
            $('.tr-' + obj).css('display', 'table-row');
            $('.span-' + obj).html(obj + ' Routes');
            $('.td-' + obj).html('<i class="fa fa-chevron-down"></i>');
        } else {
            $('.tr-' + obj).css('display', 'none');
            $('.td-' + obj).html('<i class="fa fa-chevron-right"></i>')
            $('.span-' + obj).html(obj + ' Routes');
        }
    }

    highlightLateRoutes(lateRoutes: any) {
        if (this.etaTime != undefined) {
            for (var data in this.primengTableHelper.records) {
                for (var data2 in this.primengTableHelper.records[data].routeListDetails) {
                    for (var data1 in this.etaTime) {
                        if (this.primengTableHelper.records[data].routeListDetails[data2].routeId == this.etaTime[data1].routeId) {
                            this.primengTableHelper.records[data].routeListDetails[data2].targetTimeString = this.etaTime[data1].eta;
                            this.primengTableHelper.records[data].routeListDetails[data2].targetTimezoneString = this.etaTime[data1].timezone;
                            break;
                        }
                    }
                }
                // if (this.edition == 'LABORATORY') {
                //     var sort = _.orderBy(this.primengTableHelper.records[data].routeListDetails, [function (obj) {
                //         if (obj.targetTimeString != null) {
                //             const date = new Date();
                //             const time = obj.targetTimeString;
                //             const yy = new Date(date).getFullYear();
                //             const mm = new Date(date).getMonth() + 1;
                //             const dd = new Date(date).getDate();
                //             var newDate = new Date(mm + '-' + dd + '-' + yy);
                //             newDate.setHours(time.split(' ')[1] == 'PM' && (time.split(' ')[0]).split(':')[0] < 12 ? (time.split(' ')[0]).split(':')[0] + 12 : (time.split(' ')[0]).split(':')[0]);
                //             newDate.setMinutes((time.split(' ')[0]).split(':')[1]);
                //             return newDate.getTime();
                //         }
                //     }], ['asc']);
                //     this.primengTableHelper.records[data].routeListDetails = sort;
                // }
            }
        }
    }

    getDefaultColor() {
        var color = "hsla(" + ~~(360 * Math.random()) + "," + "70%," + "80%, 1)";
        return color;
    }

    validateRoute(parentRoute: any, routeId: any, isParent: boolean, completion: any) {
        if (completion != null) {
            if (isParent && parentRoute == null) {
                if (this.filters.routeIds != null) {
                    if (this.filters.routeIds.length > 0) {
                        if (this.filters.routeIds.indexOf(routeId) != -1) {
                            return false;
                        } else {
                            return true;
                        }
                    }
                }
                return true;
            } else {
                if (this.filters.routeIds != null) {
                    if (this.filters.routeIds.length > 0) {
                        if (parentRoute != null) {
                            if (this.filters.routeIds.indexOf(parentRoute.routeId) != -1) {
                                //console.log(1);
                                document.getElementById("trRoutes"+parentRoute.routeId).classList.add('tr-LineHaulChildCompleted');
                                document.getElementById("trRoutes"+parentRoute.routeId).classList.remove('tr-LineHaulChild');
                                return true;
                            } else {
                                //console.log(2);
                                document.getElementById("trRoutes"+routeId).classList.add('tr-LineHaulChildCompleted');
                                document.getElementById("trRoutes"+routeId).classList.remove('tr-LineHaulChild');
                                return false;
                            }
                        } else {
                            return false;
                        }
                    }
                }
                return true;
            }
        }
        else {
            return false;
        }
    }

    setCompletion(data: any) {
        this.spinnerService.show();
        if (!data.locked) {

            if (data.completion == null) {
                let input = new UpdateRouteByStopInput()
                input.contactId = undefined;
                input.routeId = data.routeId;
                input.loadDate = moment.utc(new Date());

                this._controllerRouteDetails.updateRouteByStop(input).subscribe(result => {
                    this.notify.info(this.l('SavedSuccessfully'));
                    this.spinnerService.hide()
                    $('.dispatch-refresh').trigger('click');

                });
            }
            else {

                let input = new UpdateRouteByStopInput()
                input.contactId = undefined;
                input.routeId = data.routeId;
                input.routeComplete = true;
                input.loadDate = moment.utc(new Date());

                this._controllerRouteDetails.updateRouteByStop(input).subscribe(result => {
                    this.notify.info(this.l('SavedSuccessfully'));
                    this.spinnerService.hide();
                    $('.dispatch-refresh').trigger('click');

                });
            }

        } else {
            this.notify.info(this.l('Unlocked Loadlist First'));
            this.spinnerService.hide();
        }

    }

    // moveHandOffChildRoutes() {
    //     console.log(this.primengTableHelper.records);
    //     if (this.lineHaulIds.length > 0) {
    //         let childRoutes = [];
    //         var routes = this.primengTableHelper.records.find(c => c.routeType == 'Line Haul');
    //         if (routes != null) {
    //             this.primengTableHelper.records.forEach(item => {
    //                 if (item.routeType != 'Line Haul') {
    //                     if (item.routeListDetails.length > 0) {
    //                         item.routeListDetails.forEach(r => {
    //                             if (this.lineHaulIds.indexOf(r.routeId) != -1 && r.completion != null) {
    //                                 console.log(r.routeId);
    //                                 childRoutes.push(r);
    //                             }
    //                         });
    //                         if (this.filters.routeIds == undefined) {
    //                             item.routeListDetails = item.routeListDetails.filter(c => c.completion == null);
    //                         } else {
    //                             if (this.filters.routeIds.length == 0) {
    //                                 item.routeListDetails = item.routeListDetails.filter(c => c.completion == null);
    //                             } else {
    //                                 item.routeListDetails = item.routeListDetails.filter(c => c.parentRoute == null);
    //                             }
    //                         }
    //                     }
    //                     if (item.routeListDetails.length == 0) {
    //                      var records = this.primengTableHelper.records.filter(c => c.routeType != item.routeType);
    //                      this.primengTableHelper.records = records;
                            
    //                     }
    //                 }
    //             });
    //             var count = routes.routeListDetails.filter(c => c.completion == null);
    //             if (count.length == 0 && (this.filters.routeIds == undefined || this.filters.routeIds.length == 0)) {
    //                 let index = this.primengTableHelper.records.findIndex(c => c.routeType == 'Line Haul');
    //                 this.primengTableHelper.records.splice(index, 1);
    //             }
    //             Array.prototype.splice.apply(routes.routeListDetails, [routes.routeListDetails.length - 1, 0].concat(childRoutes));
    //         }
    //         //Array.prototype.splice.apply(routes.routeListDetails, [routes.routeListDetails.length - 1, 0].concat(childRoutes));
    //     }
    // }

    getContainersCount(childHandOffIds: any, routeDetails: any) {
        let child: Array<number> = childHandOffIds;
        let total = 0;

        child.forEach(ro => {
            var route = routeDetails.find(r => r.routeId == ro);
            if (route.countLineHaulPerPackage != undefined) {
                if (route.countLineHaulPerPackage.length > 0) {
                    total += parseInt(route.countLineHaulPerPackage[0].totalPickedupContainers > 0 ? route.countLineHaulPerPackage[0].totalPickedupContainers : route.countLineHaulPerPackage[0].totalWaitingPickupContainers);
                }
            }
        });
        return total == 0 ? '-' : total;
    }

    forceLogOff(driverId: number, routeId: number, routeName: any,) {
        routeName = routeName == undefined ? "" : this.edition === 'LABORATORY' ? routeName : routeId
        this.message.confirm(
            this.l('Are you sure to force log off driver of route ' + routeName + '?'),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {

                    if (driverId != null) {
                        let timeId = localStorage.getItem('timeZoneId');
                        this.spinnerService.show()
                        let dto: LogOffClockOutInput = new LogOffClockOutInput();
                        dto.userId = driverId
                        dto.timezone = timeId
                        this._employeeTimeSheetServiceProxy.forceLogOffAndClockOut(dto)
                            .subscribe(result => {
                                this.notify.info('Force Log Off Success');
                                this.spinnerService.hide();
                            })
                    }
                    else {
                        this.notify.warn("Force Log Off Fail");
                    }

                }
            }
        );
    }
    selectedTab(locationTab: any){
        if(this.filters.locationFilter != locationTab.id){
            this.filters.locationFilter = locationTab.id;
            let timeId = localStorage.getItem('timeZoneId');
            this._locationServiceProxy.getLocationForLock(this.filters.locationFilter, moment.tz(localStorage.getItem('loadDate'), timeId)).subscribe(result => {
                this.getParentComponent().lockOrUnlockLocation = result.lock;
                this.getParentComponent().lockOrUnlockLocationDetails = result;
            });
            this.kpiClick = false;
            localStorage.setItem('selectedTabLocation', locationTab.id);
            this.filters.routeIds = [];
            this.getParentComponent().filters.completed = false;
            $('.filter-right-routes-completed').removeClass('active');
            this.populateTable();
        }
    }
}