import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewChildren } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ControllerDriverListDto, ControllerDriverListServiceProxy, VehicleLogServiceProxy } from '@shared/service-proxies/service-proxies';

import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators'

@Component({
    templateUrl: './driver-details.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./driver-details.component.less'],
    providers: [ControllerDriverListServiceProxy,VehicleLogServiceProxy],

    styles: ['.table-title {margin: 0 !important;min-height: auto !important;}']
})

export class DriverDetailComponent extends AppComponentBase  implements OnInit{
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
  
    locationFilter : number;
    userId: number;
    driverName: string;
    routeIds: any;

	constructor(
        injector: Injector,
        private _driverDetailService: ControllerDriverListServiceProxy,
        private activatedRoute: ActivatedRoute,
        private _vehicleLogsService: VehicleLogServiceProxy,

    ) {
        super(injector);
    }

    ngOnInit(): void {
    
        this.activatedRoute.paramMap.subscribe(params => {
            this.locationFilter = +params.get("location");
            this.userId = +params.get("userid");
        });

        this._driverDetailService.getDriverEntity(
            undefined,
            undefined,
            undefined,
            undefined,
            this.userId,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).pipe(finalize(() => this.spinnerService.hide())
        ).subscribe(result => {
            this.driverName  = result.fullName
        });
    }
  
    getDriverDetails(event?: LazyLoadEvent): void
    {
        var key;
        this.routeIds = [];
       
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
    
        this.spinnerService.show();
       
        this._driverDetailService.getDriverDetails(
            undefined,
            undefined,
            undefined,
            undefined,
            this.userId,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())
        ).subscribe(result => {
            for (key in result.items)
            {
                this.routeIds.push(result.items[key].routeId);
            }

            // this._vehicleLogsService
            // .getTotalMileage(this.routeIds)
            // .subscribe(stopResult =>
            // {   
            //     if(stopResult !==null)
            //     {
            //         if(Object.keys(stopResult).length > 0)
            //         {
            //             for (key in stopResult)
            //             {
            //                 $('#miles' + stopResult[key].routeId).html(stopResult[key].totalMileage.toString());

            //                 var time = $('#tt'+ stopResult[key].routeId).html();
            //                 if(time != null)
            //                 {
            //                         var a = time.split(' Hours ');
            //                         var minutes = (+a[0]) * 60 + (+a[1]);
            //                         $('#tt'+ stopResult[key].routeId ).html( time + " Minutes")
            //                     $('#mph' + stopResult[key].routeId).html(((((stopResult[key].totalMileage/minutes)*60).toFixed(2).toString())));

            //                 }

            //             }
            //         }
            //     }
            // }); 
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this._driverDetailService
                .getStopCount(this.routeIds)
                .subscribe(stopResult =>
                {   
                
                    if(stopResult !== null)
                    {
                        if(Object.keys(stopResult).length > 0)
                        {
                            for (key in stopResult)
                            {   
                                for(var key2 in this.primengTableHelper.records){
                                    if(stopResult[key].routeId==this.primengTableHelper.records[key2].routeId){
                                        if(this.primengTableHelper.records[key2].hour!=0 ){
                                            this.primengTableHelper.records[key2].stopsPerHour = (stopResult[key].stops/this.primengTableHelper.records[key2].hour).toFixed(2);
                                        }else{
                                            if(this.primengTableHelper.records[key2].min!=0){
                                                this.primengTableHelper.records[key2].stopCount = stopResult[key].stops;
                                            }else{
                                                this.primengTableHelper.records[key2].stopCount = stopResult[key].stops
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                });
            
            this.spinnerService.hide();
        });
    } 

    
}