import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewChildren } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ControllerDriverListDto, ControllerDriverListServiceProxy} from '@shared/service-proxies/service-proxies';

import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators';

@Component({
	selector: 'driverTab',
    templateUrl: './driver-list.component.html',
    encapsulation: ViewEncapsulation.None,
    //styleUrls: ['./driver-list.component.less'],
    animations: [appModuleAnimation()],
    providers: [ControllerDriverListServiceProxy]
})

export class DriverListTabComponent extends AppComponentBase  implements OnInit{
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @Input() filters: {
        locationFilter : number;
        startDate: Date;
        endDate: Date;
        routeIds: Array<any>;
        completed: boolean;
        isLineHaul: boolean
    };
    txtFilter : any;
    filterId : any;
    driverIds: any;
    advancedFiltersAreShown = false;
    searchFilters: {
        route: string,
        vehicleName: string,
        driverName: string,
        showCompletion: boolean,
        routeIds: Array<number>,
        orderType: Array<string>,
        deviceType: string,
    } = <any>{};
	constructor(
        injector: Injector,
        private _driverListService: ControllerDriverListServiceProxy
        
    ) {
        super(injector);
    }

    ngOnInit(): void {
    
    }

    getFilters(){
        if (localStorage.getItem('searchFilter') == null) {
            this.searchFilters.route = '';
            this.searchFilters.driverName = '';
            this.searchFilters.showCompletion = false;
            this.searchFilters.vehicleName = '';
        }
        else {
            this.searchFilters = JSON.parse(localStorage.getItem('searchFilter'));
        }
    }

    getDriverList(event? : LazyLoadEvent): void
    {
        this.getFilters();
        var key;
        this.driverIds = [];

        let timeId = localStorage.getItem('timeZoneId');
        moment.tz.setDefault(timeId);
        
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        
        this.spinnerService.show();
       
        this._driverListService.getControllerDriverList(
            this.filters.locationFilter,
            undefined,
            moment.tz(localStorage.getItem('loadDate'), timeId),
            moment.tz(localStorage.getItem('loadDate'), timeId).endOf('day'),
            undefined,
            this.txtFilter,
            this.filterId,
            undefined,
            this.filters.completed,
            this.filters.routeIds,
            this.filters.isLineHaul,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)

        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
         
            for (key in result.items)
            {
                this.driverIds.push(result.items[key].userId);
            }
            
            this._driverListService
                .getControllerDriverRouteCount(this.driverIds)
                .subscribe(routeResult => 
                {
                    if(routeResult !== null)
                    {
                        if(Object.keys(routeResult).length > 0)
                        {
                            for (key in routeResult)
                            {   
                                $('#driver' + routeResult[key].driverId).html(routeResult[key].routeCount.toString());
                            }
                        } 
                    } 
                });
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    
}