import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewContainerRef } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ControllerRouteDetailServiceProxy, ControllerOrderPackagesListServiceProxy, OrderIncidentsServiceProxy, OrderTypesServiceProxy} from '@shared/service-proxies/service-proxies';
import { AppSessionService } from '@shared/common/session/app-session.service';
import { Table } from "primeng/table";
import { LazyLoadEvent } from "primeng/primeng";
import { Paginator } from 'primeng/components/paginator/paginator';
import { finalize } from 'rxjs/operators';
import * as _ from 'lodash';
import * as moment from 'moment';
declare var $: any;
declare var jquery:any;
@Component({
    selector:'order-incidents-group-tab',
    templateUrl: 'order-incident-list-group.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    providers: [OrderIncidentsServiceProxy]
})

export class ControllerOrderIncidentsGroupComponent extends AppComponentBase implements OnInit {

    routeId:number = null;
    formattedOrderPackagesDateTime: any
    formattedOrderPackagesPickupBy: any
    formattedOrderPackagesDeliveryBy: any
    filterText: any

    @ViewChild("dataTable", {static: true}) dataTable:Table
    @ViewChild("paginator", {static: true}) paginator: Paginator
    @Input() filters: {
        locationFilter: number;
        statusFilter: number;
        typeFilter: number;
        startDate: Date;
        endDate: Date;
        orderStatusCode: string,
        isLineHaul: boolean
    } = <any>{};
    filteredOrderType: any;
    orderTypeId: any;

    constructor(
        injector: Injector,
        private id:ActivatedRoute,
        private _controllerRouteDetails: ControllerRouteDetailServiceProxy,
        private _controllerOrderPackages: ControllerOrderPackagesListServiceProxy,
        private _appSessionService :AppSessionService,
        private _orderIncidentAppService: OrderIncidentsServiceProxy,
        private _orderTypeRepository: OrderTypesServiceProxy,
        private viewContainerRef: ViewContainerRef,  
    ) {
        super(injector);
    }

    ngOnInit(){
        $('#OrderTypeSelect6').select2();
        this.filteredOrderType = this.getParentComponent().filteredOrderType;
    }

    getParentComponent() {
        return this.viewContainerRef['_data'].componentView.component.viewContainerRef['_view'].component
    }

    getOrderIncident(event?: LazyLoadEvent){
        let date = moment(this.filters.endDate)
        let timeId = localStorage.getItem('timeZoneId');
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0); 
            return;
        }

        this.spinnerService.show();
        this.orderTypeId = Number((<HTMLInputElement>document.getElementById('OrderTypeSelect6')).value) == 0 ? undefined :
                            Number((<HTMLInputElement>document.getElementById('OrderTypeSelect6')).value);
        this._orderIncidentAppService.getAllGroupByOrderType(
            this.filterText,
            undefined,
            this.filters.locationFilter,
            moment.tz(localStorage.getItem('loadDate'), timeId),
            this.orderTypeId,
            this.filters.isLineHaul,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event))  
            .subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.totalCount;
                this.primengTableHelper.records = result.items;
                this.spinnerService.hide();
                this.filteredOrderType = this.getParentComponent().filteredOrderType;
            });
    }

    expandDetails(obj: string) {
        console.log(obj)
        if ($('.tr-' + obj).css('display') == 'none') {
            // $('.span-' + obj).html(obj);
            $('.tr-' + obj).css('display', 'table-row');
            $('.td-' + obj).html('<i class="fa fa-chevron-down"></i>');
            console.log("a");
        }
        else {
            // $('.span-' + obj).html(obj);
            $('.tr-' + obj).css('display', 'none');
            $('.td-' + obj).html('<i class="fa fa-chevron-right"></i>');
            console.log("b");
        }
    }

    // getOrderPackages(event?: LazyLoadEvent) : void {
    //     this.filters.startDate = new Date(this.filters.startDate.toString());
    //     this.filters.endDate = new Date(this.filters.endDate.toString());
    //     let startDate = moment(moment(this.filters.startDate).format('YYYY-MM-DD HH:mm:ss'));
    //     let endDate = moment(moment(this.filters.endDate).format('YYYY-MM-DD HH:mm:ss'));

    //     let startDateA = startDate['_i'].split(' ');
    //     startDateA = startDateA[0] + ' ' + '00:00:00';

    //     let endDateA = endDate['_i'].split(' ');
    //     endDateA = endDateA[0] + ' ' + '23:59:59';
        
    //     let locationFilter = this.filters.locationFilter;
    //     let startDate2 = moment.utc(moment(startDateA).toISOString());
    //     let endDate2 = moment.utc(moment(endDateA).toISOString());
    //     if (this.primengTableHelper.shouldResetPaging(event)) {
    //         this.paginator.changePage(0);
    //         return;
    //     }
    //     this.spinnerService.show();
    //     this._controllerOrderPackages
    //         .getControllerOrderPackagesList(
    //             undefined,
    //             this.filters.orderStatusCode,
    //             undefined,
    //             undefined,
    //             locationFilter,
    //             undefined,
    //             undefined,
    //             undefined,
    //             endDate2,
    //             this.primengTableHelper.getSorting(this.dataTable),
    //             this.primengTableHelper.getMaxResultCount(this.paginator, event),
    //             this.primengTableHelper.getSkipCount(this.paginator, event))
    //         .subscribe(result => {
    //             if (result.items.length != 0) {
    //                 this.formattedOrderPackagesDateTime = moment(result.items[0]['order']['orderStatus']['time']).format('MMMM Do YYYY, h:mm a')
    //                 this.formattedOrderPackagesPickupBy = moment(result.items[0]['order']['pickupDetail']['etaStart']).format('MMMM Do YYYY, h:mm a')
    //                 this.formattedOrderPackagesDeliveryBy = moment(result.items[0]['order']['deliveryDetail']['etaEnd']).format('MMMM Do YYYY, h:mm a')
    //             }
    //             this.primengTableHelper.totalRecordsCount = result.totalCount;
    //             this.primengTableHelper.records = result.items;
    //             this.spinnerService.hide();
    //         });
    // }

    // toggleMoreInfo(event){
    //     var element = event.currentTarget.nextSibling;
    //     var selector = document.querySelectorAll('.more-info-container.active');
    //     var hover = document.querySelectorAll('.more-btn.hover');

    //     if(element.classList.contains('active')){
    //         event.currentTarget.classList.remove('hover');
    //         element.classList.remove('active');
    //     } else {
    //         if(selector.length > 0){
    //             selector[0].classList.remove('active');
    //             hover[0].classList.remove('hover');
    //             // event.currentTarget.classList.remove('hover');
    //         } 
    //         element.classList.add('active');
    //         event.currentTarget.classList.add('hover');
    //     }

    // }

    onShown(){
        
    }


}