import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewChildren } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ControllerDriverListDto, ControllerVehicleListServiceProxy} from '@shared/service-proxies/service-proxies';

import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators';

@Component({
	selector: 'vehicleTab',
    templateUrl: './vehicle-list.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    providers: [ControllerVehicleListServiceProxy]
})

export class VehicleListTabComponent extends AppComponentBase  implements OnInit{
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @Input() filters: {
        locationFilter : number;
        startDate: Date;
        endDate: Date;
        routeIds: Array<any>;
        completed: boolean,
        isLineHaul: boolean
    };
    vehicleName: string;
    vehicleIds: any;
    filterId: any;
    txtFilter : any;
    advancedFiltersAreShown = false;

    searchFilters: {
        route: string,
        vehicleName: string,
        driverName: string,
        showCompletion: boolean,
        routeIds: Array<number>,
        orderType: Array<string>,
        deviceType: string,
    } = <any>{};
	constructor(
        injector: Injector,
        private _vehicleListService: ControllerVehicleListServiceProxy
        
    ) {
        super(injector);
    }

    ngOnInit(): void {
        
    }
    getFilters(){
        if (localStorage.getItem('searchFilter') == null) {
            this.searchFilters.route = '';
            this.searchFilters.driverName = '';
            this.searchFilters.showCompletion = false;
            this.searchFilters.vehicleName = '';
        }
        else {
            this.searchFilters = JSON.parse(localStorage.getItem('searchFilter'));
        }
    }

    getVehicleList(event?: LazyLoadEvent): void
    {   
        this.getFilters();
        var key;
        this.vehicleIds = [];
        let timeId = localStorage.getItem('timeZoneId');
        moment.tz.setDefault(timeId);

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
            
        }
        
        this.spinnerService.show();
       
        this._vehicleListService.getControllerVehicleList(
            this.txtFilter,
            this.filterId,
            this.filters.locationFilter,
            moment.tz(localStorage.getItem('loadDate'), timeId),
            moment.tz(localStorage.getItem('loadDate'), timeId).endOf('day'),
            this.filters.completed,
            this.filters.routeIds,
            this.filters.isLineHaul,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())
        ).subscribe(result => {
            
            // for (key in result.items)
            // {
            //     this.vehicleIds.push(result.items[key].vehicleId);
            // }
            // this._vehicleListService.getVehicleNameAndActiveRoute(this.vehicleIds
                
            // ).subscribe(vehicleResult => {

            //     if(vehicleResult !== null)
            //         {
            //             if(Object.keys(vehicleResult).length > 0)
            //             {
            //                 for (key in vehicleResult)
            //                 {   
            //                     $('#vehicleName' + vehicleResult[key].vehicleId).html(vehicleResult[key].fullName);
            //                     //$('#activeRoute' + vehicleResult[key].vehicleId).html((vehicleResult[key].activeRouteName != null ? vehicleResult[key].activeRouteName : "").toString());
            //                     $('#activeRoute' + vehicleResult[key].vehicleId).html('<a target="_blank" href="app/sprintship/controller-routes-details/'+ vehicleResult[key].activeRouteId +'">'+ (vehicleResult[key].activeRouteName != null ? vehicleResult[key].activeRouteName : "").toString()+'</a>');
                                
                               
            //                 }
            //             } 
            //         } 
            // });
            // console.log('RESULT VEHICLE: ');
            // console.log(result)

            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    
}