import { Component, Injector, OnInit, ViewChild, ViewEncapsulation, Input, ViewChildren } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ControllerDriverListDto, ControllerVehicleListServiceProxy} from '@shared/service-proxies/service-proxies';

import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators';

@Component({
    templateUrl: './vehicle-route-details.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./vehicle-route-details.component.less'],
    providers: [ControllerVehicleListServiceProxy],
    styles: ['.table-title {margin: 0 !important;min-height: auto !important;}']
})

export class VehicleDetailComponent extends AppComponentBase  implements OnInit{
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
  
    locationFilter : number;
    vehicleId: number;
    vehicleName: string;

	constructor(
        injector: Injector,
        private _vehicleListService: ControllerVehicleListServiceProxy,
        private activatedRoute: ActivatedRoute,
        
    ) {
        super(injector);
    }

    ngOnInit(): void {
    
        this.activatedRoute.paramMap.subscribe(params => {
            this.locationFilter = +params.get("location");
            this.vehicleId = +params.get("vehicleid");
        });
        
        this._vehicleListService.getVehicleEntity(
            this.vehicleId,
            undefined,
            undefined,
            undefined,
            undefined
        ).pipe(finalize(() => this.spinnerService.hide())
        ).subscribe(result => {
            this.vehicleName = result.name
        });
    }
  
    getRouteVehicles(event?: LazyLoadEvent): void
    {
 
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
    
        this.spinnerService.show();
       
        this._vehicleListService.getVehicleRoutes(
            this.vehicleId,
            this.locationFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    
}