import { Component, Injector, ViewChild } from '@angular/core';
import { PermissionTreeComponent } from '@app/admin/shared/permission-tree.component';
import { AppComponentBase } from '@shared/common/app-component-base';
import { CustomerServiceProxy, EntityDtoOfInt64, UpdateUserPermissionsInput, UserServiceProxy } from '@shared/service-proxies/service-proxies';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { ContactPersonPermissionTreeComponent } from './contact-person-permission-tree.component';

@Component({
    selector: 'contactPersonPermissionsModal',
    templateUrl: './contact-person-permission.component.html'
})
export class ContactPersonPermissionsModalComponent extends AppComponentBase {

    @ViewChild('modal', { static: false }) modal: ModalDirective;
    @ViewChild('permissionTree', {static: false}) permissionTree: ContactPersonPermissionTreeComponent;

    saving = false;
    resettingPermissions = false;

    userId: number;
    userName: string;

    constructor(
        injector: Injector,
        private _userService: UserServiceProxy,
        private _customerService: CustomerServiceProxy
    ) {
        super(injector);
    }

    show(userId: number, userName?: string): void {
        this.userId = userId;
        this.userName = userName;

        this._customerService.getUserRolePermissions(userId, "CONTACTPERSON").subscribe(result => {
            this.permissionTree.editData = result;
            this.modal.show();
        });

        // this._userService.getContacPersonPermissions(userId).subscribe(result => {
        //     this.permissionTree.editData = result;
        //     this.modal.show();
        // });
    }

    save(): void {
        let input = new UpdateUserPermissionsInput();

        input.id = this.userId;
        input.grantedPermissionNames = this.permissionTree.getGrantedPermissionNames();

        this.saving = true;
        this._customerService.updateContactPersonPermissions(input)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
            });
    }

    resetPermissions(): void {

        let input = new EntityDtoOfInt64();

        input.id = this.userId;

        this.resettingPermissions = true;
        this._userService.resetUserSpecificPermissions(input).subscribe(() => {
            this.notify.info(this.l('ResetSuccessfully'));
            this._userService.getUserPermissionsForEdit(this.userId).subscribe(result => {
                this.permissionTree.editData = result;
            });
        }, undefined, () => {
            this.resettingPermissions = false;
        });
    }

    close(): void {
        this.modal.hide();
    }
}
