import { Component, Injector, ViewChild, ViewEncapsulation, LOCALE_ID, Inject, ɵDEFAULT_LOCALE_ID, ViewChildren, QueryList } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrderServiceProxy, OrderListDto, LocationServiceProxy, LocationListDto, OrderTypesServiceProxy, RoleListDto, GetOrderListDtoNew, CustomerServiceProxy, GetCustomerProfileDto, ClinicOrders, PagedResultDtoOfGetOrderListDtoNew } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { catchError, finalize } from 'rxjs/operators';
import * as moment from 'moment';
import { PrimengTableHelper } from '@shared/helpers/PrimengTableHelper';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { HttpClient } from '@angular/common/http';
import { AppConsts } from '@shared/AppConsts';
import { FileUpload } from 'primeng/fileupload';
import * as signalR from '@aspnet/signalr';
import { LocalizationService } from 'abp-ng2-module/dist/src/localization/localization.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import { MovePendingOrder } from '@app/sprintship/controller/route-detail/move-order-modal.component';
import { HeaderNotificationsComponent } from '@app/shared/layout/notifications/header-notifications.component';
import { ProgressBarService } from '@app/shared/layout/progressbar/progressbar.service';
import { CreateOrderModalComponent } from '../order-management/orders/create-order-modal.component';
import { ImportOrderTrackingComponent } from '../order-management/orders/import-order-tracking.component';

@Component({
    templateUrl: './customer-order.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./customer-order.component.less']
})
export class CustomerOrderComponent extends AppComponentBase {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Orders.Order';
    entityHistoryEnabled = false;
    //Filters
    filterText = undefined;
    locationFilter: LocationListDto = new LocationListDto();
    filterDate: moment.Moment;
    filterRouteDate: any;
    filterDateString: any;
    maxResultCount = 1000;
    filteredOrder: any;
    filteredLocation: any;
    filteredOrderTypes: any;
    advancedFiltersAreShown: false;
    TimeCreated: any;
    RouteDate: any;
    deliveryEta: any;
    orderId: string;
    locationId: number;
    orderTypeId: number;
    isActiveFilter = -1;
    uploadUrl: string;
    orderNumberFilter: string;

    orderImporting: boolean = false;

    featurefbx: boolean = false;

    _ukLocale: boolean;
    _isLab: boolean;
    private hubConnection: signalR.HubConnection;
    unassigned: boolean;
    sprintIds: Array<{}> = new Array();
    sprintIdsList: any[];
    checkAll: boolean;
    records: any;
    profile: GetCustomerProfileDto = new GetCustomerProfileDto();
    isCustomer: boolean = false;
    sched: ClinicOrders[];
    routeId: string;
    constructor(
        injector: Injector,
        private router: Router,
        private _orderAppService: OrderServiceProxy,
        private _locationAppService: LocationServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private _orderTypeService: OrderTypesServiceProxy,
        private _httpClient: HttpClient,
        private _localization: LocalizationService,
        private _progressBar: ProgressBarService,
        private _customerRepository: CustomerServiceProxy
    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/Import/ImportFromExcel';
        this.orderImporting = abp.features.isEnabled('App.OrderImporting');
        this.featurefbx = abp.features.isEnabled('App.FBXFeature');
    }

    ngOnInit() {
        this.setIsEntityHistoryEnabled();
        $('.kt-select2').select2({
            placeholder: "Select location.."
        });
        this._isLab = abp.features.isEnabled('App.RouteDetails');
        this._ukLocale = abp.features.isEnabled('App.DateFormatFeature.UKLocale');
        // this.filterLocation();
        // this.filterOrder();
        this.filterOrderTypes()
        this.backgroundProcessListener();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetOrderListDtoNew): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order - " + role.id.toString()
        });
    }

    getOrders(event?: LazyLoadEvent) {

        // this.orderId =  Number((<HTMLInputElement>document.getElementById('OrderId')).value) ? Number((<HTMLInputElement>document.getElementById('OrderId')).value) : undefined;
        //this.locationId = Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) ? Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) : undefined;
        this.orderTypeId = Number((<HTMLInputElement>document.getElementById('orderType')).value) ? Number((<HTMLInputElement>document.getElementById('orderType')).value) : undefined;

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        this.spinnerService.show();
        if (this.TimeCreated != undefined) {
            this.filterDate = moment(this.TimeCreated);
        }
        if (this.TimeCreated != undefined) {
            this.filterDateString = new Date(this.TimeCreated).toLocaleDateString('en-US');
        }
        if (this.RouteDate != undefined) {
            this.filterRouteDate = new Date(this.RouteDate).toLocaleDateString('en-US');
        }

        this._orderAppService.getCustomerOrders(
            undefined,
            this.filterText,
            undefined,
            undefined,
            undefined,
            undefined,
            this.filterDate,
            undefined, //moment(this.filterDate),
            undefined,
            this.orderTypeId,
            this.orderId,
            this.orderNumberFilter,
            undefined,
            this.unassigned,
            false,
            this.filterRouteDate,
            this.filterDateString,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(catchError((err, caught): any => {
            this.spinnerService.hide();
        }))
        .subscribe((result: PagedResultDtoOfGetOrderListDtoNew) => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            if(result!=null){
                for (let index = 0; index < result.items.length; index++) {
                    if (this._ukLocale) {
                        result.items[index].createdString = result.items[index].createDate.utcOffset(moment.parseZone(result.items[index].createDate).utcOffset()).format('D/M/YYYY h:mm a');
                        // result.items[index].lastModifiedString = result.items[index].lastModifiedTime.utcOffset(moment.parseZone(result.items[index].lastModifiedTime).utcOffset()).format('D/M/YYYY h:mm a');
                    }
                    else {
                        result.items[index].createdString = result.items[index].createDate.utcOffset(moment.parseZone(result.items[index].createDate).utcOffset()).format('M/D/YYYY h:mm a');
                        // result.items[index].lastModifiedString = result.items[index].lastModifiedTime.utcOffset(moment.parseZone(result.items[index].lastModifiedTime).utcOffset()).format('M/D/YYYY h:mm a');
                    }
                }
            }
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    filterLocation(): void {
        this._locationAppService.getLocationNames(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            this.filteredLocation = result;

        });
    }

    filterOrderTypes(): void {
        this._orderTypeService.getAll(
            undefined,
            undefined,
            undefined,
            this.isActiveFilter,
            undefined,
            undefined,
            undefined,
            undefined,
        ).subscribe(result => {
            this.filteredOrderTypes = result.items;

        });
    }

    deleteOrder(order: OrderListDto): void {
        this.message.confirm(
            this.l(''),
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderAppService.deleteOrder(order.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    createOrUpdate(url: any, id?: number) {
        if (id != null) {
            var myurl = `${url}/${id}`;
        } else {
            var myurl = `${url}`;
        }
        this.router.navigateByUrl(myurl);
    }

    exportToExcel(event?: LazyLoadEvent): void {
        //this.locationId = Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) ? Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) : undefined;
        this.orderTypeId = Number((<HTMLInputElement>document.getElementById('orderType')).value) ? Number((<HTMLInputElement>document.getElementById('orderType')).value) : undefined;

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        this.spinnerService.show();
        if (this.TimeCreated != undefined) {
            this.filterDate = moment(this.TimeCreated);
        }
        if (this.TimeCreated != undefined) {
            this.filterDateString = new Date(this.TimeCreated).toLocaleDateString('en-US');
        }
        if (this.RouteDate != undefined) {
            this.filterRouteDate = new Date(this.RouteDate).toLocaleDateString('en-US');
        }
        this.spinnerService.show();
        this._orderAppService.getCustomerOrdersToExcel(
            undefined,
            this.filterText,
            undefined,
            undefined,
            undefined,
            undefined,
            this.filterDate,
            undefined, //moment(this.filterDate),
            undefined,
            this.orderTypeId,
            this.orderId,
            undefined,
            undefined,
            this.unassigned,
            false,
            this.filterRouteDate,
            this.filterDateString,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        )
            .subscribe(result => {
                if(result!=undefined){
                    if(result.fileUrl!=undefined){
                        this.router.navigate([]).then(result1 => {
                            window.open(result.fileUrl, '_blank');
                        });
                        this.spinnerService.hide();
                    }else{
                        this.spinnerService.hide();
                    }
                }else{
                    this.message.warn('', "No data to export");
                    this.spinnerService.hide();
                }
            });
    }



    uploadExcel(data: { files: File }): void {
        //this.importOrderTracking.clearMessages();
        //this.importOrderTracking.modal.show();
        const formData: FormData = new FormData();
        const file = data.files[0];
        formData.append('file', file, file.name);
        formData.append('key', 'orders');
        formData.append('locale', abp.localization.currentLanguage.name);
        // formData.append('entity','addresses','addresses');
        // console.log(abp.localization.currentLanguage.name);
        this._httpClient
            .post<any>(this.uploadUrl, formData)
            .subscribe(response => {
                if (response.success) {
                    this.notify.info('Import process is queued. Please wait'); //this.l('ImportOrderStart')
                    this.toggle(true);
                } else if (response.error != null) {
                    this.toggle(false);
                    this.notify.error(this.l('ImportOrderFailed'));
                }
            }, error => {
                this.toggle(false);
                this.notify.error(this.l('ImportOrderFailed'));
            });
    }

    onUploadExcelError(): void {
        this.notify.error(this.l('ImportOrderFailed'));
    }

    backgroundProcessListener() {
        this.hubConnection = new signalR.HubConnectionBuilder()
            .withUrl(AppConsts.remoteServiceBaseUrl + '/import')
            .build();

        this.hubConnection
            .start()
            .then(() => console.log('Connection started'))
            .catch(err => console.log('Error while starting connection: ' + err));

        this.hubConnection.on('BackgroundMessage', (message) => {

            // if(message.match(/Route created/) || message.match(/Order created/)) {

            //     this.importOrderTracking.logMessage(message, true);
            // }
            // else {
            // if (message.match(/End Importing/)) {
            //     this.importOrderTracking.importing = false;
            //     this.toggle(false);
            //     this.notify.success('Import process is complete');
            //     this.getOrders();
            // } else {
            //     this.importOrderTracking.logMessage(message);
            // }
            this.sendMessage(message);
            // }
        });
    }

    createIncidents(url: any, id?: number) {
        if (id != null) {
            var myurl = `${url}/${id}`;
        } else {
            var myurl = `${url}`;
        }
        //window.open(myurl, "_blank")
        this.router.navigateByUrl(myurl);
    }

    toggle(toggle: boolean) {
        this._progressBar.toggleProgressBar(toggle);
    }

    sendMessage(message: string) {
        this._progressBar.sendMessage(message);
    }

    selectAllOrders() {
        this.sprintIdsList = null;
        this.records = this.primengTableHelper.records
        for (var i = 0; i < this.records.length; i++) {
            if (this.checkAll == true) {
                this.sprintIds.push(this.records[i].id);
                this.records[i].selected = this.checkAll;
            }
            else {
                this.sprintIds.splice(this.sprintIds.indexOf(this.records[i].id), 1);
                this.records[i].selected = this.checkAll;
            }
        }
        console.log(this.sprintIdsList)
    }
    moveOrders(id: number) {
        if (this.sprintIds.indexOf(id) == -1) {
            this.sprintIds.push(id);
        } else {
            this.sprintIds.splice(this.sprintIds.indexOf(id), 1);
        }

        this.sprintIdsList = this.sprintIds;
    }
}
