﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { DataRetentionsServiceProxy, DataRetentionDto , EntityEnum, Frequency } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditDataRetentionModalComponent } from './create-or-edit-dataRetention-modal.component';
import { ViewDataRetentionModalComponent } from './view-dataRetention-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
import { ViewDataretentionLogModalComponent } from './view-dataretention-log-modal.component';

@Component({
    templateUrl: './data-retention.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class DataRetentionsComponent extends AppComponentBase {

    @ViewChild('createOrEditDataRetentionModal', { static: true }) createOrEditDataRetentionModal: CreateOrEditDataRetentionModalComponent;
    @ViewChild('viewDataRetentionModalComponent', { static: true }) viewDataRetentionModal: ViewDataRetentionModalComponent;
    @ViewChild('viewDataRetentionLogModalComponent', {static: true}) viewDataRetentionLogModal: ViewDataretentionLogModalComponent
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    entityFilter = -1;
    frequencyFilter = -1;
    maxDurationFilter : number;
		maxDurationFilterEmpty : number;
		minDurationFilter : number;
		minDurationFilterEmpty : number;

    entityEnum = EntityEnum;
    frequency = Frequency;

    _entityTypeFullName = 'SprintTek.Shipping.DataRetentions.DataRetention';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _dataRetentionsServiceProxy: DataRetentionsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getDataRetentions(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._dataRetentionsServiceProxy.getAll(
            this.filterText,
            this.entityFilter,
            this.frequencyFilter,
            this.maxDurationFilter == null ? this.maxDurationFilterEmpty: this.maxDurationFilter,
            this.minDurationFilter == null ? this.minDurationFilterEmpty: this.minDurationFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createDataRetention(): void {
        this.createOrEditDataRetentionModal.show();
    }

    showHistory(dataRetention: DataRetentionDto): void {
        this.entityTypeHistoryModal.show({
            entityId: dataRetention.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteDataRetention(dataRetention: DataRetentionDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._dataRetentionsServiceProxy.delete(dataRetention.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._dataRetentionsServiceProxy.getDataRetentionsToExcel(
            this.filterText,
            this.entityFilter,
            this.frequencyFilter,
            this.maxDurationFilter == null ? this.maxDurationFilterEmpty: this.maxDurationFilter,
            this.minDurationFilter == null ? this.minDurationFilterEmpty: this.minDurationFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    timeZoneFormat(date: moment.Moment) {
        let format = date.tz(moment.tz.guess()).zoneName();
        return format;
    }

}
