import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { IncidentServiceProxy, DriverIncidentServiceProxy, DriverIncidentListDto, DriverIncidentsServiceProxy, EmployeeIncidentTypesServiceProxy, GetDriverIncidentTypeForViewDto, DriverIncidentTypesServiceProxy, DriverIncidentsListDto } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import * as moment from 'moment';
import { Router } from '@angular/router';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
// import { CreateOrEditDriverIncidentModalComponent } from './create-or-edit-driver-incident.component';

@Component({
    templateUrl: './driver-incidents.component.html',
    animations: [appModuleAnimation()],
    encapsulation: ViewEncapsulation.None
})

export class DriverIncidentsComponent extends AppComponentBase {

    // @ViewChild('createOrEditDriverIncidentModal', {static: false}) createOrEditDriverIncidentModal: CreateOrEditDriverIncidentModalComponent;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Incidents.DriverIncident';
    entityHistoryEnabled = false;

    public dateRange: moment.Moment[] = [moment("2019-01-01").startOf('day'), moment().endOf('day')];
    driverIncidentType: GetDriverIncidentTypeForViewDto[];

    advancedFiltersAreShown = false;
    sortingColumns:string = null;

    filters: {
        filter: string;
        employeeName: string;
        content: string;
        incidentDate: any;
        incidentTypeId: number;
        startDate: any;
        endDate: any;
    } = <any>{};

    constructor(
        injector: Injector,
        private _driverIncidentServiceProxy: DriverIncidentServiceProxy,
        private _driverIncidentsServiceProxy: DriverIncidentsServiceProxy,
        private _driverIncidentTypeServiceProxy: DriverIncidentTypesServiceProxy,
        private _incidentServiceProxy: IncidentServiceProxy,
        private _employeeIncidentTypeServiceProxy: EmployeeIncidentTypesServiceProxy,
        private _router: Router,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);  
    }

    onShown(): void {
        $('.kt-select2').select2();
    }

    ngOnInit(){
        this.getDriverIncidentType(1000);
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: DriverIncidentsListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Driver Incident - " +role.driverName
        });
    }

    delete(driverIncident: DriverIncidentsListDto): void {

        this.message.confirm(
            this.l('IncidentDeleteWarningMessage', driverIncident.incidentContent),
            this.l('AreYouSure'),
            (isConfirmed)=> {
                if(isConfirmed) {
                    this._driverIncidentsServiceProxy
                        .deleteDriverIncident(driverIncident.id)
                        .subscribe(()=> {
                            this.reloadPage();
                            this.notify.success(this.l('Successfully Deleted'));
                        });
                }
            }
        )
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    getIncidents(event?: LazyLoadEvent) {
        this.spinnerService.show();
        this.filters.incidentTypeId = Number((<HTMLInputElement>document.getElementById('driverIncidentType')).value) != 0 ? 
        Number((<HTMLInputElement>document.getElementById('driverIncidentType')).value) : undefined;
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._driverIncidentsServiceProxy.getDriverIncident(
            undefined,this.filters.filter,this.filters.content,
            this.filters.incidentTypeId, undefined, this.filters.employeeName, 
            this.dateRange[0],
            this.dateRange[1].endOf('day'),
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.sortingColumns = this.primengTableHelper.getSorting(this.dataTable);
            this.spinnerService.hide();
        });
    }
    getDriverIncidentType(maxcount): void{
        this._driverIncidentTypeServiceProxy.getAll(undefined, undefined, undefined, undefined, undefined, -1, undefined, undefined, maxcount)
                                              .subscribe(result =>{
                                                this.driverIncidentType = result.items;
                                              });
    }
    createOrEditDriverIncident(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        this._router.navigateByUrl(myurl);
    }

    exportToExcel(){
        this.spinnerService.show();
        this._driverIncidentsServiceProxy.getDriverIncidentsExcel(
            undefined,this.filters.filter,this.filters.content,
            this.filters.incidentTypeId, undefined, this.filters.employeeName, 
            this.dateRange[0],
            this.dateRange[1].endOf('day'),
            this.sortingColumns,
            undefined,
            undefined
        ).subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
     })
    }
}