﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { EmployeeIncidentTypesServiceProxy, CreateOrEditEmployeeIncidentTypeDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditEmployeeIncidentTypeModal',
    templateUrl: './create-or-edit-employeeIncidentType-modal.component.html'
})
export class CreateOrEditEmployeeIncidentTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    employeeIncidentType: CreateOrEditEmployeeIncidentTypeDto = new CreateOrEditEmployeeIncidentTypeDto();



    constructor(
        injector: Injector,
        private _employeeIncidentTypesServiceProxy: EmployeeIncidentTypesServiceProxy
    ) {
        super(injector);
    }

    show(employeeIncidentTypeId?: number): void {

        if (!employeeIncidentTypeId) {
            this.employeeIncidentType = new CreateOrEditEmployeeIncidentTypeDto();
            this.employeeIncidentType.id = employeeIncidentTypeId;

            this.active = true;
            this.modal.show();
        } else {
            this._employeeIncidentTypesServiceProxy.getEmployeeIncidentTypeForEdit(employeeIncidentTypeId).subscribe(result => {
                this.employeeIncidentType = result.employeeIncidentType;


                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

			
            this._employeeIncidentTypesServiceProxy.createOrEdit(this.employeeIncidentType)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
