import { Component, Injector, ViewEncapsulation, ViewChild, OnInit, Injectable, AfterViewInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy, OrderServiceProxy, IncidentTypesServiceProxy, OrderIncidentsServiceProxy, IncidentStatusServiceProxy, WillCallServiceServiceProxy, IncidentServiceProxy, OrderIncidentNotesForUpdate, OrderIncidentAttachmentForUpdate, CreateAttachmentIncidentInput, UpdateOrderIncidentInput, DriverIncidentsServiceProxy, DriverIncidentTypesServiceProxy, DriversServiceProxy, GetDriverListDto, DriverIncidentNotesForUpdate, DriverIncidentAttachmentForUpdate, CreateOrEditDriverIncidentsInput, DriverIncidentNotes, DriverIncidentAttachment, EmployeeIncidentsServiceProxy, EmployeeIncidentTypesServiceProxy, CreateOrEditEmployeeIncidentsInput } from '@shared/service-proxies/service-proxies';
import { UserServiceProxy, UserListDto, SupportTicketServiceProxy, SupportTicketStatusServiceProxy, SupportTicketTypeServiceProxy, TicketTypeList, TicketStatusList, CreateTicketDto, CreateAttachments, CreateNotes, AttachmentList, NoteList, GetTicketForEdit, UpdateTicketDto } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppSessionService } from '@shared/common/session/app-session.service';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
import { Router } from '@angular/router';
import { BsDatepickerConfig } from 'ngx-bootstrap/datepicker';
import { HttpEventType, HttpClient } from '@angular/common/http';
import { FormControl } from '@angular/forms';
import {Location} from '@angular/common';
import { AppConsts } from '@shared/AppConsts';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import { HttpHeaders } from '@angular/common/http';
import { map, findIndex } from "rxjs/operators";
import { NgxDropzoneModule } from 'ngx-dropzone';
import { saveAs } from 'file-saver';
import { Observable } from 'rxjs';
import { Subject } from 'rxjs';
import { CreateIncidentNotesModalComponent } from '@app/sprintship/incident/create-incidentnote-modal';
import { DriverIncidentTypesComponent } from '@app/sprintship/fleet-management/drivers/driver-incident-types/driverIncidentTypes.component';
@Component({
    templateUrl: './edit-employee-incident.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./employee-incident.component.less'],
})
@Injectable()
export class EditEmployeeIncidentComponent extends AppComponentBase implements AfterViewInit {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('createIncidentNotesModal', {static: false}) createIncidentNotesModal: CreateIncidentNotesModalComponent;
    bsConfig: Partial<BsDatepickerConfig>;

    id: any;
    defaultDate = new Date();
    incidentDate: any;
    incidentType: any;
    typesId: number;
    content: string;
    notes: DriverIncidentNotesForUpdate[] = [];
    filesDropDown: DriverIncidentAttachmentForUpdate[] = [];
    noteList: NoteList[] = [];
    today: any;
    uploadUrl: string;
    downloadUrl: string;
    uploadedFiles: any[] = [];
    data: any[] = [];
    noteCreator: string;
    tenancy: boolean = false;
    hideNew: boolean = false;
    showAlert: boolean = false;
    assignedToId: any;
    assignedTo: any;
    ticketTypeId: any;
    ticketStatusId: any;
    shortDescription: any;
    description: any;
    parentId: any;
    ticketStatus: any;
    selectedType: any;
    selectedAssignedId: any;
    selectedStatusId: any;
    hasDownloads: boolean = false;
    files: File[] = [];
    notesCreate: Array<any> = [];
    attachMents: Array<any> = [];
    incidentTypes: any;
    managers: any;
    statuses: any;
    typeId: any;
    statusId: any;
    managerId: any;
    routedriverString: any;
    orderId: any;
    idate: any;
    email:string;
    timeId: any;
    userId: any;
    driverIncidentType: any;
    filter: any;
    users: GetDriverListDto[] = [];
    incidentId: any;
    employeeName: string;
    constructor(
        injector: Injector,
        private _httpClient: HttpClient,
        private _router: Router,
        private route: ActivatedRoute,
        private _orderIncidentsService : OrderIncidentsServiceProxy,
        private _employeeIncidentsService : EmployeeIncidentsServiceProxy,
        private _incidentStatus : IncidentStatusServiceProxy,
        private _appSessionService: AppSessionService,
        private _incidentAppService: IncidentServiceProxy,
        private _employeeIncidentTypeServiceProxy: EmployeeIncidentTypesServiceProxy,
        private _location: Location,
    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/DemoUiComponents/UploadFiles';
        
    }

    ngOnInit(): void {
        $('.kt-select2').select2();
        this.getAllData();
        this.timeId = localStorage.getItem('timeZoneId');
        moment.tz.setDefault(this.timeId);
        
        this.getDriverIncidentType(1000);
        this.getManagers();
        this.getIncidentStatus();
        // this.showAllUsers(1000);
        this._incidentStatus.getAllIncidentStatus(undefined,undefined)
        .subscribe(result=>{
            this.statuses = result;
        })

    }
    
    ngAfterViewInit(): void {
        let maxcount = 1000;
        this.today = new Date();
        this.noteCreator = this._appSessionService.user.name + ' ' + this._appSessionService.user.surname;
    }

    getAllData(){
        this.notes = [];
        this.filesDropDown = [];
        this.spinnerService.show();
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id");
        });
        this._employeeIncidentsService.getEmployeeIncidentForEdit(this.id).subscribe((result) => {
          this.incidentId = result.incidentId
          this.userId = result.userId;
          this.typeId = result.employeeIncidentTypeId;
          this.statusId = result.statusId;
          this.managerId = result.managerId;
          this.content = result.incidentContent;
          this.defaultDate = moment.tz(result.incidentDate, this.timeId).toDate();
          this.notes = result.notes;
          this.filesDropDown = result.attachments;
          this.email = result.email;
        //   this.id = result.incidentId;
          this.employeeName = result.employeeName;
          this.defaultDate = moment.tz(result.incidentDate, localStorage.getItem('timeZoneId')).toDate();
          this.spinnerService.hide();
        });
    }

    showAllFilters(){
        
    }

    incidentNotes(): void {
        this.createIncidentNotesModal.show();
    }
    
    onSelect(event) {
      if(event.rejectedFiles.length != 0) {
        this.showAlert = true;
      }

      this.files.push(...event.addedFiles);


      let f = event.addedFiles;
      for (const file of f) { 
         this.getData(file);
      }

      
    }
    

    closeAlert() {
      this.showAlert = false;
    }
     
    onRemove(event) {
      this.files.splice(this.files.indexOf(event), 1);
      //this.data.splice(this.data.indexOf(event), 1);
      
      _.remove(this.data, function (el) {
        
           return el.fileName === event.name;
           
        
       });

    }

    goBack(){
        this._location.back();
    }

   downloadFile(id: number, fileName: string) {
              let checkFileType = fileName.split('.').pop();
              var fileType;
              
                      if (checkFileType == ".txt") {
                          fileType = "text/plain";
                      }
                      if (checkFileType == ".pdf") {
                          fileType = "application/pdf";
                      }
                      if (checkFileType == ".doc") {
                          fileType = "application/vnd.ms-word";
                      }
                      if (checkFileType == ".docx") {
                          fileType = "application/vnd.ms-word";
                      }
                      if (checkFileType == ".xls") {
                          fileType = "application/vnd.ms-excel";
                      }
                      if (checkFileType == ".png") {
                          fileType = "image/png";
                      }
                      if (checkFileType == ".jpg") {
                          fileType = "image/jpeg";
                      }
                      if (checkFileType == ".jpeg") {
                          fileType = "image/jpeg";
                      }
                      if (checkFileType == ".gif") {
                          fileType = "image/gif";
                      }
                      if (checkFileType == ".csv") {
                          fileType = "text/csv";
                      }
                      this.DownloadFile(fileName, fileType, id)
                          .subscribe(
                              success => {
                                  saveAs(success, fileName);
                              },
                              err => {
                                  alert("Server error while downloading file.");
                              }
                          );
       
  }


        DownloadFile(filePath: string, fileType: string, id: number): Observable<any> {

                let fileExtension = fileType;
                let input = filePath;

                return this._httpClient.get(AppConsts.remoteServiceBaseUrl + '/api/services/app/Incident/DownloadFile?id=' + id, {
                    responseType: 'blob',
                    observe: 'response'
                }).pipe(
                        map((res: any) => {
                            return new Blob([res.body], { type: fileExtension });
                        })
                    );
        }


   
    save(): void {
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id");
        });
        let that = this;
        that.typesId = Number($('#driverIncidentType1').val()) == 0 ? null :  Number($('#driverIncidentType1').val());
        let update = new CreateOrEditEmployeeIncidentsInput();
        update.employeeIncidentTypeId = that.typesId;
        update.incidentContent = that.content;
        update.incidentDate = moment.utc(this.defaultDate);     
        update.managerId = Number($('#Manager').val());
        update.incidentStatusId= Number($('#Status').val());
        update.email = that.email;
        update.id = that.id
        update.userId = this.userId;
        for(let item of this.notes) {
            let a = new DriverIncidentNotes({
              incidentId: 0,
              notes: item.notes,
            });
            this.notesCreate.push(a);
        } 
        for (let item of this.data) {
            let a = new DriverIncidentAttachment({
                fileName: item.fileName,
                base64EncodedData: item.base64EncodedData,
                incidentId: 0,
            });
            this.attachMents.push(a);
        }
        update.createAttachments = this.attachMents;
        update.createNotes = this.notesCreate;
        this.spinnerService.show();
        this._employeeIncidentsService.updateEmployeeIncident(update).pipe().subscribe(result => {
            this.notify.info(this.l('SavedSuccessfully'));
            this.spinnerService.hide();
            this.getAllData();
            // this._router.navigateByUrl('/app/sprintship/driver-incidents');
       });
        
    }

    onBeforeSend(event): void {
        event.xhr.setRequestHeader('Authorization', 'Bearer ' + abp.auth.getToken());
    }


    getAttachmentsRefresh(){
        this.notes = [];
        this.filesDropDown = [];
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id");
        });
        this._employeeIncidentsService.getEmployeeIncidentForEdit(this.id).subscribe((result) => {
          this.userId = result.userId;
          this.statusId = result.statusId;
          this.defaultDate = moment.tz(result.incidentDate, this.timeId).toDate();
          this.notes = result.notes;
          this.filesDropDown = result.attachments;
          this.email = result.email;
          this.id = result.incidentId;
          this.incidentId = result.incidentId;
          this.employeeName = result.employeeName;
          this.defaultDate = moment.tz(result.incidentDate, localStorage.getItem('timeZoneId')).toDate();
        });
    }
    
    getData(file) {
              
              var reader:any;
              let me = this;
              
              let data = new CreateAttachmentIncidentInput();
              reader = new FileReader();
              reader.readAsDataURL(file);
              reader.onload = function() {
                              me.data.push(      
                              { fileName: file.name, base64EncodedData: reader.result.toString().split(',')[1] });
                              data.base64EncodedData = reader.result.toString().split(',')[1];
                              data.fileName = file.name;
                              data.incidentId = me.incidentId;
                              me._incidentAppService.createAttachement(data).subscribe(result=>{
                                me.getAttachmentsRefresh();
                              });
              };
              reader.onerror = function (error) {
                console.log('Error: ', error);
              };
    }

     reloadPage(): void {
        //this.paginator.changePage(this.paginator.getPage());
         this.ngOnInit();
    }

    deleteRow(id){
                    this.message.confirm(
                        '',
                        '',
                        (isConfirmed) => {
                            if (isConfirmed) {
                            }
                        }
                    );
             
    }

    deleteFile(id) {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._incidentAppService.deleteAttachment(id)
                        .subscribe(() => {
                            this.getAttachmentsRefresh();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    attach(){
        $('#FileDrop').click();
        this.getData('#FileDrop');
     }
     getDriverIncidentType(maxcount): void {
        this._employeeIncidentTypeServiceProxy.getAll(undefined, undefined, undefined, undefined, undefined, -1, undefined, undefined, maxcount)
           .subscribe(result => {
              this.driverIncidentType = result.items;
           });
     }
     getManagers(): void {
        this._employeeIncidentsService.getManagers().subscribe(result => {
              this.managers = result;
           })
     }
  
     getIncidentStatus(): void {
        this._incidentStatus.getAllIncidentStatus(undefined, undefined).subscribe(result => {
              this.statuses = result;
           })
     }
     refreshNote(){
        //  console.log(this.id)
        this.spinnerService.show()
        this._employeeIncidentsService.getEmployeeIncidentForEdit(this.id).subscribe((result) => {
            this.notes = result.notes;
            this.spinnerService.hide()
          }), error=>{
              this.spinnerService.hide()
          };
      }
}
