import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { IncidentServiceProxy, DriverIncidentServiceProxy, DriverIncidentListDto, EmployeeIncidentsServiceProxy, EmployeeIncidentTypesServiceProxy, GetEmployeeIncidentTypeForViewDto, EmployeeIncidentsListDto } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import * as moment from 'moment';
import { id } from '@swimlane/ngx-charts/release/utils';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { Router } from '@angular/router';
import { create } from 'domain';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    templateUrl: './employee-incidents.component.html',
    animations: [appModuleAnimation()],
    encapsulation: ViewEncapsulation.None
})

export class EmployeeIncidentsComponent extends AppComponentBase {
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Employees.EmployeeIncident';
    entityHistoryEnabled = false;

    public dateRange: moment.Moment[] = [moment("2020-01-01").startOf('day'), moment().endOf('day')];

    employeeIncident: GetEmployeeIncidentTypeForViewDto[];

    filters: {
        filter: string;
        employeeName: string;
        content: string;
        incidentDate: any;
        incidentTypeId: number;
        startDate: any;
        endDate: any;
    } = <any>{};

    advancedFiltersAreShown = false;
    sortingColumns:string = null;
    constructor(
        injector: Injector,
        private _employeeIncidentServiceProxy: EmployeeIncidentsServiceProxy,
        private _employeeIncidentTypesServiceProxy: EmployeeIncidentTypesServiceProxy,
        private _incidentServiceProxy: IncidentServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private _router: Router,
    ) {
        super(injector);  
    }

    onShown(): void {
        $('.kt-select2').select2();
    }

    ngOnInit(): void{
        $('.kt-select2').select2();
        this.setIsEntityHistoryEnabled();
        this.getEmployeeIncidentType(1000);
        this.setIsEntityHistoryEnabled();
        
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: EmployeeIncidentsListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Employee Incident - " +role.employeeName
        });
    }

    delete(employee: EmployeeIncidentsListDto): void {

        this.message.confirm(
            this.l('IncidentDeleteWarningMessage', employee.incidentContent),
            this.l('AreYouSure'),
            (isConfirmed)=> {
                if(isConfirmed) {
                    this._employeeIncidentServiceProxy
                        .deleteEmployeeIncident(employee.id)
                        .subscribe(()=> {
                            this.reloadPage();
                            this.notify.success(this.l('Successfully Deleted'));
                        });
                }
            }
        )
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    getIncidents(event?: LazyLoadEvent) {
        this.spinnerService.show();
        this.filters.incidentTypeId = Number((<HTMLInputElement>document.getElementById('employeeIncidentTypeId')).value) != 0 ? Number((<HTMLInputElement>document.getElementById('employeeIncidentTypeId')).value) : undefined;
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        this._employeeIncidentServiceProxy.getEmployeeIncident(
            undefined,this.filters.filter,this.filters.content,
            this.filters.incidentTypeId, undefined, this.filters.employeeName, 
            this.dateRange[0],
            this.dateRange[1].endOf('day'),
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.sortingColumns = this.primengTableHelper.getSorting(this.dataTable);
            this.spinnerService.hide();
        });
    }

    getEmployeeIncidentType(maxcount): void{
        this._employeeIncidentTypesServiceProxy.getAll(undefined, undefined, undefined, undefined, undefined, -1, undefined, undefined, maxcount)
                                              .subscribe(result =>{
                                                this.employeeIncident = result.items;
                                              });
    }
    exportToExcel(){
        this.spinnerService.show();
        this._employeeIncidentServiceProxy.getEmployeeIncidentsExcel(
            undefined,this.filters.filter,this.filters.content,
            this.filters.incidentTypeId, undefined, this.filters.employeeName, 
            this.dateRange[0],
            this.dateRange[1].endOf('day'),
            this.sortingColumns,
            undefined,
            undefined
        ).subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this._router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
     })
    }

    createOrEditEmployeeIncident(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        this._router.navigateByUrl(myurl);
    }
    
}