import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { LocationServiceProxy, UserLocationServiceProxy, PasswordComplexitySetting, CreateContactInput, AddressTypeServiceProxy, CreateEmployeeInput, EmployeesServiceProxy, CreatePostalCodeInput, PostalCodeServiceProxy
} from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { finalize } from 'rxjs/operators';
import * as moment from 'moment';
import * as _ from 'lodash';
import { Location } from '@angular/common';
import { ContactFormComponent } from '@app/shared/layout/form/contact.form.component';
import { AddressFormComponent } from '@app/shared/layout/form/address-form.component';
import { BookmarkCollectionBaseApi } from 'devexpress-richedit/lib/base-api/collections/bookmark-collection';
declare var KTWizard: any;
declare var $: any;
declare var KTApp: any;
declare let swal: any;

@Component({
    templateUrl: './create-employee.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class CreateEmployeeComponent extends AppComponentBase {

    @ViewChild(ContactFormComponent, { static: false }) contactForm;
    @ViewChild(AddressFormComponent, { static: false }) addressForm;

    employee: CreateEmployeeInput = new CreateEmployeeInput();
    postalCodeInput: CreatePostalCodeInput = new CreatePostalCodeInput();

    setRandomPassword = true;
    userPasswordRepeat = '';
    passwordComplexityInfo = '';

    active = false;
    saving = false;
    id: any;

    contactInput: CreateContactInput = new CreateContactInput();
    filteredLocations: any;
    getUserLocationId: any;
    getAddressId: any;
    getLocationId: any;

    selectedAddressType: any;
    selectedCountry: any;
    selectedPostalCode: any;
    selectedState: any;
    selectedCity: any;
    filteredAddressType: any;

    canChangeUserName = true;
    isActive = true;
    passwordComplexitySetting: PasswordComplexitySetting = new PasswordComplexitySetting();
    sendActivationEmail = true;
    shouldChangePasswordOnNextLogin = true;
    isTwoFactorEnabled: boolean = this.setting.getBoolean('Abp.Zero.UserManagement.TwoFactorLogin.IsEnabled');
    isLockoutEnabled: boolean = this.setting.getBoolean('Abp.Zero.UserManagement.UserLockOut.IsEnabled');
    stateFeature: boolean;

    constructor(
        injector: Injector,
        private _locationService: LocationServiceProxy,
        private _addressTypeService: AddressTypeServiceProxy,
        private _postalCodeServiceProxy: PostalCodeServiceProxy,
        private router: Router,
        private route: ActivatedRoute,
        private _location: Location,
        private _employeeService: EmployeesServiceProxy
    ) {
        super(injector);
    }

    setPasswordComplexityInfo(): void {
        this.passwordComplexityInfo = '<ul>';
        if (this.passwordComplexitySetting.requireDigit) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireDigit_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireLowercase) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireLowercase_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireUppercase) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireUppercase_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireNonAlphanumeric) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireNonAlphanumeric_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requiredLength) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequiredLength_Hint', this.passwordComplexitySetting.requiredLength) + '</li>';
        }
        this.passwordComplexityInfo += '</ul>';
    }


    ngOnInit() {
        this.stateFeature = abp.features.isEnabled('App.StateFeature');
        $('.kt-select2').select2({
            placeholder: "Select.."
        });
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id")
        })

        $("#addressTypeSelectInput").change(function () {
            this.selectedAddressType = $("#addressTypeSelectInput option:selected").text();
            $("#selectedAddressType").text(this.selectedAddressType);
        });

        $("#countrySelectInput").change(function () {
            this.selectedCountry = $("#countrySelectInput option:selected").text();
            $("#selectedCountry").text(this.selectedCountry + ",");
        });

        // $("#postalSelectInput").change(function () {
        //     this.selectedPostalCode = $("#postalSelectInput option:selected").text();
        //     $("#selectedPostalCode").text(this.selectedPostalCode + ",");
        // });

        // $("#stateSelectInput").change(function () {
        //     this.selectedState = $("#stateSelectInput option:selected").text();
        //     $("#selectedState").text(this.selectedState + ",");
        // });
        // $("#citySelectInput").change(function () {
        //     this.selectedCity = $("#citySelectInput option:selected").text();
        //     $("#selectedCity").text(this.selectedCity + ",");
        // });
        $("#locationSelectInput").change(function () {
            this.selectedCity = $("#locationSelectInput option:selected").text();
            $("#locationSelected").text(this.selectedCity + "");
        });

        var KTWizard2 = function () {
            var e, r, i;
            return {
                init: function () {
                    var t; KTUtil.get("kt_wizard_v2"), e = $("#kt_form"),
                        (i = new KTWizard("kt_wizard_v2",
                            { startStep: 1, clickableSteps: !0 })).on("beforeNext", function (e) { !0 !== r.form() && e.stop() }),
                        i.on("beforePrev", function (e) { !0 !== r.form() && e.stop() }), i.on("change", function (e) { KTUtil.scrollTop() }),
                        r = e.validate({
                            ignore: ":hidden",
                            rules: {
                                Name: { required: !0 },
                                Surname: { required: !0 },
                                EmailAddress: { required: !0, email: !0 },
                                locationSelectInput: { required: !0 },
                                UserName: { required: !0 },
                                Address1: { required: !0 },
                                postalSelectInput: { required: !0 }
                            },
                            invalidHandler: function (e, r) {
                            },
                            submitHandler: function (e) { }
                        })
                }
            }
        }();
        jQuery(document).ready(function () { KTWizard2.init() });
    }

    ngAfterViewInit(userId?: number) {
        let maxcount = 1000;
        this.filterLocations();
        this.filterAddressTypes(maxcount);
    }

    save(): void {
        this.saving = true;
        if (this.contactForm.pNumber == null) {
            this.contactForm.pNumber = "n/a";
        }
        this.employee.user.name = this.contactForm.fName;
        this.employee.user.surname = this.contactForm.sName;
        this.employee.user.phoneNumber = this.contactForm.pNumber;
        this.employee.user.emailAddress = this.contactForm.emailAddress;
        this.employee.user.shouldChangePasswordOnNextLogin = this.shouldChangePasswordOnNextLogin;
        this.employee.user.sendActivationEmail = this.sendActivationEmail;
        this.employee.setRandomPassword = this.setRandomPassword;
        this.employee.sendActivationEmail = this.sendActivationEmail;
        this.employee.user.isActive = this.isActive;
        this.employee.user.isLockoutEnabled = this.isLockoutEnabled;
        this.employee.user.isTwoFactorEnabled = this.isTwoFactorEnabled;
        this.employee.location = $("#locationSelectInput").val();

        this.employee.address = this.addressForm.address;
        this.employee.address.formattedAddress = this.addressForm.formattedAddress;
        this.employee.latitude = this.addressForm.latitude;
        this.employee.longitude = this.addressForm.longitude;
        // this.employee.address.postalCodeId = Number((<HTMLInputElement>document.getElementById('postalSelectInput')).value);
        this.employee.address.addressTypeId = Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value) == 0
            ? null : Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value);

        this.contactInput.lastName = this.contactForm.sName;
        this.contactInput.firstName = this.contactForm.fName;
        this.employee.contact = this.contactInput;
        this.employee.startDate = this.employee.startDate ? moment.utc(this.employee.startDate) : null;

        this.postalCodeInput.city = this.addressForm.city;
        
        // this.postalCodeInput.state = this.addressForm.state;
        if(!this.stateFeature){
            this.postalCodeInput.state = "";    
        }
        else{
            this.postalCodeInput.state = this.addressForm.state;
        }
        this.postalCodeInput.value = this.addressForm.value;
        this.postalCodeInput.country = this.addressForm.country;
        this._postalCodeServiceProxy.createPostalCode(this.postalCodeInput)
                                            .pipe()
                                            .subscribe(result => {
                                                this.employee.address.postalCodeId = result
            this._employeeService.createOrUpdateEmployee(this.employee).subscribe(
                () => {
                    this.saving = false;
                    this.notify.info(this.l('SavedSuccessfully'));
                    this.router.navigate(['/app/sprintship/employees']);
            });
        });
    }

    filterLocations(): void {
        this._locationService.getLocationNames(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            this.filteredLocations = result;
        });
    }

    filterAddressTypes(maxCount): void {
        this._addressTypeService.getAddressType(
            undefined,
            undefined,
            undefined,
            undefined,
            maxCount,
            undefined
        ).subscribe(result => {
            this.filteredAddressType = result.items;
        });
    }

    goBack() {
        this._location.back();
    }
}
