import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { LocationServiceProxy, UserLocationServiceProxy, PasswordComplexitySetting, CreateContactInput, AddressTypeServiceProxy, CreateEmployeeInput, EmployeesServiceProxy, GetEmployeeForEditOutput, AddressListDto, UpdateAddressInput, CreatePostalCodeInput, PostalCodeServiceProxy
} from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { finalize } from 'rxjs/operators';
import * as moment from 'moment';
import * as _ from 'lodash';
import { Location } from '@angular/common';
import { ContactFormComponent } from '@app/shared/layout/form/contact.form.component';
import { AddressFormComponent } from '@app/shared/layout/form/address-form.component';
import { NgFormSelectorWarning } from '@angular/forms';
declare var KTWizard: any;
declare var $: any;
declare var KTApp: any;
declare let swal: any;

@Component({
    templateUrl: './edit-employee.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class EditEmployeeComponent extends AppComponentBase {

    @ViewChild(ContactFormComponent, { static: false }) contactForm;
    @ViewChild(AddressFormComponent, { static: false }) addressForm;

    employee: CreateEmployeeInput = new CreateEmployeeInput();
    postalCodeInput: CreatePostalCodeInput = new CreatePostalCodeInput();

    setRandomPassword = false;
    userPasswordRepeat = '';
    passwordComplexityInfo = '';

    active = false;
    saving = false;
    id: number = null;

    contactInput: CreateContactInput = new CreateContactInput();
    vEmployee: GetEmployeeForEditOutput = new GetEmployeeForEditOutput();
    address: UpdateAddressInput = new UpdateAddressInput();

    filteredLocations: any;
    getUserLocationId: any;
    getAddressId: any;
    getLocationId: any;

    selectedAddressType: any;
    selectedCountry: any;
    selectedPostalCode: any;
    selectedState: any;
    selectedCity: any;
    filteredAddressType: any;

    canChangeUserName = true;
    isActive = true;
    passwordComplexitySetting: PasswordComplexitySetting = new PasswordComplexitySetting();
    sendActivationEmail = true;
    shouldChangePasswordOnNextLogin = false;

    firstName: string;
    surName: string;
    email: string;
    phoneNumber: string;
    userName: string;
    password: string;
    postalCodeId: number;
    locationSelected: any;
    isTwoFactorEnabled: boolean = this.setting.getBoolean('Abp.Zero.UserManagement.TwoFactorLogin.IsEnabled');
    isLockoutEnabled: boolean = this.setting.getBoolean('Abp.Zero.UserManagement.UserLockOut.IsEnabled');
    stateFeature: boolean;
    countrySelected: string;
    userId: number;
    // IdNumber: string;
    // StartDate: moment.Moment = moment();
    // HourlyRate: number=0.0;
    // DailyRate: number=0.0;
    constructor(
        injector: Injector,
        private _locationService: LocationServiceProxy,
        private _addressTypeService: AddressTypeServiceProxy,
        private router: Router,
        private route: ActivatedRoute,
        private _location: Location,
        private _employeeService: EmployeesServiceProxy,
        private _postalCodeServiceProxy: PostalCodeServiceProxy
    ) {
        super(injector);
    }

    setPasswordComplexityInfo(): void {
        this.passwordComplexityInfo = '<ul>';
        if (this.passwordComplexitySetting.requireDigit) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireDigit_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireLowercase) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireLowercase_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireUppercase) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireUppercase_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requireNonAlphanumeric) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequireNonAlphanumeric_Hint') + '</li>';
        }
        if (this.passwordComplexitySetting.requiredLength) {
            this.passwordComplexityInfo += '<li>' + this.l('PasswordComplexity_RequiredLength_Hint', this.passwordComplexitySetting.requiredLength) + '</li>';
        }
        this.passwordComplexityInfo += '</ul>';
    }


    ngOnInit() {
        this.stateFeature = abp.features.isEnabled('App.StateFeature');
        $('.kt-select2').select2({
            placeholder: "Select.."
        });
        this.route.paramMap.subscribe(params => {
            this.id = Number(params.get("id"))
        });
        this.initial();
        this.getEmployeeForEdit();
        this.locationSelected = $("#locationSelectInput option:selected").text();
        // this.selectedCountry = $("#countrySelectInput option:selected").text();
        this.setRandomPassword = false;
    }

    ngAfterViewInit(userId?: number) {
        let maxcount = 1000;
        
        this.filterAddressTypes(maxcount);
        this.locationSelected = $("#locationSelectInput option:selected").text();
        // this.selectedCountry = $("#countrySelectInput option:selected").text();
    }

    getEmployeeForEdit(){
        this.spinnerService.show();
        this._employeeService.getEmployeeForEdit(this.id).subscribe(result => {
            this.vEmployee = result;
            if(this.vEmployee.contact==null)
            {
                this.selectedCountry = "";
                this.selectedPostalCode = "";
                this.selectedState = "";
                this.selectedCity = "";
                this.selectedAddressType = "";
            }else{
                this.selectedCountry =  this.vEmployee.contact.address.postalCode.country;
                // this.selectedPostalCode =  this.vEmployee.contact.address.postalCode.value;
                // this.selectedState =  this.vEmployee.contact.address.postalCode.state;
                // this.selectedCity =  this.vEmployee.contact.address.postalCode.city;
                this.addressForm.formattedAddress = this.vEmployee.contact.address.formattedAddress;

                this.addressForm.country = this.vEmployee.contact.address.postalCode.country;
                this.addressForm.value = this.vEmployee.contact.address.postalCode.value;
                this.addressForm.state = this.vEmployee.contact.address.postalCode.state;
                
                this.addressForm.city = this.vEmployee.contact.address.postalCode.city;
                this.addressForm.latitude = this.vEmployee.contact.address.geocode.latitude;
                this.addressForm.longitude = this.vEmployee.contact.address.geocode.longitude;
                this.addressForm.address.addressLine1 = this.vEmployee.contact.address.addressLine1;
                this.addressForm.address.addressLine2 = this.vEmployee.contact.address.addressLine2;

                this.selectedAddressType = this.vEmployee.contact.address.addressType ? this.vEmployee.contact.address.addressType.name : "";
                //this.address.addressLine1 = this.vEmployee.contact.address.addressLine1;
                //this.address.addressLine2 = this.vEmployee.contact.address.addressLine2;
                this.address.addressTypeId = this.vEmployee.contact.address.addressType ? this.vEmployee.contact.address.addressType.id : null;
                this.postalCodeId = this.vEmployee.contact.address.postalCode.id;

            }
            if(this.vEmployee.employee!=null)
            {
                this.employee.idNumber = this.vEmployee.employee.idNumber;
                this.employee.startDate =this.vEmployee.employee.startDate ? moment.tz(this.vEmployee.employee.startDate, localStorage.getItem('timeZoneId')) : null;
                this.employee.hourlyRate = this.vEmployee.employee.hourlyRate;
                this.employee.dailyRate = this.vEmployee.employee.dailyRate;
            }
            this.firstName = this.vEmployee.user.name;
            this.surName = this.vEmployee.user.surname;
            this.email = this.vEmployee.user.emailAddress;
            this.phoneNumber = this.vEmployee.user.phoneNumber;
            this.userName = this.vEmployee.user.userName;
            this.password = this.vEmployee.user.password;
            this.isActive = this.vEmployee.user.isActive;
            this.shouldChangePasswordOnNextLogin = result.user.shouldChangePasswordOnNextLogin;
            this.sendActivationEmail = result.user.sendActivationEmail;
            this.userId = this.vEmployee.user.id
            setTimeout(function(){ 
                $('.kt-select2').select2();
                
                this.locationSelected = $("#locationSelectInput option:selected").text();
                $("#locationSelected").text(this.locationSelected);

                this.countrySelected = $("#countrySelectInput option:selected").text();
                $("#selectedCountry").text(this.countrySelected + ",");
                // console.log('trigerred')
            }, 1500);
            this.filterLocations();
            this.spinnerService.hide();
        });
    }

    save(): void {
        this.saving = true;
        if (this.contactForm.pNumber == null) {
            this.contactForm.pNumber = "n/a";
        }
        this.employee.user.id = this.vEmployee.user.id;
        this.employee.user.userName = this.userName;
        this.employee.user.password = this.password;
        this.employee.user.name = this.contactForm.fName;
        this.employee.user.surname = this.contactForm.sName;
        this.employee.user.phoneNumber = this.contactForm.pNumber;
        this.employee.user.emailAddress = this.contactForm.emailAddress;
        this.employee.user.shouldChangePasswordOnNextLogin = this.shouldChangePasswordOnNextLogin;
        this.employee.user.sendActivationEmail = this.sendActivationEmail;
        this.employee.setRandomPassword = this.setRandomPassword;
        this.employee.sendActivationEmail = this.sendActivationEmail;
        this.employee.user.isActive = this.isActive;
        this.employee.user.isLockoutEnabled = this.isLockoutEnabled;
        this.employee.user.isTwoFactorEnabled = this.isTwoFactorEnabled;
        this.employee.location = $("#locationSelectInput").val();

        this.employee.address = this.addressForm.address;
        this.employee.longitude = this.addressForm.longitude;
        this.employee.latitude = this.addressForm.latitude;
        this.employee.address.formattedAddress = this.addressForm.formattedAddress;
        if(this.vEmployee.contact!=null) {
            this.employee.address.id = this.vEmployee.contact.address.id;
        }
        // this.employee.address.postalCodeId = Number((<HTMLInputElement>document.getElementById('postalSelectInput')).value);
        this.employee.address.addressTypeId = Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value) == 0
            ? null : Number((<HTMLInputElement>document.getElementById('addressTypeSelectInput')).value);
        this.employee.address.formattedAddress = this.address.formattedAddress;
        this.contactInput.lastName = this.contactForm.sName;
        this.contactInput.firstName = this.contactForm.fName;
        this.employee.contact = this.contactInput;
        this.employee.contactId = this.vEmployee.contact==null ? null : this.vEmployee.contact.id;

        this.employee.startDate = this.employee.startDate ? moment.utc(this.employee.startDate) : null;

        this.postalCodeInput.city = this.addressForm.city;
        // this.postalCodeInput.state = this.addressForm.state;
        if(!this.stateFeature){
            this.postalCodeInput.state = "";    
        }
        else{
            this.postalCodeInput.state = this.addressForm.state;
        }
        this.postalCodeInput.value = this.addressForm.value;
        this.postalCodeInput.country = this.addressForm.country;

        if(this.postalCodeInput.city == null &&
            this.postalCodeInput.country == null && 
            this.postalCodeInput.state == null&&
            this.postalCodeInput.value == null){
             this.saving = false
             this.message.warn(
                //  "Please Add Address"                 
                 this.l("AddressValidation")
             );
            }
         else if(this.employee.location == null || 
             this.employee.location.length <= 0){
             this.saving = false
             this.message.warn(
                //  "Please Add Location"                 
                 this.l("LocationValidation")
             );
         }else if(this.employee.user.emailAddress==null || this.employee.user.emailAddress==""){
            this.saving = false
            this.message.warn(
               //  "Please Add Location"                 
                this.l("EmailValidation")
            );
         }else{
            this._postalCodeServiceProxy.createPostalCode(this.postalCodeInput)
                                            .pipe()
                                            .subscribe(result => {
                                                this.employee.address.postalCodeId = result
            this._employeeService.createOrUpdateEmployee(this.employee)
                .pipe(finalize(()=>{ this.saving = false; }))
                .subscribe(
                () => {
                    this.saving = false;
                    this.notify.info(this.l('SavedSuccessfully'));
                    this.router.navigate(['/app/sprintship/employees']);
                });
            });
         }
        
    }

    filterLocations(): void {
        this._locationService.getLocationNames(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            let key, key2, arrAny = {}, arrHolder = [];
            for (key in result) {
                arrAny = {
                    'name': result[key].name,
                    'id': result[key].id,
                    'selected': false
                };
                for (key2 in this.vEmployee.locationId) {
                    if (this.vEmployee.locationId[key2] == result[key].id) {
                        arrAny = {
                            'name': result[key].name,
                            'id': result[key].id,
                            'selected': true
                        };
                    }
                }
                arrHolder.push(arrAny);
            }
            this.filteredLocations = arrHolder;
        });
    }

    filterAddressTypes(maxCount): void {
        this._addressTypeService.getAddressType(
            undefined,
            undefined,
            undefined,
            undefined,
            maxCount,
            undefined
        ).subscribe(result => {
            this.filteredAddressType = result.items;
        });
    }

    goBack() {
        this._location.back();
    }

    initial(){
        $("#addressTypeSelectInput").change(function () {
            this.selectedAddressType = $("#addressTypeSelectInput option:selected").text();
            $("#selectedAddressType").text(this.selectedAddressType);
        });

        $("#countrySelectInput").change(function () {
            this.selectedCountry = $("#countrySelectInput option:selected").text();
            $("#selectedCountry").text(this.selectedCountry + ",");
        });

        $("#postalSelectInput").change(function () {
            this.selectedPostalCode = $("#postalSelectInput option:selected").text();
            $("#selectedPostalCode").text(this.selectedPostalCode + ",");
        });

        $("#stateSelectInput").change(function () {
            this.selectedState = $("#stateSelectInput option:selected").text();
            $("#selectedState").text(this.selectedState + ",");
        });
        $("#citySelectInput").change(function () {
            this.selectedCity = $("#citySelectInput option:selected").text();
            $("#selectedCity").text(this.selectedCity + ",");
        });
        $("#locationSelectInput").change(function () {
            this.locationSelected = $("#locationSelectInput option:selected").text();
            $("#locationSelected").text(this.locationSelected);
        });

        var KTWizard2 = function () {
            var e, r, i;
            return {
                init: function () {
                    var t; KTUtil.get("kt_wizard_v2"), e = $("#kt_form"),
                        (i = new KTWizard("kt_wizard_v2",
                            { startStep: 1, clickableSteps: !0 })).on("beforeNext", function (e) { !0 !== r.form() && e.stop() }),
                        i.on("beforePrev", function (e) { !0 !== r.form() && e.stop() }), i.on("change", function (e) { KTUtil.scrollTop() }),
                        r = e.validate({
                            ignore: ":hidden",
                            rules: {
                                Name: { required: !0 },
                                Surname: { required: !0 },
                                EmailAddress: { required: !0, email: !0 },
                                locationSelectInput: { required: !0 },
                                UserName: { required: !0 },
                                Address1: { required: !0 },
                                postalSelectInput: { required: !0 },
                            },
                            invalidHandler: function (e, r) {
                            },
                            submitHandler: function (e) { }
                        })
                }
            }
        }();
        jQuery(document).ready(function () { KTWizard2.init() });
    }
}
