import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { EntityDtoOfInt64, ContactServiceProxy, ContactListDto, PostalCodeServiceProxy, AddressTypeServiceProxy, DriversServiceProxy, UserServiceProxy, UserListDto, GetUserIdInputForResetPassword, EmployeesServiceProxy, LocationServiceProxy, GetEmployeeListDto } from '@shared/service-proxies/service-proxies';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { FileUpload } from 'primeng/fileupload';
import { finalize } from 'rxjs/operators';
import * as moment from 'moment-timezone';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    templateUrl: './employees.component.html',
    encapsulation: ViewEncapsulation.None,
    styleUrls: ['./employees.component.less'],
    animations: [appModuleAnimation()]
})
export class EmployeesComponent extends AppComponentBase {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Authorization.Users.User';
    entityHistoryEnabled = false;

    //Filters
    filters: any;
    nameFilter: string
    surnameFilter: string
    emailAddressFilter: string
    advancedFiltersAreShown = false;
    saving: boolean
    completed: boolean = false;
    uploadUrl: string;
    onlyLockedUsers: boolean =false;
    isEmailConfirmed: boolean =false;
    utcNow: Date = moment.utc().toDate();
    emailCompletedFilter : boolean = false;
    locationIds: any;
    IdNumber: string;
    filteredLocations: any;
    isLockoutEnabled: boolean = this.setting.getBoolean('Abp.Zero.UserManagement.UserLockOut.IsEnabled');
    constructor(
        injector: Injector,
        private _employeesService: EmployeesServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private _activatedRoute: ActivatedRoute,
        private _httpClient: HttpClient,
        private _router: Router,
        private _userServiceProxy: UserServiceProxy,
        private _locationServiceProxy: LocationServiceProxy,
        private router: Router,

    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/Import/ImportFromExcel';
    }

    ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
        $('.kt-select2').select2();
        this.filterLocations();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetEmployeeListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Employee - " +role.name +" " +role.surname
        });
    }

    getContact(event?: LazyLoadEvent) {
        this.spinnerService.show();
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }
        this.spinnerService.show();
        this.locationIds = $("#locationFilter").val();
        this._employeesService.getEmployees(
            this.filters,
            this.emailAddressFilter,
            this.nameFilter,
            this.surnameFilter,
            this.emailCompletedFilter,
            this.onlyLockedUsers,
            this.locationIds,
            this.IdNumber,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteContact(driver: UserListDto): void {

        this.message.confirm(
            this.l('DriverDeleteWarningMessage', driver.name + " " + driver.surname),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._employeesService.deleteEmployee(driver.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    createEmployee(url: any, id?: number) {
        if (id != null) {
            var myurl = `${url}/${id}`;
        } else {
            var myurl = `${url}`;
        }
        // console.log(myurl)
        this._router.navigateByUrl(myurl);
    }

    unlockUser(record): void {
        this._employeesService.unlockEmployee(new EntityDtoOfInt64({ id: record.id })).subscribe(() => {
            this.notify.success(this.l('UnlockedTheUser', record.userName));
        });
        this.filters = "";
        this.nameFilter = "";
        this.surnameFilter = "";
        this.emailAddressFilter = "";
        var that = this;
        // that.getContact()
        setTimeout(function () {
            that.getContact()
        }, 1000)
    }
    lockUser(record): void {
        this._employeesService.lockEmployee(new EntityDtoOfInt64({ id: record.id })).subscribe(() => {
            this.notify.success(this.l('LockedUser') + " " + record.userName);
            this.filters = "";
            this.nameFilter = "";
            this.surnameFilter = "";
            this.emailAddressFilter = "";
            var that = this;
            // that.getContact()
            setTimeout(function () {
                that.getContact()
            }, 500)

        });


    }
    resetPassword(userId) {
        let input = new GetUserIdInputForResetPassword
        input.userId = userId
        var that = this;
        this.spinnerService.show
        this._employeesService.updateResetUserPassword(input)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(() => {
                this.notify.info(this.l('ResetPasswordSuccess'));
                this.spinnerService.hide
                this.filters = "";
                this.nameFilter = "";
                this.surnameFilter = "";
                this.emailAddressFilter = "";
                setTimeout(function () {
                    that.getContact()
                }, 200)
            });
    }

    exportToExcel(){
        this.spinnerService.show();
        this._employeesService.getEmployeesToExcel(
            this.filters,
            this.emailAddressFilter,
            this.nameFilter,
            this.surnameFilter,
            this.emailCompletedFilter,
            this.onlyLockedUsers,
            this.locationIds,
            this.IdNumber,
            undefined,
            undefined, 
            undefined
        ).subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
     })
    }

    setLocked(record):void {
        if(record.lockOutEndDateUtc > this.utcNow){
            this.unlockUser(record);
        }else{
            this.lockUser(record);
        }
    }

    setEmailConfirm(record): void {
        this._employeesService.employeeEmailConfirm(new EntityDtoOfInt64({ id: record.id })).subscribe(() => {
            this.notify.success(this.l('EmailConfirm', record.name +" " +record.surname));
        });
        this.filters = "" ;
        this.nameFilter = "";
        this.surnameFilter = "";
        this.emailAddressFilter = "";
        var that = this;
        setTimeout(function () {
            that.getContact()
        }, 1000)
    }

    uploadExcel(data: { files: File }): void {
        const formData: FormData = new FormData();
        const file = data.files[0];
        formData.append('file', file, file.name);
        formData.append('key','employee');
        this._httpClient
            .post<any>(this.uploadUrl, formData)
            .subscribe(response => {
                if (response.success) {
                    this.notify.success('Import process is queued. Please wait');
                } else if (response.error != null) {
                    this.notify.error(this.l('ImportFailed'));
                }
            });
    }

    filterLocations(): void {
        this._locationServiceProxy.getLocationNames(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            this.filteredLocations = result;
            console.log(this.filteredLocations);
        });
    }
}
