import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TimeSheetServiceProxy, TimeSheetListDto } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { CreateOrEditTimeSheetModalComponent } from './create-or-edit-timesheet.component';
import { ViewTimeSheetModalComponent } from './view-timesheet.component';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';

@Component({
    templateUrl: './timesheet.component.html',
    animations: [appModuleAnimation()],
    encapsulation: ViewEncapsulation.None
})

export class TimeSheetComponent extends AppComponentBase {  

   @ViewChild('createOrEditTimeSheetModal', {static: true}) createOrEditTimeSheetModal: CreateOrEditTimeSheetModalComponent;
   @ViewChild('viewTimeSheetModalComponent', { static: true }) viewTimeSheetModal: ViewTimeSheetModalComponent;
   @ViewChild('dataTable', {static: true}) dataTable: Table;
   @ViewChild('paginator', {static: true}) paginator: Paginator;
   @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
   _entityTypeFullName = 'SprintTek.Shipping.TimeSheets.TimeSheet';
   entityHistoryEnabled = false;

   filters: {
        id: number;
        driver: string;
        employeeFilter: string;
        clockInDateFilter: any;
        clockInString: string;
        clockOutDateFilter: any;
    } = <any>{};
    advancedFiltersAreShown = false;
    sampleDate: Date;

   constructor(
      injector: Injector,
      private _timeSheetServiceProxy: TimeSheetServiceProxy
   ) {
      super(injector);  
   }

   ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
   }

   private setIsEntityHistoryEnabled(): void {
    let customSettings = (abp as any).custom;
    this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
}

showHistory(role: TimeSheetListDto): void {
    var display = "n/a";
    if(role.user != undefined) {
      var display = role.user.name +" " +role.user.surname
    }
    this.entityTypeHistoryModal.show({
        entityId: role.id.toString(),
        entityTypeFullName: this._entityTypeFullName,
        entityTypeDescription: "Timesheet - " + display
    });
}

   create() {
      this.createOrEditTimeSheetModal.show();
   }

   update(id: number): void {
      this.createOrEditTimeSheetModal.show(id);
   }

   delete(timesheet: TimeSheetListDto): void {

      this.message.confirm(
         this.l('TimeSheetDeleteWarningMessage'),
         this.l('AreYouSure'),
         (isConfirmed)=> {
             if(isConfirmed) {
                 this._timeSheetServiceProxy
                     .deleteTimeSheet(timesheet.id)
                     .subscribe(()=> {
                         this.reloadPage();
                         this.notify.success(this.l('SuccessfullyDeleted'));
                     });
             }
         }
     )
   }

   reloadPage(): void {
      this.paginator.changePage(this.paginator.getPage());
   }

   getInDate(event){
       
       if(event != null){
        let inDate = new Date(event).toLocaleDateString()
        
        this.filters.clockInDateFilter = inDate
       }
       else{
        let inDate = undefined
       
        this.filters.clockInDateFilter = inDate
       }
       
    //    .toLocaleDateString()
       
   }

   getOutDate(event){
    if(event != null){
        let outDate = new Date(event).toLocaleDateString()
        this.filters.clockOutDateFilter = outDate
    }
    else{
        let outDate = undefined
            this.filters.clockOutDateFilter = outDate
    }
    
    }

   getData(event?: LazyLoadEvent) {
    // this.filters.clockInDateFilter = (<HTMLInputElement>document.getElementById("timesheetClockIn")).value != null ? 
    //                                     new Date((<HTMLInputElement>document.getElementById("timesheetClockIn")).value) :
    //                                     undefined
    // this.filters.clockOutDateFilter = (<HTMLInputElement>document.getElementById("timesheetClockOut")).value != null ?
    //                                     new Date((<HTMLInputElement>document.getElementById("timesheetClockOut")).value) :
    //                                     undefined
    
    this.spinnerService.show();

      if (this.primengTableHelper.shouldResetPaging(event)) {
          this.paginator.changePage(0);
          return;
      }

      this.spinnerService.show();
      this._timeSheetServiceProxy.getTimeSheet(
          undefined, 
          undefined,
          undefined,
          undefined,
          this.filters.driver,
          this.filters.employeeFilter,
          this.filters.clockInDateFilter,
          this.filters.clockOutDateFilter,
          this.primengTableHelper.getSorting(this.dataTable),
          this.primengTableHelper.getMaxResultCount(this.paginator, event),
          this.primengTableHelper.getSkipCount(this.paginator, event)
      ).subscribe(result => {
          this.primengTableHelper.totalRecordsCount = result.totalCount;
          this.primengTableHelper.records = result.items;
        //   console.log(result.items);
          this.spinnerService.hide();
      });
  }
}