﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { DriverIncidentTypesServiceProxy, CreateOrEditDriverIncidentTypeDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditDriverIncidentTypeModal',
    templateUrl: './create-or-edit-driverIncidentType-modal.component.html'
})
export class CreateOrEditDriverIncidentTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    driverIncidentType: CreateOrEditDriverIncidentTypeDto = new CreateOrEditDriverIncidentTypeDto();



    constructor(
        injector: Injector,
        private _driverIncidentTypesServiceProxy: DriverIncidentTypesServiceProxy
    ) {
        super(injector);
    }

    show(driverIncidentTypeId?: number): void {

        if (!driverIncidentTypeId) {
            this.driverIncidentType = new CreateOrEditDriverIncidentTypeDto();
            this.driverIncidentType.id = driverIncidentTypeId;

            this.active = true;
            this.modal.show();
        } else {
            this._driverIncidentTypesServiceProxy.getDriverIncidentTypeForEdit(driverIncidentTypeId).subscribe(result => {
                this.driverIncidentType = result.driverIncidentType;


                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

			
            this._driverIncidentTypesServiceProxy.createOrEdit(this.driverIncidentType)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
