﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { DriverIncidentTypesServiceProxy, DriverIncidentTypeDto, GetDriverIncidentTypeForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditDriverIncidentTypeModalComponent } from './create-or-edit-driverIncidentType-modal.component';
import { ViewDriverIncidentTypeModalComponent } from './view-driverIncidentType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './driverIncidentTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class DriverIncidentTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditDriverIncidentTypeModal', { static: true }) createOrEditDriverIncidentTypeModal: CreateOrEditDriverIncidentTypeModalComponent;
    @ViewChild('viewDriverIncidentTypeModalComponent', { static: true }) viewDriverIncidentTypeModal: ViewDriverIncidentTypeModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Drivers.DriverIncidentType';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    nameFilter = '';
    codeFilter = '';
    maxSortOrderFilter : number;
		maxSortOrderFilterEmpty : number;
		minSortOrderFilter : number;
		minSortOrderFilterEmpty : number;
    isStaticFilter = -1;




    constructor(
        injector: Injector,
        private _driverIncidentTypesServiceProxy: DriverIncidentTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

  
    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetDriverIncidentTypeForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.driverIncidentType.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Driver Incident Type - " +role.driverIncidentType.name
        });
    }

    getDriverIncidentTypes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._driverIncidentTypesServiceProxy.getAll(
            this.filterText,
            this.nameFilter,
            this.codeFilter,
            this.maxSortOrderFilter == null ? this.maxSortOrderFilterEmpty: this.maxSortOrderFilter,
            this.minSortOrderFilter == null ? this.minSortOrderFilterEmpty: this.minSortOrderFilter,
            this.isStaticFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createDriverIncidentType(): void {
        this.createOrEditDriverIncidentTypeModal.show();
    }

    deleteDriverIncidentType(driverIncidentType: DriverIncidentTypeDto): void {
        this.message.confirm('',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._driverIncidentTypesServiceProxy.delete(driverIncidentType.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._driverIncidentTypesServiceProxy.getDriverIncidentTypesToExcel(
        this.filterText,
            this.nameFilter,
            this.codeFilter,
            this.maxSortOrderFilter == null ? this.maxSortOrderFilterEmpty: this.maxSortOrderFilter,
            this.minSortOrderFilter == null ? this.minSortOrderFilterEmpty: this.minSortOrderFilter,
            this.isStaticFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
