import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { EntityDtoOfInt64,  LocationServiceProxy, RateServiceProxy, RateDto, DriverRateDto, OrderSourceRate } from '@shared/service-proxies/service-proxies';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { FileUpload } from 'primeng/fileupload';
import { finalize } from 'rxjs/operators';
import * as moment from 'moment-timezone';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import { RatesDescriptionModalComponent } from './rates-description-modal.component';
import * as _ from 'lodash';

@Component({
    templateUrl: './billing.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class BillingComponent extends AppComponentBase {

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
   

    //Filters
    filters : any;
    advancedFiltersAreShown = false;
    saving : boolean
    completed: boolean = false;
    filteredLocations: any;
    locationIds: any;
    dateInput: any;
    date = new Date();
    loc: any;
    constructor(
        injector: Injector,
        private _rateService: RateServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _httpClient: HttpClient,
        private _router: Router,
        private _locationServiceProxy: LocationServiceProxy,
        private router: Router,

    ) {
        super(injector);
        
    }

    ngOnInit(){

        $('.kt-select2').select2({
            placeholder: "Select.." 
        });
        this.filterLocations();
        //this.locationIds = 69;

        let loadDate = localStorage.getItem('loadDate')
        this.setDate(loadDate);
        this.dateInput = loadDate;
        this.date = new Date(loadDate);
        
        let locId = localStorage.getItem('operatingLocationId');
        if(locId.includes(',')) {
            let a = locId.split(",");
            this.loc = a[0];
           
        }
        else {
            this.loc = locId;
        }
        $('#locationFilter').on('select2:select', function (e) {
        var data = e.params.data;
        console.log(data);
        });
        /*$('#locationFilter').on('select2:select', function (e) { 
            let data = e.params.data.id;
        
            this.locationIds = data;
        });*/


        
        
    }


    setDate(newDate) {
        //console.log(newDate);
        this.dateInput = newDate;
    }

    getRate(event?: LazyLoadEvent) {
        this.spinnerService.show();
       
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();
        if($("#locationFilter").val() != null) {
            this.locationIds = $("#locationFilter").val();
        }
        else {
            this.locationIds = this.loc;
        }
        this._rateService.getDriverRate(
            this.filters,
            this.locationIds,
            undefined,
            this.dateInput,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
            ).subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.totalCount;
                this.primengTableHelper.records = result.items;
                console.log(result.items);
                this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    /*deleteRate(rate: RateDto): void {

        this.message.confirm(
            this.l('Delete'),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._rateService.delete(rate.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }*/

   
    
    
    filterLocations(): void {
        this._locationServiceProxy.getLocationNames(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            this.filteredLocations = result;
            //this.locationIds = result[0].id;
        });
    }


}

