import { Component, Injector, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { Router } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { EntityDtoOfInt64, DriversServiceProxy, UserListDto, GetDriverDetailsDto } from '@shared/service-proxies/service-proxies';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { CreateDriverComponent } from './create-driver.component';
import { HttpClient } from '@angular/common/http';
import { CreateIncidentModalComponent } from '@app/sprintship/incident/create-incident-modal.component';
// import { CreateOrEditDriverIncidentModalComponent } from '@app/sprintship/employees/driver-incident/create-or-edit-driver-incident.component';
import { Location } from '@angular/common';
import * as moment from 'moment-timezone';
import { DriversPermissionsComponent } from '@app/sprintship/fleet-management/drivers/drivers-permissions.component';


@Component({
    templateUrl: './view-driver.component.html',
    encapsulation: ViewEncapsulation.None,
    // styleUrls: ['./drivers.component.less'],
    animations: [appModuleAnimation()]
})
export class ViewDriverCompnent extends AppComponentBase {

    // @ViewChild('viewFleetModalComponent', { static: true }) viewFleetModal: ViewFleetModalComponent;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    // @ViewChild('createOrEditDriverIncidentModal', {static: false}) createOrEditDriverIncidentModal: CreateOrEditDriverIncidentModalComponent;

    driverInfo: GetDriverDetailsDto = new GetDriverDetailsDto();
    
    id: any;
    driver: any;
    name: string;
    company: string;
    isActive: string;
    address: string;
    driverIncident:any;
    driverIncidentLen:number;
    timeCreated:any;
    phone: string;
    email: string;
    startDate: string;
    endDate: string;
    DlExpireDate: string;


    constructor(
        injector: Injector,
        private _driverService: DriversServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private _activatedRoute: ActivatedRoute,
        private _httpClient: HttpClient,
        private router: Router,
        private _router: Router,
        private route: ActivatedRoute,
        private _location:Location,
    ) {
        super(injector);
    }

    ngOnInit(){
        this.route.paramMap.subscribe(params => {
            this.id = params.get("id")
        });
        this._driverService.getDriverDetails(this.id).subscribe(result => {
            this.driverInfo = result;
            this.name = result.user.name +" " +result.user.surname;
            this.phone = result.user.phoneNumber;
            this.email = result.user.emailAddress;
            this.isActive = result.driverProfile ? result.driverProfile.isActive ? "Yes" : "No" : "";
            var addressLine2 = result.address.addressLine2==null ? "" : result.address.addressLine2 +", ";
            this.address = result.address.addressLine1 +", "+addressLine2
                            +result.address.postalCode.city +", "+result.address.postalCode.state
                            +", "+result.address.postalCode.value +", "+result.address.postalCode.country;
            this.driverIncident = result.driverIncident;
            this.driverIncidentLen = result.driverIncident.length;
            this.startDate = result.driverProfile ? (result.driverProfile.startDate ? moment(result.driverProfile.startDate).tz(localStorage.getItem('timeZoneId')).format('MM/DD/YYYY'): '') : '';
            this.endDate = result.driverProfile ? (result.driverProfile.endDate ? moment(result.driverProfile.endDate).tz(localStorage.getItem('timeZoneId')).format('MM/DD/YYYY'): '') :'';
            this.DlExpireDate = result.driverProfile ? (result.driverProfile.driverLicenseExpirationDate ? moment(result.driverProfile.driverLicenseExpirationDate).tz(localStorage.getItem('timeZoneId')).format('MM/DD/YYYY') : '') :'';

        });
        // this.getDriverDetails();

    }

    getDriverDetails(){
        
    }
    // create() {
    //     this.createOrEditDriverIncidentModal.show();
    // }

    viewDriver(v:boolean){
        this.ngOnInit();
        console.log("refresh");
    }

    createOrEditDriver(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        this._router.navigateByUrl(myurl);
    }

    goBack(){
        this._location.back();
    }


}
