import { Component, EventEmitter, Injector, Output, ViewChild } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { FindOrganizationUnitUsersInput, NameValueDto, OrganizationUnitServiceProxy, UsersToOrganizationUnitInput, FleetUserServiceProxy, CreateDriverToFleetInput, UserListDto } from '@shared/service-proxies/service-proxies';
import * as _ from 'lodash';
import { ModalDirective } from 'ngx-bootstrap';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IDriversWithFleet } from './drivers-with-fleet.component';

@Component({
    selector: 'fleetAddDriverComponent',
    templateUrl: './fleet-add-driver.component.html'
})
export class FleetAddDriverComponent extends AppComponentBase {

    fleetId: number;

    @Output() driverAdded: EventEmitter<IDriversWithFleet> = new EventEmitter<IDriversWithFleet>();

    @ViewChild('modal', {static: true}) modal: ModalDirective;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;

    isShown = false;
    filterText = '';
    tenantId?: number;
    saving = false;
    selectedVehicles: UserListDto[];

    constructor(
        injector: Injector,
        private _fleetUserService: FleetUserServiceProxy

    ) {
        super(injector);
    }

    show(): void {
        this.modal.show();
    }

    refreshTable(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    close(): void {
        this.modal.hide();
    }

    shown(): void {
        this.isShown = true;
        this.getRecordsIfNeeds(null);
    }

    getRecordsIfNeeds(event: LazyLoadEvent): void {
        if (!this.isShown) {
            return;
        }

        this.getRecords(event);
    }

    getRecords(event?: LazyLoadEvent): void {

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();
        
        this._fleetUserService.getUnassignedDrivers(
            this.filterText,
            undefined,
            undefined,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
            ).subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.totalCount;
                this.primengTableHelper.records = result.items;
                this.spinnerService.hide();
        });
    }

    addDriversToFleet(): void {
        const input = new CreateDriverToFleetInput();
        input.fleetId = this.fleetId;
        input.userIds = _.map(this.selectedVehicles, selectedVehicles => Number(selectedVehicles.id));
        this.saving = true;
        this._fleetUserService
            .createDriverToFleet(input)
            .subscribe(() => {
                this.notify.success(this.l('SuccessfullyAdded'));
                this.driverAdded.emit({
                    userIds: input.userIds,
                    ouId: input.fleetId
                });
                this.saving = false;
                this.close();
                this.selectedVehicles = [];
            });
    }
}
