import { Component, EventEmitter, Injector, Output, ViewChild } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { FindOrganizationUnitUsersInput, NameValueDto, OrganizationUnitServiceProxy, UsersToOrganizationUnitInput, FleetRateServiceProxy, FleetRateListDto, FleetUserServiceProxy, CreateRateToFleetInput, UserListDto } from '@shared/service-proxies/service-proxies';
import * as _ from 'lodash';
import { ModalDirective } from 'ngx-bootstrap';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IRatesWithFleet } from './rates-with-fleet.component';

@Component({
    selector: 'fleetAddRateComponent',
    templateUrl: './fleet-add-rate.component.html'
})
export class FleetAddRateComponent extends AppComponentBase {

    fleetId: number;

    @Output() rateAdded: EventEmitter<IRatesWithFleet> = new EventEmitter<IRatesWithFleet>();

    @ViewChild('modal', {static: true}) modal: ModalDirective;
    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;

    isShown = false;
    filterText = '';
    tenantId?: number;
    saving = false;
    selectedRates: FleetRateListDto[];
    fId: any;
    constructor(
        injector: Injector,
        private _fleetRateService: FleetRateServiceProxy

    ) {
        super(injector);
    }

    show(id?: number): void {
        this.fId = id;
        this.modal.show();
    }

    refreshTable(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    close(): void {
        this.modal.hide();
    }

    shown(): void {
        this.isShown = true;
        this.getRecordsIfNeeds(null);
    }

    getRecordsIfNeeds(event: LazyLoadEvent): void {
        if (!this.isShown) {
            return;
        }

        this.getRecords(event);
    }

    getRecords(event?: LazyLoadEvent): void {

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();
        
       this._fleetRateService.getRateForFleet(
            this.filterText,
            this.fId,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
            ).subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.totalCount;
                this.primengTableHelper.records = result.items;
                this.spinnerService.hide();
        });
    }

    addRatesToFleet(): void {
        const input = new  CreateRateToFleetInput();
        input.fleetId = this.fleetId;
        input.rateIds = _.map(this.selectedRates, selectedRates => Number(selectedRates.id));
        this.saving = true;
        this._fleetRateService
            .createRateToFleet(input)
            .subscribe(() => {
                this.notify.success(this.l('SuccessfullyAdded'));
                this.rateAdded.emit({
                    rateIds: input.rateIds,
                    ouId: input.fleetId
                });
                this.saving = false;
                this.close();
                this.selectedRates = [];
            });
    }
}
