import { Component, EventEmitter, Injector, Output, ViewChild } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { FindOrganizationUnitUsersInput, NameValueDto, OrganizationUnitServiceProxy, UsersToOrganizationUnitInput, FleetVehicleServiceProxy, CreateVehicleToFleetInput, GetUnassignedVehicleListDto } from '@shared/service-proxies/service-proxies';
import * as _ from 'lodash';
import { ModalDirective } from 'ngx-bootstrap';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { IVehiclesWithFleet } from './vehicles-with-fleet.component';

@Component({
    selector: 'fleetAddVehicleComponent',
    templateUrl: './fleet-add-vehicle.component.html'
})
export class FleetAddVehicleComponent extends AppComponentBase {

    fleetId: number;

    @Output() vehicleAdded: EventEmitter<IVehiclesWithFleet> = new EventEmitter<IVehiclesWithFleet>();

    @ViewChild('modal', { static: true }) modal: ModalDirective;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    isShown = false;
    filterText = '';
    tenantId?: number;
    saving = false;
    selectedVehicles: GetUnassignedVehicleListDto[];

    constructor(
        injector: Injector,
        private _organizationUnitService: OrganizationUnitServiceProxy,
        private _fleetVehicleService: FleetVehicleServiceProxy


    ) {
        super(injector);
    }

    show(): void {
        this.modal.show();
    }

    refreshTable(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    close(): void {
        this.modal.hide();
    }

    shown(): void {
        this.isShown = true;
        this.getRecordsIfNeeds(null);
    }

    getRecordsIfNeeds(event: LazyLoadEvent): void {
        if (!this.isShown) {
            return;
        }

        this.getRecords(event);
    }

    getRecords(event?: LazyLoadEvent): void {

        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();

        this._fleetVehicleService.getUnassignedVehicle(
                this.filterText,
                this.primengTableHelper.getSorting(this.dataTable),
                this.primengTableHelper.getMaxResultCount(this.paginator, event),
                this.primengTableHelper.getSkipCount(this.paginator, event)
            )
            .pipe(finalize(() => this.spinnerService.hide()))
            .subscribe(result => {
                this.primengTableHelper.totalRecordsCount = result.items.length;
                this.primengTableHelper.records = result.items;
                this.spinnerService.hide();
            });
    }

    addVehicleToFleet(): void {
        const input = new CreateVehicleToFleetInput();
        input.fleetId = this.fleetId;
        input.vehicleIds = _.map(this.selectedVehicles, selectedVehicles => Number(selectedVehicles.id));
        this.saving = true;
        this._fleetVehicleService.createVehicleToFleet(input).subscribe(() => {
                this.notify.success(this.l('SuccessfullyAdded'));
                this.vehicleAdded.emit({
                    vehicleIds: input.vehicleIds,
                    ouId: input.fleetId
                });
                this.saving = false;
                this.close();
                this.selectedVehicles = [];
            });
    }
}
