import { Component, Injector, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { AppComponentBase } from '@shared/common/app-component-base';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { VehicleServiceProxy, VehicleListDto, CreateVehicleInput} from '@shared/service-proxies/service-proxies';
import * as jquery from 'jquery';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { ViewVehicleModalComponent } from '@app/sprintship/fleet-management/vehicle/view-vehicle-modal.component';
import { AppConsts } from '@shared/AppConsts';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import { Router } from '@angular/router';

@Component({
    templateUrl: './vehicles.component.html',
    encapsulation: ViewEncapsulation.None,
    styleUrls: ['./vehicles.component.less'],
    animations: [appModuleAnimation()]
})
export class VehiclesComponent extends AppComponentBase{

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('ViewVehicleModalComponent', {static: true}) viewVehicle: ViewVehicleModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Vehicles.Vehicle';
    entityHistoryEnabled = false;
    
    advancedFiltersAreShown = false;
    filters: {
        id: number;
        vehicleName: string;
    } = <any>{};

    newVehicle: CreateVehicleInput = null;
    vehicleFilter = '';
    licenseFilter = '';
    driverFilter = '';

    completed: boolean = false;
    uploadUrl: string;
    constructor(
        injector: Injector,
        private _vehicleService:VehicleServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private _httpClient: HttpClient,
        private router: Router,

    ) {
        super(injector);
        this.uploadUrl = AppConsts.remoteServiceBaseUrl + '/Import/ImportFromExcel';
    }

    ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: VehicleListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Vehicle - " +role.name
        });
    }
    exportToExcel(): void{
        this.spinnerService.show();
        this._vehicleService.getVehiclesToExcel(
            this.filters.vehicleName,
            this.vehicleFilter,
            this.licenseFilter,
            this.driverFilter
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
        })
    }
    
    getVehicle(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);

            return;
        }

        this.spinnerService.show();

        this._vehicleService.getVehiclePaged(
            this.filters.id,
            this.vehicleFilter,
            this.licenseFilter,
            this.filters.vehicleName,
            undefined,
            undefined,
            this.driverFilter,
            this.driverFilter,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),   
            this.primengTableHelper.getMaxResultCount(this.paginator, event),        
            this.primengTableHelper.getSkipCount(this.paginator, event),
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
            
        });

    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteVehicle(vehicle: VehicleListDto): void {
        this.message.confirm(
            this.l('VehicleDeleteWarningMessage', vehicle.name),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._vehicleService.deleteVehicle(vehicle.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    uploadExcel(data: { files: File }): void {
        
        //this.importRouteScheduleTracking.clearMessages();
        
        // this.importRouteScheduleTracking.modal.show();
        const formData: FormData = new FormData();
        const file = data.files[0];
        formData.append('file', file, file.name);
        formData.append('key','vehicles');
        // formData.append('entity','addresses','addresses');

        this._httpClient
            .post<any>(this.uploadUrl, formData)
            .subscribe(response => {
                if (response.success) {
                    this.notify.success(this.l('ImportStart'));
                } else if (response.error != null) {
                    this.notify.error(this.l('ImportFailed'));
                }
            });
    }
   
}

