﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { VehicleChecklistsServiceProxy, CreateOrEditVehicleChecklistDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditVehicleChecklistModal',
    templateUrl: './create-or-edit-vehicleChecklist-modal.component.html'
})
export class CreateOrEditVehicleChecklistModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    vehicleChecklist: CreateOrEditVehicleChecklistDto = new CreateOrEditVehicleChecklistDto();



    constructor(
        injector: Injector,
        private _vehicleChecklistsServiceProxy: VehicleChecklistsServiceProxy
    ) {
        super(injector);
    }

    show(vehicleChecklistId?: number): void {

        if (!vehicleChecklistId) {
            this.vehicleChecklist = new CreateOrEditVehicleChecklistDto();
            this.vehicleChecklist.id = vehicleChecklistId;
            this.vehicleChecklist.color = '#000000';
            this.active = true;
            this.modal.show();
        } else {
            this._vehicleChecklistsServiceProxy.getVehicleChecklistForEdit(vehicleChecklistId).subscribe(result => {
                this.vehicleChecklist = result.vehicleChecklist;
                

                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

            // if(this.vehicleChecklist.icon == ""){
            //     this.vehicleChecklist.icon = 'fas fa-car'
            // }
            
            if(this.vehicleChecklist.icon != undefined){
                if(this.vehicleChecklist.icon.indexOf('fas fa') >=0){
                    this.vehicleChecklist.icon=this.vehicleChecklist.icon
                }
                else if(this.vehicleChecklist.icon.indexOf('fas fa') < 0 && this.vehicleChecklist.icon != ''){
                    this.vehicleChecklist.icon='fas fa-'+this.vehicleChecklist.icon
                }
            }
            else{
                this.vehicleChecklist.icon = 'fas fa-car'
            }
                // else{
                //     this.vehicleChecklist.icon = 'fas fa-car'
                // }
            
           
			
            this._vehicleChecklistsServiceProxy.createOrEdit(this.vehicleChecklist)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
