﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { VehicleChecklistsServiceProxy, VehicleChecklistDto, GetVehicleChecklistForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditVehicleChecklistModalComponent } from './create-or-edit-vehicleChecklist-modal.component';
import { ViewVehicleChecklistModalComponent } from './view-vehicleChecklist-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './vehicleChecklists.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class VehicleChecklistsComponent extends AppComponentBase {

    @ViewChild('createOrEditVehicleChecklistModal', { static: true }) createOrEditVehicleChecklistModal: CreateOrEditVehicleChecklistModalComponent;
    @ViewChild('viewVehicleChecklistModalComponent', { static: true }) viewVehicleChecklistModal: ViewVehicleChecklistModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.VehicleChecklists.VehicleChecklist';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    colorFilter = '';
    iconFilter = '';
    isActiveFilter : boolean = true
    isRequiredFilter : boolean = true
    // isActiveFilter = -1;
    // isRequiredFilter = -1;




    constructor(
        injector: Injector,
        private _vehicleChecklistsServiceProxy: VehicleChecklistsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }
    
    ngOnInit(){
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetVehicleChecklistForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.vehicleChecklist.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Vehicle Checklist - " +role.vehicleChecklist.description
        });
    }

    getVehicleChecklists(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._vehicleChecklistsServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.colorFilter,
            this.iconFilter,
            this.isActiveFilter,
            this.isRequiredFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createVehicleChecklist(): void {
        this.createOrEditVehicleChecklistModal.show();
    }

    deleteVehicleChecklist(vehicleChecklist: VehicleChecklistDto): void {
        this.message.confirm(
            this.l(''),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._vehicleChecklistsServiceProxy.delete(vehicleChecklist.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._vehicleChecklistsServiceProxy.getVehicleChecklistsToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.colorFilter,
            this.iconFilter,
            this.isActiveFilter,
            this.isRequiredFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
