﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { VehicleTypesServiceProxy, CreateOrEditVehicleTypeDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditVehicleTypeModal',
    templateUrl: './create-or-edit-vehicleType-modal.component.html'
})
export class CreateOrEditVehicleTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    vehicleType: CreateOrEditVehicleTypeDto = new CreateOrEditVehicleTypeDto();



    constructor(
        injector: Injector,
        private _vehicleTypesServiceProxy: VehicleTypesServiceProxy
    ) {
        super(injector);
    }

    show(vehicleTypeId?: number): void {

        if (!vehicleTypeId) {
            this.vehicleType = new CreateOrEditVehicleTypeDto();
            this.vehicleType.id = vehicleTypeId;

            this.active = true;
            this.modal.show();
        } else {
            this._vehicleTypesServiceProxy.getVehicleTypeForEdit(vehicleTypeId).subscribe(result => {
                this.vehicleType = result.vehicleType;


                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

			
            this._vehicleTypesServiceProxy.createOrEdit(this.vehicleType)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }
    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
