﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { VehicleTypesServiceProxy, VehicleTypeDto, GetVehicleForViewDto, GetVehicleTypeForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditVehicleTypeModalComponent } from './create-or-edit-vehicleType-modal.component';
import { ViewVehicleTypeModalComponent } from './view-vehicleType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './vehicleTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class VehicleTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditVehicleTypeModal', { static: true }) createOrEditVehicleTypeModal: CreateOrEditVehicleTypeModalComponent;
    @ViewChild('viewVehicleTypeModalComponent', { static: true }) viewVehicleTypeModal: ViewVehicleTypeModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.VehicleType.VehicleTypes';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    descriptionFilter = '';
    codeFilter = '';




    constructor(
        injector: Injector,
        private _vehicleTypesServiceProxy: VehicleTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetVehicleTypeForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.vehicleType.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Vehicle Type - " +role.vehicleType.description
        });
    }

    getVehicleTypes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._vehicleTypesServiceProxy.getAll(
            this.filterText,
            this.descriptionFilter,
            this.codeFilter,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createVehicleType(): void {
        this.createOrEditVehicleTypeModal.show();
    }

    deleteVehicleType(vehicleType: VehicleTypeDto): void {
        this.message.confirm(
            '','Are you sure?',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._vehicleTypesServiceProxy.delete(vehicleType.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._vehicleTypesServiceProxy.getVehicleTypesToExcel(
        this.filterText,
            this.descriptionFilter,
            this.codeFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
