import { AfterViewChecked, Component, ElementRef, EventEmitter, Injector, Output, ViewChild } from '@angular/core';
import { AppConsts } from '@shared/AppConsts';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TagTypeServiceProxy, CreateTagInput, TagServiceProxy, TagTypeListDto, ShippingPackageListDto, OrderServiceProxy, ShippingPackageServiceProxy, CreateOrderPackageInput, OrderListDto, OrderPackageServiceProxy, IncidentTypesServiceProxy, CreateOrEditIncidentTypeDto} from '@shared/service-proxies/service-proxies';
import { ModalDirective } from 'ngx-bootstrap';
import * as _ from 'lodash';
import { finalize } from 'rxjs/operators';
import { OrderList } from 'primeng/primeng';

@Component({
    selector: 'createUpdateIncidentTypeModal',
    templateUrl: './create-update-incident-type.component.html'
})
export class CreateUpdateIncidentTypeComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', {static: false}) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    code: string;
    name: string;
    sort: number;
    id: number;
    createUpdate: CreateOrEditIncidentTypeDto = new CreateOrEditIncidentTypeDto();;
    constructor(
        injector: Injector,
        private _incidentTypeService: IncidentTypesServiceProxy,
    ) {
        super(injector);
    }

    show(id?): void {
        this.active = true;
        this.id = id;
        if(id){
            this._incidentTypeService.getIncidentTypeForView(id).subscribe(result=>{
                this.code = result.incidentType.code;
                this.name = result.incidentType.incidentType;
                this.sort = result.incidentType.sortOrder;
            })
        }
        this.modal.show();
    }

    save(): void {
        this.saving = true;
        this.createUpdate.id = this.id;
        this.createUpdate.incidentType = this.name;
        this.createUpdate.sortOrder = this.sort;
        this.createUpdate.code = this.code;
        console.log(this.createUpdate);
        this._incidentTypeService.createOrEdit(this.createUpdate)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.name = null;
                this.sort = null;
                this.code = null;
                this.close();
                this.modalSave.emit(null);
            });
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }

}
