import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { IncidentSourcesServiceProxy, CreateOrEditIncidentSourceDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditIncidentSourceModal',
    templateUrl: './create-or-edit-incidentSource-modal.component.html',
    providers: [IncidentSourcesServiceProxy]
})
export class CreateOrEditIncidentSourceModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    incidentSource: CreateOrEditIncidentSourceDto = new CreateOrEditIncidentSourceDto();



    constructor(
        injector: Injector,
        private _incidentSourcesServiceProxy: IncidentSourcesServiceProxy
    ) {
        super(injector);
    }

    show(incidentSourceId?: number): void {

        if (!incidentSourceId) {
            this.incidentSource = new CreateOrEditIncidentSourceDto();
            this.incidentSource.id = incidentSourceId;

            this.active = true;
            this.modal.show();
        } else {
            this._incidentSourcesServiceProxy.getIncidentSourceForEdit(incidentSourceId).subscribe(result => {
                this.incidentSource = result.incidentSource;


                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;

			
            this._incidentSourcesServiceProxy.createOrEdit(this.incidentSource)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
