import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { IncidentSourcesServiceProxy, IncidentSourceDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditIncidentSourceModalComponent } from './create-or-edit-incidentSource-modal.component';
import { ViewIncidentSourceModalComponent } from './view-incidentSource-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './incidentSources.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    providers: [ IncidentSourcesServiceProxy]
})
export class IncidentSourcesComponent extends AppComponentBase {

    @ViewChild('createOrEditIncidentSourceModal', { static: true }) createOrEditIncidentSourceModal: CreateOrEditIncidentSourceModalComponent;
    @ViewChild('viewIncidentSourceModalComponent', { static: true }) viewIncidentSourceModal: ViewIncidentSourceModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    nameFilter = '';
    isActiveFilter = -1;
    descriptionFilter = '';
    isActiveFilterSwitch: boolean;

    _entityTypeFullName = 'SprintTek.Shipping.Incidents.IncidentSource';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _incidentSourcesServiceProxy: IncidentSourcesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getIncidentSources(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._incidentSourcesServiceProxy.getAll(
            this.filterText,
            this.nameFilter,
            this.isActiveFilter,
            this.descriptionFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    switchClicked(){
        if(this.isActiveFilterSwitch == false){
            this.isActiveFilter = 0
        }else{
            this.isActiveFilter = 1
        }
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createIncidentSource(): void {
        this.createOrEditIncidentSourceModal.show();
    }

    showHistory(incidentSource: IncidentSourceDto): void {
        this.entityTypeHistoryModal.show({
            entityId: incidentSource.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteIncidentSource(incidentSource: IncidentSourceDto): void {
        this.message.confirm(
            '','',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._incidentSourcesServiceProxy.delete(incidentSource.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._incidentSourcesServiceProxy.getIncidentSourcesToExcel(
        this.filterText,
            this.nameFilter,
            this.isActiveFilter,
            this.descriptionFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
