import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { OrderTypesServiceProxy, OrderTypeDto, IncidentTypesServiceProxy, IncidentStatusServiceProxy, IncidentStatusListDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { CreateUpdateIncidentTypeComponent } from './create-update-incident-type.component';
import { CreateUpdateIncidentStatusComponent } from './create-update-incidentstatus.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './incident-status.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class IncidentStatusComponent extends AppComponentBase {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    
    @ViewChild('createUpdateIncidentStatusModal', {static: false}) createUpdateIncidentStatusModal: CreateUpdateIncidentStatusComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Incidents.IncidentStatuses';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    isActiveFilter = -1;
    colorFilter = '';


    constructor(
        injector: Injector,
        private _incidentStatus: IncidentStatusServiceProxy
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

  
    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: IncidentStatusListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Incident Status - " +role.name
        });
    }

    getIncidentStatus(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._incidentStatus.getPagedIncidentStatus(
            undefined,
            this.filterText,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    // reloadPage(): void {
    //     this.paginator.changePage(this.paginator.getPage());
    // }

    createUpdateIncidentStatus(id?): void {
        this.createUpdateIncidentStatusModal.show(id);
    }

    // deleteIncidentType(record){      
    //     this.spinnerService.show();
    //     this._incidentTypeService.delete(record.incidentType.id).subscribe(result=>{
    //         this.spinnerService.hide();
    //         this.reloadPage();
    //     })
    // }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteIncidentStatus(id): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._incidentStatus.deleteIncidentStatus(id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    // exportToExcel(): void {
    //     this._orderTypesServiceProxy.getOrderTypesToExcel(
    //     this.filterText,
    //         this.codeFilter,
    //         this.descriptionFilter,
    //         this.isActiveFilter,
    //         this.colorFilter,
    //     )
    //     .subscribe(result => {
    //         this._fileDownloadService.downloadTempFile(result);
    //      });
    // }
}
