import { Component, Injector, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { IncidentListDto, IncidentServiceProxy, WillCallServiceServiceProxy, IncidentStatusServiceProxy, IncidentTypesServiceProxy, PagedIncidentList, OrderTypesServiceProxy } from '@shared/service-proxies/service-proxies';
import { IncidentImageListDto, IncidentImageServiceProxy } from '@shared/service-proxies/service-proxies';
import { ImageListDto, ImageServiceProxy } from '@shared/service-proxies/service-proxies';

import { FileDownloadService } from '@shared/utils/file-download.service';
import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { FileUpload } from 'primeng/fileupload';
import { finalize } from 'rxjs/operators';
import { CreateIncidentModalComponent } from './create-incident-modal.component';
import * as _ from 'lodash';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
@Component({
    templateUrl: 'incident.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})

export class IncidentComponent extends AppComponentBase implements OnInit {

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('createIncidentModal', {static: false}) createIncidentModal: CreateIncidentModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Incidents.OrderIncident';
    entityHistoryEnabled = false;
    
    images: IncidentImageListDto[] = [];
    
    
    imageId: any;
    incidentImageId: any;

    //images: {
        //id: number;
        //path: string;
        //name: string;
        
    //} = <any>{};

    filters: {
        id: number;
        content: string;
        CurrentPage: number;
    } = <any>{};
    managers:any;
    statuses:any;
    types:any;
    incidentId: any;
    t=undefined;
    s=undefined;
    advancedFiltersAreShown = false;
    orderNumber: number;
    incidentNumber: number;
    createdBy: string;
    dateRange=null;
    startString: any;
    endString: any;
    orderTypeId:number;
    filteredOrderTypes: any;
    constructor(
        injector: Injector,
        private router: Router,
        private _incidentService: IncidentServiceProxy,
        private _incidentImageService: IncidentImageServiceProxy,
        private _willcallService: WillCallServiceServiceProxy,
        private _incidentStatus: IncidentStatusServiceProxy,
        private _incidentType: IncidentTypesServiceProxy,
        private _imageService: ImageServiceProxy,
        private _orderTypeService: OrderTypesServiceProxy
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
        $('.kt-select2').select2();
        
        // this._willcallService.getManagers()
        // .subscribe(result=>{
        //     this.managers = result;
        // })
        this._incidentStatus.getAllIncidentStatus(undefined,undefined)
        .subscribe(result=>{
            this.statuses = result
        })
        this._incidentType.getAllIncidentType()
        .subscribe(result=>{
            this.types = result.items
        })
        this.filterOrderTypes();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: PagedIncidentList): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order Incident - " +role.creator
        });
    }

    getIncidents(event?: LazyLoadEvent) {
          
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        if(this.advancedFiltersAreShown){
            this.t = Number((<HTMLInputElement>document.getElementById('Type')).value);
            this.s = Number((<HTMLInputElement>document.getElementById('Status')).value);
        }else{
            
            this.t = undefined;
            this.s = undefined;
        }
        if (this.dateRange!=undefined){
            this.startString = new Date(this.dateRange[0]).toLocaleDateString('en-US');
            this.endString = new Date(this.dateRange[1]).toLocaleDateString('en-US');
        }
        this.orderTypeId =  Number((<HTMLInputElement>document.getElementById('orderType')).value) ? Number((<HTMLInputElement>document.getElementById('orderType')).value) : undefined;
        this.spinnerService.show();
        this._incidentService.getPagedIncident(
            this.filters.content,
            this.incidentNumber,
            this.t,
            this.orderTypeId,
            this.s,
            this.filters.content,
            this.orderNumber,
            this.startString, 
            this.endString,
            this.createdBy,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            // console.log(result)
            this.spinnerService.hide();
            
            this.incidentId =  this.primengTableHelper.records;           
                        

            //console.log(this.incidentId.id);
            
        });

    }

    exportToExcel(): void {
        this.spinnerService.show();
        if(this.advancedFiltersAreShown){
            this.t = Number((<HTMLInputElement>document.getElementById('Type')).value);
            this.s = Number((<HTMLInputElement>document.getElementById('Status')).value);
        }else{
            
            this.t = undefined;
            this.s = undefined;
        }
        if (this.dateRange!=undefined){
            this.startString = new Date(this.dateRange[0]).toLocaleDateString('en-US');
            this.endString = new Date(this.dateRange[1]).toLocaleDateString('en-US');
        }
        this.orderTypeId =  Number((<HTMLInputElement>document.getElementById('orderType')).value) ? Number((<HTMLInputElement>document.getElementById('orderType')).value) : undefined;

        this._incidentService.getOrderIncidentToExcel(
            this.filters.content,
            this.incidentNumber,
            this.t,
            this.orderTypeId,
            this.s,
            this.filters.content,
            this.orderNumber,
            this.startString, 
            this.endString,
            this.createdBy,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    deleteIncidentImage(incidentId): void {

            this._incidentImageService.getIncidentImage(
            undefined,
            undefined,
            incidentId,
            undefined,
            undefined,
            undefined

            ).subscribe(result => {

                if(result.totalCount == 0) {
                    console.log('nothing to delete');
                }
                else {


                                    this.incidentImageId = result.items;

                                    for(let item of this.incidentImageId) {
                                        //console.log(item.id);

                                        this._incidentImageService.deleteIncidentImage(item.id).subscribe(() => {
                                                                   
                                        });

                                        this._imageService.deleteImage(item.imageId).subscribe(() => {
                                                                   
                                        });
                                    }

                }
                
                
                
            });
            
             
        
    }
    
    createIncident(): void {
        this.createIncidentModal.show();
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteIncident(incident: IncidentListDto): void {
        this.message.confirm(
            '',
            '',
            isConfirmed => {
                if (isConfirmed) {
                    this._incidentService.deleteIncident(incident.id).subscribe(() => {
                        this.deleteIncidentImage(incident.id);
                        this.reloadPage();
                        this.notify.info(this.l('SuccessfullyDeleted'));
                        
                    });
                    
                }
            }
        );
    } 
    update(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        console.log(myurl);
        this.router.navigateByUrl(myurl);
    }

 
    view(url: any, id?: number){
        if(id != null){
            var myurl = `${url}/${id}`;
        }else{
            var myurl = `${url}`;
        }
        console.log(myurl);
        this.router.navigateByUrl(myurl);
    }
    filterOrderTypes(): void {
        this._orderTypeService.getOrderTypes().subscribe(result => {
            this.filteredOrderTypes = result;
        });
    }
   

}
