import { Component, Injector, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { VehicleLocationListDto, VehicleLocationServiceProxy, LocationServiceProxy } from '@shared/service-proxies/service-proxies';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { FileUpload } from 'primeng/fileupload';
import { finalize } from 'rxjs/operators';
import { CreateVehicleLocationModalComponent } from './create-vehicle-location-modal.component';

import * as _ from 'lodash';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
@Component({
    templateUrl: 'vehicle-location.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})

export class VehicleLocationComponent extends AppComponentBase implements OnInit {

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('createVehicleLocationModal', {static: false}) createVehicleLocationModal: CreateVehicleLocationModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Locations.VehicleLocation';
    entityHistoryEnabled = false;

    
    filters: {
        id: number;
        vehicleId: number;
        locationId: number;
        vehicle: string;
        CurrentPage: number;
        filter: string;
        vehicleFilter: string;
        locationFilter: string;
    } = <any>{};
    advancedFiltersAreShown = false;
    locations:any;
    locationId: number;


    vehicleLocation : VehicleLocationListDto;
    constructor(
        injector: Injector,
        private _vehicleLocationService: VehicleLocationServiceProxy,
        private _locationService: LocationServiceProxy,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        $('.kt-select2').select2();
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: VehicleLocationListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Vehicle Location - " +role.vehicle.name
        });
    }

    exportToExcel(): void{
        this.spinnerService.show();
        this._vehicleLocationService.getVehicleLocationsToExcel(
            this.filters.id,
            this.filters.filter,
            this.filters.vehicle,
            this.filters.locationFilter,
            this.locationId
        ).subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
        });
    }

    getVehicleLocations(event?: LazyLoadEvent) {
        
        this.locationId = Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) != 0 ? Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) : undefined;
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        this.spinnerService.show();
        this._vehicleLocationService.getVehicleLocation(
            this.filters.id,
            this.filters.vehicleId,
            this.locationId,
            this.filters.vehicle,
            this.filters.filter,
            this.filters.vehicleFilter,
            this.filters.locationFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
            // console.log(result.items);
        });

        this.getLocation()
        $('.kt-select2').select2();
    }

     
   createVehicleLocation(): void {
        this.createVehicleLocationModal.show();
    }

    

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }
    
   deleteVehicleLocation(vehicleLocation): void {
    this.message.confirm(
        this.l('Delete Vehicle Location'),
        '',
        isConfirmed => {
            if (isConfirmed) {
                this._vehicleLocationService.deleteVehicleLocation(vehicleLocation).subscribe(() => {
                    this.reloadPage();
                    this.notify.info(this.l('Successfully Deleted'));
                    
                });
                
            }
        }
      );
    } 

    getLocation(){
        // this._locationService.getLocation(
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined,
        //     undefined
        // ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
        //     this.locations = result.items 
        // });
        this._locationService.getLocationTableOnly(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.locations = result; 
        });
    }

}
