import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { RouteOptimizersServiceProxy, RouteOptimizerDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './route-optimizer.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RouteOptimizersComponent extends AppComponentBase {

    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    nameFilter = '';
    trafficFilter = -1;
    highwaysFilter = -1;
    ferriesFilter = -1;
    indoorFilter = -1;
    activeFilter = -1;
    maxStartPeriodFilter : moment.Moment;
		minStartPeriodFilter : moment.Moment;
    maxEndPeriodFilter : moment.Moment;
		minEndPeriodFilter : moment.Moment;
        locationNameFilter = '';




    constructor(
        injector: Injector,
        private _routeOptimizersServiceProxy: RouteOptimizersServiceProxy,
        private router: Router,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getRouteOptimizers(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._routeOptimizersServiceProxy.getAll(
            this.filterText,
            this.nameFilter,
            this.trafficFilter,
            this.highwaysFilter,
            this.ferriesFilter,
            this.indoorFilter,
            this.activeFilter,
            this.maxStartPeriodFilter,
            this.minStartPeriodFilter,
            this.maxEndPeriodFilter,
            this.minEndPeriodFilter,
            this.locationNameFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteRouteOptimizer(routeOptimizer: RouteOptimizerDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeOptimizersServiceProxy.delete(routeOptimizer.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._routeOptimizersServiceProxy.getRouteOptimizersToExcel(
        this.filterText,
            this.nameFilter,
            this.trafficFilter,
            this.highwaysFilter,
            this.ferriesFilter,
            this.indoorFilter,
            this.activeFilter,
            this.maxStartPeriodFilter,
            this.minStartPeriodFilter,
            this.maxEndPeriodFilter,
            this.minEndPeriodFilter,
            this.locationNameFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    createOrUpdateRouteOptimizer(id?){
        let url = 'app/sprintship/route-optimizer/create-update-route-optimizer/';
        if(id != null){
            var myurl = url + `${id}`;
        }else{
            var myurl = url;
        }
        this.router.navigateByUrl(myurl);
    }
}

