import { Injector, ViewChild } from '@angular/core';
import { Component, OnInit } from '@angular/core';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { ContainersServiceProxy } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { CreateOrEditContainerModalComponent } from './create-or-edit-container-modal.component';

@Component({
  selector: 'containers',
  templateUrl: './container.component.html',
  animations: [appModuleAnimation()]
})
export class ContainerComponent extends AppComponentBase implements OnInit {

  @ViewChild('dataTable', { static: true }) dataTable: Table;
  @ViewChild('paginator', { static: true }) paginator: Paginator;
  @ViewChild(CreateOrEditContainerModalComponent, { static: true}) createOrEditContainerModal;

  advancedFiltersAreShown = false;
  filterText = '';
  codeFilter = '';
  nameFilter = '';
  inActiveFilter: number = -1;
  colorFilter = '';

  constructor(
      injector: Injector,
      private _containerServiceProxy: ContainersServiceProxy
  ) {  super(injector); }

  ngOnInit() {

  }

  getContainers(event?: LazyLoadEvent) {
    if (this.primengTableHelper.shouldResetPaging(event)) {
        this.paginator.changePage(0);
        return;
    }

    this.spinnerService.show();

    this._containerServiceProxy.getPaged(
        this.filterText,
        this.codeFilter,
        this.nameFilter,
        this.inActiveFilter,
        this.colorFilter,
        this.primengTableHelper.getSorting(this.dataTable),
        this.primengTableHelper.getMaxResultCount(this.paginator, event),
        this.primengTableHelper.getSkipCount(this.paginator, event)
    ).subscribe(result => {
        this.primengTableHelper.totalRecordsCount = result.totalCount;
        this.primengTableHelper.records = result.items;
        this.spinnerService.hide();
    });
  }

  deleteContainer(id: number): void {
    this.message.confirm(
        '',
        '',
        (isConfirmed) => {
            if (isConfirmed) {
                this._containerServiceProxy.delete(id)
                    .subscribe(() => {
                        this.reloadPage();
                        this.notify.success(this.l('SuccessfullyDeleted'));
                    });
            }
        }
    );
  }

  reloadPage(): void {
    this.paginator.changePage(this.paginator.getPage());
  }

  createContainer() {
    this.createOrEditContainerModal.show();
  }

}
