﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { OrderNotificationsServiceProxy, OrderNotificationDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderNotificationModalComponent } from './create-or-edit-orderNotification-modal.component';
import { ViewOrderNotificationModalComponent } from './view-orderNotification-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { TestNotificationComponent } from './test-notification.component';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './orderNotifications.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./order-notification.component.less']
})
export class OrderNotificationsComponent extends AppComponentBase {

    @ViewChild('testNotificationModalComponent', { static: true }) testNotificationComponent: TestNotificationComponent;
    @ViewChild('createOrEditOrderNotificationModal', { static: true }) createOrEditOrderNotificationModal: CreateOrEditOrderNotificationModalComponent;
    @ViewChild('viewOrderNotificationModalComponent', { static: true }) viewOrderNotificationModal: ViewOrderNotificationModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    isActiveFilter = -1;
    descriptionFilter = '';
        orderNotificationTemplateDescriptionFilter = '';
        orderTypeDescriptionFilter = '';
        smsTemplateDescriptionFilter = '';
        orderStatusTypeDescriptionFilter = '';


    _entityTypeFullName = 'SprintTek.Shipping.Orders.OrderNotification';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _orderNotificationsServiceProxy: OrderNotificationsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    testNotification():void {
        this.testNotificationComponent.show();
    }

    getOrderNotifications(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._orderNotificationsServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.isActiveFilter,
            this.descriptionFilter,
            this.orderNotificationTemplateDescriptionFilter,
            this.orderTypeDescriptionFilter,
            this.smsTemplateDescriptionFilter,
            this.orderStatusTypeDescriptionFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createOrderNotification(): void {
        this.createOrEditOrderNotificationModal.show();
    }

    showHistory(orderNotification: OrderNotificationDto): void {
        this.entityTypeHistoryModal.show({
            entityId: orderNotification.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order Notification- " +orderNotification.description
        });
    }

    deleteOrderNotification(orderNotification: OrderNotificationDto): void {
        this.message.confirm(
            "","",
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderNotificationsServiceProxy.delete(orderNotification.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._orderNotificationsServiceProxy.getOrderNotificationsToExcel(
        this.filterText,
            this.codeFilter,
            this.isActiveFilter,
            this.descriptionFilter,
            this.orderNotificationTemplateDescriptionFilter,
            this.orderTypeDescriptionFilter,
            this.smsTemplateDescriptionFilter,
            this.orderStatusTypeDescriptionFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
