import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { OrderStatusTypesServiceProxy, OrderStatusTypeDto, GetOrderStatusTypeForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderStatusTypeModalComponent } from './create-or-edit-orderStatusType-modal.component';
import { ViewOrderStatusTypeModalComponent } from './view-orderStatusType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './orderStatusTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class OrderStatusTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditOrderStatusTypeModal', { static: true }) createOrEditOrderStatusTypeModal: CreateOrEditOrderStatusTypeModalComponent;
    @ViewChild('viewOrderStatusTypeModalComponent', { static: true }) viewOrderStatusTypeModal: ViewOrderStatusTypeModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Orders.OrderStatusType';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    maxSortOrderFilter : number;
		maxSortOrderFilterEmpty : number;
		minSortOrderFilter : number;
		minSortOrderFilterEmpty : number;




    constructor(
        injector: Injector,
        private _orderStatusTypesServiceProxy: OrderStatusTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetOrderStatusTypeForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.orderStatusType.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order Status Type - " +role.orderStatusType.description
        });
    }

    getOrderStatusTypes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();
        

        this._orderStatusTypesServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.maxSortOrderFilter == null ? this.maxSortOrderFilterEmpty: this.maxSortOrderFilter,
            this.minSortOrderFilter == null ? this.minSortOrderFilterEmpty: this.minSortOrderFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createOrderStatusType(): void {
        this.createOrEditOrderStatusTypeModal.show();
    }

    deleteOrderStatusType(orderStatusType: OrderStatusTypeDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderStatusTypesServiceProxy.delete(orderStatusType.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._orderStatusTypesServiceProxy.getOrderStatusTypesToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.maxSortOrderFilter == null ? this.maxSortOrderFilterEmpty: this.maxSortOrderFilter,
            this.minSortOrderFilter == null ? this.minSortOrderFilterEmpty: this.minSortOrderFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
