import { Component, Injector, ViewChild, ViewEncapsulation, Output, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { OrderStatusTypeServiceProxy, OrderStatusTypeListDto, OrderStatusTypeDescriptionServiceProxy, OrderStatusTypeDescriptionListDto, UpdateOrderStatusTypeDescriptionInput, SortOrderStatusDescription } from '@shared/service-proxies/service-proxies';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { finalize } from 'rxjs/operators';
import { CreateOrderStatuTypeModalComponent } from './create-order-status-types-modal.component';

// Complete SortableJS (with all plugins)
//import Sortable from 'sortablejs/modular/sortable.complete.esm.js';
import { HtmlHelper } from '@shared/helpers/HtmlHelper';
import * as _ from 'lodash';
import { Observable } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { TreeNode, MenuItem } from 'primeng/api';
declare var $: any;

import { ArrayToTreeConverterService } from '@shared/utils/array-to-tree-converter.service';
import { TreeDataHelperService } from '@shared/utils/tree-data-helper.service';


@Component({
    templateUrl: './order-status-type.component.html',
    styleUrls: ['./order-status-type.component.less'],
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class OrderStatusTypeComponent extends AppComponentBase {

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    // @ViewChild('ExcelFileUpload') excelFileUpload: FileUpload;
    @ViewChild('createOrderStatuTypeModal', {static: false}) createOrderStatuTypeModal: CreateOrderStatuTypeModalComponent;

    uploadUrl: string;
    treeData: any;
    //Filters
    saving = false;
    advancedFiltersAreShown = false;
    filterText = '';
    inactiveStatus = false;
    visibleStatus = true;
    parents: any;
    active_class: string;
    inactive_class: string;
    invisible_class: string;
    visible_class: string;
    icon_class: string;
    selectedOu: TreeNode;
    ouContextMenuItems: MenuItem[];
    code: any;
    color: any;
    parent: string;
    orderStatusTypeDescription: UpdateOrderStatusTypeDescriptionInput = new UpdateOrderStatusTypeDescriptionInput();
    newData: Array<any> = [];
    update: Array<any> = [];
    statusTypes: OrderStatusTypeListDto[] = [];
    
    constructor(
        injector: Injector,
        private _orderStatusTypeService: OrderStatusTypeServiceProxy,
        private _orderStatusTypeDescriptionService: OrderStatusTypeDescriptionServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _arrayToTreeConverterService: ArrayToTreeConverterService,
        private _treeDataHelperService: TreeDataHelperService,
        private _orderStatusTypeAppService: OrderStatusTypeServiceProxy,
    ) {
        super(injector);
        // this.filterText = this._activatedRoute.snapshot.queryParams['filterText'] || '';
    }
    totalUnitCount = 0;
  
    ngOnInit() {
        this.getAllDropDown();
        this.active_class = "btn btn-primary";
        this.inactive_class = "btn btn-light";

        this.visible_class = "btn btn-primary";
        this.invisible_class = "btn btn-light";
        this.icon_class = "far fa-clock"

        this.spinnerService.show();
        this.getTreeDataFromServerDefault();

       
       

    }

    createOrderStatuType(): void {
        this.createOrderStatuTypeModal.show();
    }


    deleteOrderStatusType(id): void {
        this.message.confirm(
            this.l('AreYouSure'),
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderStatusTypeDescriptionService.deleteOrderStatusTypeDescription(id)
                        .subscribe(() => {
                            // this.reloadPage();
                            window.location.reload()
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    settings(nodeData): void {
        //console.log(nodeData);
        this.getAllDropDown();
        this.code = nodeData.orderStatusTypeCode;
        if(nodeData.color != null) {
            this.color = nodeData.color;
         
        }
        else {
            this.color = "#494a49";
        
        }
        
        this.orderStatusTypeDescription.id = nodeData.id;
        this.orderStatusTypeDescription.orderStatusTypeId = nodeData.orderStatusTypeId;
        this.orderStatusTypeDescription.name = nodeData.name;
        this.orderStatusTypeDescription.sortOrder = nodeData.sortOrder;
        this.orderStatusTypeDescription.icon = nodeData.icon;
        this.orderStatusTypeDescription.visible = nodeData.visible;
        this.orderStatusTypeDescription.inactive = nodeData.inactive;
        this.orderStatusTypeDescription.description = nodeData.description;
        this.orderStatusTypeDescription.parentId = nodeData.parentId;
        if(this.orderStatusTypeDescription.icon != null) {
            this.icon_class = this.orderStatusTypeDescription.icon;
        
        }
        else {
             this.icon_class = "far fa-clock";
             this.orderStatusTypeDescription.icon = "far fa-clock";
        }

        if(this.orderStatusTypeDescription.inactive != true) {
            this.inactive_class = "btn btn-light";
            this.active_class = "btn btn-primary";
        
        }
        else {
            
            this.active_class = "btn btn-light";
            this.inactive_class = "btn btn-primary";
        
        }

        if(this.orderStatusTypeDescription.visible != true) {
            this.visible_class = "btn btn-light";
            this.invisible_class = "btn btn-primary";
        }
        else {
             
            
            
            this.invisible_class = "btn btn-light";
            this.visible_class = "btn btn-primary";
        }


        if(nodeData.parentId != null) {
            this._orderStatusTypeDescriptionService.getParentStatus(
            nodeData.parentId
            ).pipe()
            .subscribe(result => {
                this.parent = result.name;

                
            });
        }
        else {
            this.parent = 'None';
        }
       
        //this.reloadPage();
        

    }

    reloadPage(): void {
        //this.paginator.changePage(this.paginator.getPage());
        this.getTreeDataFromServer();

    }

    active($event): void {
            this.inactiveStatus = false;
            this.inactive_class = "btn btn-light";
            this.active_class = "btn btn-primary";
            
    }
    inactive($event): void {
            this.inactiveStatus = true;
            this.active_class = "btn btn-light";
            this.inactive_class = "btn btn-primary";
            
    }

    visible($event): void {
            this.visibleStatus = true;
            this.invisible_class = "btn btn-light";
            this.visible_class = "btn btn-primary";
            
    }
    invisible($event): void {
            this.visibleStatus = false;
            this.visible_class = "btn btn-light";
            this.invisible_class = "btn btn-primary";
            
    }

    nodeSelect(event) {
        //this.ouSelected.emit(<IBasicOrganizationUnitInfo>{
            //id: event.node.data.id,
            //displayName: event.node.data.displayName
        //});
    }

    getAllDropDown() {
        jQuery(document).ready(function () { 
            $('.kt-select2').select2();
            $('.select2-container').css('width', '100%');

        });
        


        this._orderStatusTypeAppService.getOrderStatusTypeAll(
            
        ).pipe()
        .subscribe(result => {
            //this.parents = result.items;
            
            this.statusTypes = result.items;
        });

        
    }

    private getTreeDataFromServerDefault(): void {
        let self = this;
        this.spinnerService.show()
        this._orderStatusTypeDescriptionService.getOrderStatusTypeDescriptions(
            
        ).pipe(finalize(() => this.spinnerService.hide()))
        .subscribe(result => {
            //this.parents = result.items;
            this.totalUnitCount = result.items.length;
            
            this.treeData = this._arrayToTreeConverterService.createTree(result.items,
                'parentId',
                'orderStatusTypeId',
                null,
                'children',
                [
                    {
                        target: 'label',
                        targetFunction(item) {
                            return item.name;
                        }
                    },
                    {
                        target: 'selectable',
                        value: true
                    },
                    {
                        target: 'expanded',
                        value: true
                    }
                    
            ]);
            //console.log(this.treeData);
            this.code = this.treeData[0].data.orderStatusTypeCode;
            if(this.treeData[0].data.color != null) {
                this.color = this.treeData[0].data.color;
             
            }
            else {
                this.color = "#494a49";
            
            }
            this.parent = "None";
            this.orderStatusTypeDescription.id = this.treeData[0].data.id;
            this.orderStatusTypeDescription.orderStatusTypeId = this.treeData[0].data.orderStatusTypeId;
            this.orderStatusTypeDescription.name = this.treeData[0].data.name;
            this.orderStatusTypeDescription.sortOrder = this.treeData[0].data.sortOrder;
            this.orderStatusTypeDescription.icon = this.treeData[0].data.icon;
            this.orderStatusTypeDescription.visible = this.treeData[0].data.visible;
            this.orderStatusTypeDescription.inactive = this.treeData[0].data.inactive;
            this.orderStatusTypeDescription.description = this.treeData[0].data.description;
            this.orderStatusTypeDescription.parentId = this.treeData[0].data.parentId;
            if(this.orderStatusTypeDescription.icon != null) {
                this.icon_class = this.orderStatusTypeDescription.icon;
    
            }
            else {
                 this.icon_class = "far fa-clock";
                 this.orderStatusTypeDescription.icon = "far fa-clock"
            }


            
            if(this.orderStatusTypeDescription.inactive != true) {
                this.inactive_class = "btn btn-light";
                this.active_class = "btn btn-primary";
            
            }
            else {
                
                this.active_class = "btn btn-light";
                this.inactive_class = "btn btn-primary";
            
            }

            if(this.orderStatusTypeDescription.visible != true) {
                this.visible_class = "btn btn-light";
                this.invisible_class = "btn btn-primary";
            }
            else {
                 
                
                
                this.invisible_class = "btn btn-light";
                this.visible_class = "btn btn-primary";
            }


        });

        

    }

    private getTreeDataFromServer(): void {
        let self = this;
        this.spinnerService.show();

        this._orderStatusTypeDescriptionService.getOrderStatusTypeDescriptions(
            
        ).pipe(finalize(() => this.spinnerService.hide()))
        .subscribe(result => {
            //this.parents = result.items;

            this.totalUnitCount = result.items.length;
            this.treeData = this._arrayToTreeConverterService.createTree(result.items,
                'parentId',
                'id',
                null,
                'children',
                [
                    {
                        target: 'label',
                        targetFunction(item) {
                            return item.name;
                        }
                    },
                    {
                        target: 'selectable',
                        value: true
                    },
                    {
                        target: 'expanded',
                        value: true
                    }
                    
            ]);
           

        });

        

    }

    nodeDrop(event) {
        //console.log(this.treeData);
        let that = this;
        let newParentId;
        
        if(event.dropIndex) {
            newParentId = null;
        }
        else if(event.dropIndex == 0) {
            newParentId = null;
        }
        else {
            
            newParentId = event.dropNode.data.orderStatusTypeId;
        }
       
               
                let childId = event.dragNode.data.id;
                that.searchTree(this.treeData, newParentId, childId);

        for (let index = 0; index < this.newData.length; index++) {
            var element = this.newData[index];
            //console.log(element);

            let a = new UpdateOrderStatusTypeDescriptionInput({
                      id: element.id,
                      orderStatusTypeId: element.orderStatusTypeId,
                      description: element.description,
                      name: element.name,
                      color: element.color,
                      icon: element.icon,
                      inactive: element.inactive,
                      visible: element.visible,
                      parentId: element.parentId,
                      sortOrder: element.sortOrder
                
            });
            
            this.update.push(a);
        }

        let input = new SortOrderStatusDescription({
            sortStatusTypeDescription: this.update,
        });
        
        this._orderStatusTypeDescriptionService.sortOrderStatusDescription(
                   input
               ).pipe().subscribe(result => {
                   this.notify.info(this.l('SavedSuccessfully'));
        });


    }

    save(): void {
        let color = (<HTMLInputElement>document.getElementById('colors')).value;
        
        //console.log(color.value);
        this.orderStatusTypeDescription.visible = this.visibleStatus;
        this.orderStatusTypeDescription.inactive = this.inactiveStatus;
        let orderStatusTypeId =  (<HTMLInputElement>document.getElementById('statusTypeCreate')).value;
        this.orderStatusTypeDescription.orderStatusTypeId = Number(orderStatusTypeId);
        this.orderStatusTypeDescription.color = color;
        this.saving = true;
        this._orderStatusTypeDescriptionService.updateOrderStatusTypeDescription(this.orderStatusTypeDescription)
            .pipe(finalize(() => { this.saving = false; }))
            .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.reloadPage();
        });
    }

    searchTree(elements, parentId, childId) {
       
         
         let i;
         for(i=0; i < elements.length; i++){
            elements[i].data.sortOrder = i;
            if(elements[i].data.id == childId){
                elements[i].data.parentId = parentId;
            }
            this.newData.push(elements[i].data);
            if(elements[i].children.length != 0) {
                //console.log(elements[i].children);
                //console.log(elements[i].children);
                this.searchTree(elements[i].children, parentId, childId);
            }
         }
         
    }

}
