﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { OrderTypesServiceProxy, CreateOrEditOrderTypeDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditOrderTypeModal',
    templateUrl: './create-or-edit-orderType-modal.component.html'
})
export class CreateOrEditOrderTypeModalComponent extends AppComponentBase {
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;
    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();
    active = false;
    saving = false;
    orderType: CreateOrEditOrderTypeDto = new CreateOrEditOrderTypeDto();
    public color: string = '#2889e9';
    constructor(
        injector: Injector,
        private _orderTypesServiceProxy: OrderTypesServiceProxy
    ) {
        super(injector);
    }

    show(orderTypeId?: number): void {

        if (!orderTypeId) {
            this.orderType = new CreateOrEditOrderTypeDto();
            this.orderType.id = orderTypeId;
            //this.orderType.isActive = true;
            this.active = true;
            this.orderType.color='#000000';
            this.orderType.textColor='#000000';
            this.modal.show();
        } else {
            this._orderTypesServiceProxy.getOrderTypeForEdit(orderTypeId).subscribe(result => {
                this.orderType = result.orderType;


                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
            this._orderTypesServiceProxy.createOrEdit(this.orderType)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
