import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { OrderTypesServiceProxy, OrderTypeDto, GetOrderTypeForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderTypeModalComponent } from './create-or-edit-orderType-modal.component';
import { ViewOrderTypeModalComponent } from './view-orderType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './orderTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class OrderTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditOrderTypeModal', { static: true }) createOrEditOrderTypeModal: CreateOrEditOrderTypeModalComponent;
    @ViewChild('viewOrderTypeModalComponent', { static: true }) viewOrderTypeModal: ViewOrderTypeModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.OrderTypes.OrderType';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    isActiveFilter = -1;
    colorFilter = '';


    constructor(
        injector: Injector,
        private _orderTypesServiceProxy: OrderTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,
    ) {
        super(injector);
    }

    ngOnInit(): void{
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetOrderTypeForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.orderType.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order Type - " +role.orderType.description
        });
    }

    getOrderTypes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._orderTypesServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.isActiveFilter,
            this.colorFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createOrderType(): void {
        this.createOrEditOrderTypeModal.show();
    }

    deleteOrderType(orderType: OrderTypeDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderTypesServiceProxy.delete(orderType.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this.spinnerService.show();
        this._orderTypesServiceProxy.getOrderTypesToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.isActiveFilter,
            this.colorFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    // this.message.success('','EDI was sent successfully to your email');
                    // this._fileDownloadService.downloadTempFile(result);
                    this.spinnerService.hide();
                }else{
                    // this.message.warn('', result.message);
                    this.spinnerService.hide();
                }
            }else{
                this.spinnerService.hide();
            }
         });
    }
}
