﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { OrderStatusMappingsServiceProxy, OrderStatusMappingDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditOrderStatusMappingModalComponent } from './create-or-edit-orderStatusMapping-modal.component';
import { ViewOrderStatusMappingModalComponent } from './view-orderStatusMapping-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
declare var $: any;
@Component({
    templateUrl: './orderStatusMappings.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class OrderStatusMappingsComponent extends AppComponentBase {

    @ViewChild('createOrEditOrderStatusMappingModal', { static: true }) createOrEditOrderStatusMappingModal: CreateOrEditOrderStatusMappingModalComponent;
    @ViewChild('viewOrderStatusMappingModalComponent', { static: true }) viewOrderStatusMappingModal: ViewOrderStatusMappingModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
        orderStatusDescriptionNameFilter = '';
        orderSourceNameFilter = '';
    id = null;

    _entityTypeFullName = 'SprintTek.Shipping.OrderStatusMappings.OrderStatusMapping';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _orderStatusMappingsServiceProxy: OrderStatusMappingsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
        this._activatedRoute.paramMap.subscribe(params => {
            this.id = params.get("id") ;    
        });

        jQuery(document).ready(function () { 
            $('.kt-select2').select2();
            $('.select2-container').css('width', '100%');

        });
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getOrderStatusMappings(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._orderStatusMappingsServiceProxy.getAll(
            this.id,
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.orderStatusDescriptionNameFilter,
            this.orderSourceNameFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createOrderStatusMapping(): void {
        this.createOrEditOrderStatusMappingModal.show();
    }

    showHistory(orderStatusMapping: OrderStatusMappingDto): void {
        this.entityTypeHistoryModal.show({
            entityId: orderStatusMapping.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteOrderStatusMapping(orderStatusMapping: OrderStatusMappingDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._orderStatusMappingsServiceProxy.delete(orderStatusMapping.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._orderStatusMappingsServiceProxy.getOrderStatusMappingsToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.orderStatusDescriptionNameFilter,
            this.orderSourceNameFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
