﻿import { Component, ViewChild, Injector, Output, EventEmitter, OnInit, ElementRef} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { RecurringOrderSchedulesServiceProxy, CreateOrEditRecurringOrderScheduleDto, TimeZoneServiceProxy, ListResultDtoOfTimeZoneListDto, TimeZoneListDto, NameValueDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';




@Component({
    selector: 'createOrEditRecurringOrderScheduleModal',
    templateUrl: './create-or-edit-recurringOrderSchedule-modal.component.html'
})
export class CreateOrEditRecurringOrderScheduleModalComponent extends AppComponentBase implements OnInit{
   
    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    recurringOrderSchedule: CreateOrEditRecurringOrderScheduleDto = new CreateOrEditRecurringOrderScheduleDto();

    defaultTz: string = 'UTC-11'

    timeZones : any;
    orderScheduleDate : Date;
    orderScheduleTime:any;
    description: string;
    code: string;
    input:any;
    orderScheduleId?: number;
    orderScheduleRepeat: string;
    orderScheduleEndDate: Date;
    orderScheduleEndTime: any;
    endDateActive: boolean = false 
    repeatValues: Array<{
        name: any,
        value: any
    }> = new Array<any>();
    distinctRepeatValues: any
    orderScheduleEndDateValidation:any
    constructor(
        injector: Injector,
        private _recurringOrderSchedulesServiceProxy: RecurringOrderSchedulesServiceProxy,
        private _timeZoneServiceProxy: TimeZoneServiceProxy
    ) {
        super(injector);
    }
    
    show(recurringOrderScheduleId?: number): void {
        
    
        this._timeZoneServiceProxy.getTimeZoneForDropDown(
        ).subscribe(result => {
            this.timeZones = result.items;
        });
        
        if (!recurringOrderScheduleId) {
            // console.log(new Date());
            // console.log(moment().toDate());
            // this.defaultTz = 'UTC-11';
            // this.orderScheduleRepeat = 'DAILY'

            this.defaultTz=""
            this.distinctRepeatCodes()
            this.getUserProfileTimezone();
            this.recurringOrderSchedule.id = recurringOrderScheduleId;
            this.orderScheduleDate = moment().toDate();
            this.orderScheduleTime = moment().toDate();
            this.orderScheduleEndDate = moment().add(1,'days').toDate();
            this.orderScheduleEndTime = moment().toDate();
            this.description = "";
            this.code = "";
            this.endDateActive = false
            this.orderScheduleId=null;
            this.orderScheduleEndDateValidation = moment().add(1,'days').toDate()
            this.active = true;
            this.modal.show();
        } else {
            this.spinnerService.show()
            
            this.orderScheduleId = recurringOrderScheduleId
            this._recurringOrderSchedulesServiceProxy.getRecurringOrderScheduleForEdit(recurringOrderScheduleId).subscribe(result => {
                // this.recurringOrderSchedule = result.recurringOrderSchedule;
                this.distinctRepeatCodes(result.recurringOrderSchedule.repeat)
                let startTime = result.recurringOrderSchedule.startDate.toDate().toDateString() + ' ' + result.recurringOrderSchedule.startTime;
                let endTime = result.recurringOrderSchedule.endDate != undefined ? result.recurringOrderSchedule.endDate.toDate().toDateString() + ' ' + result.recurringOrderSchedule.endTime : undefined;
                this.description = result.recurringOrderSchedule.description
                this.code = result.recurringOrderSchedule.code
                this.defaultTz = result.recurringOrderSchedule.timezone
                this.orderScheduleDate = result.recurringOrderSchedule.startDate.toDate()
                // .tz(localStorage.getItem('timeZoneId'))
                this.orderScheduleTime = moment(startTime).toDate();
                console.log(this.orderScheduleDate)
                console.log(result.recurringOrderSchedule.startDate.toDate())
                this.orderScheduleEndDate = result.recurringOrderSchedule.endDate != undefined ? result.recurringOrderSchedule.endDate.toDate() : undefined;
                this.orderScheduleEndTime =result.recurringOrderSchedule.endDate != undefined ? moment(endTime).toDate() : moment().toDate();
                this.orderScheduleEndDateValidation = result.recurringOrderSchedule.endDate != undefined ? result.recurringOrderSchedule.endDate.toDate() : moment().toDate();
                this.orderScheduleRepeat=result.recurringOrderSchedule.repeat
                this.active = true;
                this.endDateActive = result.recurringOrderSchedule.endDate != undefined ? true : false
                this.spinnerService.hide()
                this.modal.show();
            });
        }
    }

    distinctRepeatCodes(edit?: string){
        this.repeatValues = []
        this._recurringOrderSchedulesServiceProxy.getDistinctRepeatCreated().subscribe(result => {
            this.repeatValues.push({name:"Daily",value:"DAILY"})
            this.repeatValues.push({name:"Weekly",value:"WEEKLY"})
            this.repeatValues.push({name:"Monthly",value:"MONTHLY"})
            result.forEach(element => {
                if(element != edit){
                    let index: number 
                    index = this.repeatValues.findIndex(d=> d.value === element)
                    this.repeatValues.splice(index,1)
                    this.orderScheduleRepeat = this.repeatValues[0].value
                }
            });
        });
    }

    getUserProfileTimezone(){
        
        this.spinnerService.show()
        this._recurringOrderSchedulesServiceProxy.getUserDefaultTimezone().subscribe(result => {
            this.defaultTz = result
            this.spinnerService.hide();
        })

    }

    onShown(){

        setTimeout(() => {
            $('.kt-select2').select2();
        }, 3500);
        
    }

    save(): void {
            this.saving = true;
            this.input = new CreateOrEditRecurringOrderScheduleDto();
            if(this.orderScheduleId != null || this.orderScheduleId != undefined){
                this.input.id=this.orderScheduleId;
            }
            this.input.startDate = new Date(this.orderScheduleDate.setHours(this.orderScheduleDate.getHours() - this.orderScheduleDate.getTimezoneOffset() / 60));
            this.input.startTime = new Date(this.orderScheduleTime).toLocaleTimeString('en-US', { hour12: true, 
                                    hour: "numeric", 
                                    minute: "numeric",
                                    second: "numeric"
                                    }) as any;
            if(!this.endDateActive){
                this.input.endDate = undefined
                this.input.endTime = undefined
            }
            else{
                this.input.endDate = new Date(this.orderScheduleEndDate.setHours(this.orderScheduleEndDate.getHours() - this.orderScheduleEndDate.getTimezoneOffset() / 60));
                this.input.endTime = new Date(this.orderScheduleEndTime).toLocaleTimeString('en-US', { hour12: true, 
                                        hour: "numeric", 
                                        minute: "numeric",
                                        second: "numeric"
                                        }) as any;
                // console.log(this.input.endDate)
            }

            
            this.input.description = this.description;
            this.input.code = this.code;
            this.input.timezone=$('#orderScheduleTimezone').val();
            this.input.repeat=$('#orderScheduleRepeatInput').val();
			
            this._recurringOrderSchedulesServiceProxy.createOrEdit(this.input)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.orderScheduleId=null
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }

    close(): void {
        this.active = false;
        this.modal.hide();
    }
    
     ngOnInit(): void {
        
     }    
}
