﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { RecurringOrderSchedulesServiceProxy, RecurringOrderScheduleDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditRecurringOrderScheduleModalComponent } from './create-or-edit-recurringOrderSchedule-modal.component';
import { ViewRecurringOrderScheduleModalComponent } from './view-recurringOrderSchedule-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';
import { ProgressBarService } from '@app/shared/layout/progressbar/progressbar.service';
import {
    interval,
    Subscription
} from 'rxjs';
import * as signalR from '@aspnet/signalr';
import { AppConsts } from '@shared/AppConsts';
@Component({
    templateUrl: './recurringOrderSchedules.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class RecurringOrderSchedulesComponent extends AppComponentBase {

    @ViewChild('createOrEditRecurringOrderScheduleModal', { static: true }) createOrEditRecurringOrderScheduleModal: CreateOrEditRecurringOrderScheduleModalComponent;
    @ViewChild('viewRecurringOrderScheduleModalComponent', { static: true }) viewRecurringOrderScheduleModal: ViewRecurringOrderScheduleModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    timezoneFilter = '';
    maxStartDateFilter : moment.Moment;
		minStartDateFilter : moment.Moment;
    repeatFilter = '';
    maxEndDateFilter : moment.Moment;
		minEndDateFilter : moment.Moment;

    maxCreateCount: number
    private hubConnection: signalR.HubConnection;

    _entityTypeFullName = 'SprintTek.Shipping.RecurringOrderSchedules.RecurringOrderSchedule';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _recurringOrderSchedulesServiceProxy: RecurringOrderSchedulesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _progressBar: ProgressBarService,
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
        this.backgroundProcessListener();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getRecurringOrderSchedules(event?: LazyLoadEvent) {
        this.distinctRepeatCodes();
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._recurringOrderSchedulesServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.timezoneFilter,
            this.maxStartDateFilter === undefined ? this.maxStartDateFilter : moment(this.maxStartDateFilter).endOf('day'),
            this.minStartDateFilter === undefined ? this.minStartDateFilter : moment(this.minStartDateFilter).startOf('day'),
            this.repeatFilter,
            this.maxEndDateFilter === undefined ? this.maxEndDateFilter : moment(this.maxEndDateFilter).endOf('day'),
            this.minEndDateFilter === undefined ? this.minEndDateFilter : moment(this.minEndDateFilter).startOf('day'),
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createRecurringOrderSchedule(): void {
        this.createOrEditRecurringOrderScheduleModal.show();
    }

    toggle(toggle: boolean) {
        this._progressBar.toggleProgressBar(toggle);
    }
    sendMessage(message: string) {
        this._progressBar.sendMessage(message);
    }
    backgroundProcessListener() {
        this.hubConnection = new signalR.HubConnectionBuilder()
        .withUrl(AppConsts.remoteServiceBaseUrl + '/signalr-background')
        .build();

        this.hubConnection
            .start()
            .then(() => console.log('Connection started'))
            .catch(err => {
                console.log('Error while starting connection: ' + err);
                this.toggle(false);
            });

        this.hubConnection.on('BackgroundMessage', (message) => {
            if (message.match(/End Recurring/)) {
                this.toggle(false);
                this.notify.success('Recurring process is complete');
            }
            this.sendMessage(message);
        });
    }

    // importListener() {
    //     this.importConnection = new signalR.HubConnectionBuilder()
    //         .withUrl(AppConsts.remoteServiceBaseUrl + '/import')
    //         .build();

    //     this.importConnection
    //         .start()
    //         .then(() => console.log('Connection started'))
    //         .catch(err => console.log('Error while starting connection: ' + err));


    //     this.importConnection.on('BackgroundMessage', (message) => {

    //         if (message.match(/End Importing/)) {
    //             this.importRouteScheduleTracking.importing = false;
    //             this.importConnection.off;
    //         } else {
    //             this.importRouteScheduleTracking.logMessage(message);
    //         }
    //     });
    // }

    async triggerRecurring(recurringOrderSchedule) {
        
        this._recurringOrderSchedulesServiceProxy.runRecurringHangFire(recurringOrderSchedule).subscribe( response => {
            this.notify.success('Recurring Schedule is queued. Please wait');
            this.toggle(true);
        }, error => {
            this.notify.error(this.l('Recurring Schedule Failed'));
            this.toggle(false);
        });

        
       // let run = await this._recurringOrderSchedulesServiceProxy.runSchedgHangFire(recurringOrderSchedule.id,recurringOrderSchedule.description,recurringOrderSchedule.timezone).toPromise();
        // let observable = interval(1500).subscribe((val) => {
        //     this._recurringOrderSchedulesServiceProxy.runRecurringHangFire(recurringOrderSchedule.id,recurringOrderSchedule.description,recurringOrderSchedule.timezone).subscribe(watcher => {
        //         this.toggle(false);
        //         this.notify.success('Job is complete');
        //         observable.unsubscribe();
        //     });
        // });
    }
    
    showHistory(recurringOrderSchedule: RecurringOrderScheduleDto): void {
        this.entityTypeHistoryModal.show({
            entityId: recurringOrderSchedule.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: ''
        });
    }

    deleteRecurringOrderSchedule(recurringOrderSchedule: RecurringOrderScheduleDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._recurringOrderSchedulesServiceProxy.delete(recurringOrderSchedule.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._recurringOrderSchedulesServiceProxy.getRecurringOrderSchedulesToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.timezoneFilter,
            this.maxStartDateFilter === undefined ? this.maxStartDateFilter : moment(this.maxStartDateFilter).endOf('day'),
            this.minStartDateFilter === undefined ? this.minStartDateFilter : moment(this.minStartDateFilter).startOf('day'),
            this.repeatFilter,
            this.maxEndDateFilter === undefined ? this.maxEndDateFilter : moment(this.maxEndDateFilter).endOf('day'),
            this.minEndDateFilter === undefined ? this.minEndDateFilter : moment(this.minEndDateFilter).startOf('day'),
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }

    distinctRepeatCodes(){
        this._recurringOrderSchedulesServiceProxy.getDistinctRepeatCreated().subscribe(result => {
            this.maxCreateCount = result.length
        });
    }
}
