import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TagsServiceProxy, TagDto, TagTypeServiceProxy, CreateOrEditTagDto, GetTagForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditTagModalComponent } from './create-or-edit-tag-modal.component';
import { ViewTagModalComponent } from './view-tag-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './tags.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class TagsComponent extends AppComponentBase {

    @ViewChild('createOrEditTagModal', { static: true }) createOrEditTagModal: CreateOrEditTagModalComponent;
    @ViewChild('viewTagModalComponent', { static: true }) viewTagModal: ViewTagModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Orders.Tag';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    tagTypeId : number;
    nameFilter = '';
    filteredTagType: any;
    tagId : number;

    tag = new CreateOrEditTagDto();

    constructor(
        injector: Injector,
        private _tagsServiceProxy: TagsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _tagTypeAppService: TagTypeServiceProxy,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(){
        this.filterTagType();
        $('.kt-select2').select2({ 
            placeholder: "Select.."
        });
        this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetTagForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.tag.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Order Tag - " +role.tag.name
        });
    }

    getTags(event?: LazyLoadEvent) {
        this.tagTypeId =  Number((<HTMLInputElement>document.getElementById('selectPreview')).value) ? Number((<HTMLInputElement>document.getElementById('selectPreview')).value) : undefined;
        if (this.tagTypeId==0){
            this.tagTypeId = undefined;
        }
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._tagsServiceProxy.getAll(
            this.tagId,
            this.filterText,
            this.tagTypeId,
            this.nameFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createTag(): void {
        this.createOrEditTagModal.show();
    }

    deleteTag(tag: TagDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._tagsServiceProxy.delete(tag.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._tagsServiceProxy.getTagsToExcel(
        this.filterText,
        this.tagTypeId,
        this.nameFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }

    filterTagType(): void {
        this._tagTypeAppService.getTagType(
            undefined,
            undefined,
            undefined,
            undefined,
            undefined
        ).subscribe(result => {
            this.filteredTagType = result.items;
            console.log(this.filteredTagType);
        });
      }
}
