﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { UserOrderTypesServiceProxy, CreateOrEditUserOrderTypeDto ,UserOrderTypeUserLookupTableDto
					,UserOrderTypeOrderTypeLookupTableDto
					} from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';
import { CommonLookupModalComponent } from '@app/shared/common/lookup/common-lookup-modal.component';

@Component({
    selector: 'createOrEditUserOrderTypeModal',
    templateUrl: './create-or-edit-userOrderType-modal.component.html'
})
export class CreateOrEditUserOrderTypeModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;
    userOrderType: CreateOrEditUserOrderTypeDto = new CreateOrEditUserOrderTypeDto();
    @ViewChild('userList', {static: true}) userList: CommonLookupModalComponent;
    userName = '';
    orderTypeCode = '';
	allUsers: UserOrderTypeUserLookupTableDto[];
    allOrderTypes: UserOrderTypeOrderTypeLookupTableDto[];
    userFilter: any;
    userId: any;
    showSearchOnStartUser:any;
    searchUser: boolean;
    txtuserFilter: any;
    selectedTicketType: any;
    constructor(
        injector: Injector,
        private _userOrderTypesServiceProxy: UserOrderTypesServiceProxy
    ) {
        super(injector);
    }

    ngOnInit(){
        $('.kt-select2').select2();
        this.getUsersInit();
    }
    onShow(){
        let that = this;
        $('.kt-select2').select2();
        $('.kt-select2').on('select2:select', function (e) {
            if($('#uotUserId').val() != '' ||  that.txtuserFilter!= '') {
              $('#uotSubmit').prop('disabled', false);
            }
          });
    }

    show(userOrderTypeId?: number): void {
        if (!userOrderTypeId) {
            this.userOrderType = new CreateOrEditUserOrderTypeDto();
            this.userOrderType.id = userOrderTypeId;
            this.txtuserFilter = '';
            this.orderTypeCode = '';
            this.userId = undefined;
            this.active = true;
            this.modal.show();
        } else {
            this._userOrderTypesServiceProxy.getUserOrderTypeForEdit(userOrderTypeId).subscribe(result => {
                this.userOrderType = result.userOrderType;
                this.txtuserFilter = result.userName;
                this.orderTypeCode = result.orderTypeCode;
                this.userId = result.userOrderType.userId;
                this.active = true;
                this.modal.show();
            });
        }
        this._userOrderTypesServiceProxy.getAllOrderTypeForTableDropdown().subscribe(result => {
            this.allOrderTypes = result;
        });
    }

    getUsersInit(){
        this.userList.configure({
            title: this.l('Select User'),
            type:'select User',
            dataSource: (skipCount: number, maxResultCount: number, filter: string, tenantId?: number, locationId?: number) => {
                return this._userOrderTypesServiceProxy.getAllUserPaged(filter, undefined, skipCount, maxResultCount);
            }
        });
    }

    getUsers(filter : any) {
        this.userList.filterText = filter;
        this.userList.show();
    }

    clearUserList(){
        this.txtuserFilter = '';
        this.userId = null;
        this.showSearchOnStartUser = true;
    }

    selectUser(item) {
        var resultId = item.value;
        var name = item.name
        if(this.searchUser) {
            this.userId = resultId;
            this.txtuserFilter = name;
            this.searchUser = false;
            this.selectedTicketType = name;
        }
        else{
            this.userId = resultId;
            this.txtuserFilter = name;
            this.selectedTicketType = name;
            this.showSearchOnStartUser = false;
      }
    }

    save(): void {
            this.saving = true;
            this.userOrderType.userId = this.userId;
			this.userOrderType.orderTypeId = Number((<HTMLInputElement>document.getElementById('uotId')).value);
            if(this.userOrderType.userId==undefined || (this.userOrderType.orderTypeId==undefined || this.userOrderType.orderTypeId==0)){
                if(this.userOrderType.userId==undefined){
                    this.notify.error(this.l('Please Select User'));
                    this.saving = false;
                }
                if(this.userOrderType.orderTypeId==undefined || this.userOrderType.orderTypeId==0){
                    this.notify.error(this.l('Please Select Order Type'));
                    this.saving = false;
                }
            }else{
                this._userOrderTypesServiceProxy.createOrEdit(this.userOrderType)
                    .pipe(finalize(() => { this.saving = false;}))
                    .subscribe(() => {
                        this.notify.info(this.l('SavedSuccessfully'));
                        this.close();
                        this.modalSave.emit(null);
                    });
            }
        }

    close(): void {
        this.userId = undefined;
        this.userOrderType.orderTypeId = undefined;
        this.userOrderType.userId = undefined;
        this.active = false;
        this.modal.hide();
    }
}
