import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TimeZonesServiceProxy, TimeZoneDto, PackageOptionsServiceProxy, ShippingPackageServiceProxy, PackageOptionsListDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
// import { CreateOrEditTimeZoneModalComponent } from './create-or-edit-timeZone-modal.component';
// import { ViewTimeZoneModalComponent } from './view-timeZone-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { CreateOrEditPackageOptionModalComponent } from './create-or-edit-package-option-modal.component';
import { ViewPackageOptionModalComponent } from './view-package-option-modal.component';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './package-option.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    providers: [PackageOptionsServiceProxy]
})
export class PackageOptionsComponent extends AppComponentBase {

    @ViewChild('createOrEditPackageOptionsModal', { static: true }) createOrEditPackageOptionsModal: CreateOrEditPackageOptionModalComponent;
    @ViewChild('viewPackageOptionModalComponent', { static: true }) viewPackageOptionModal: ViewPackageOptionModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.ShippingPackages.PackageOption';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    valueFilter = '';
    codeFilter = '';
    keyFilter = '';
    packages: Array<{ id: number, name: string, total: number }> = new Array();




    constructor(
        injector: Injector,
        private _timeZonesServiceProxy: TimeZonesServiceProxy,
        private _packageOptionsServiceProxy: PackageOptionsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _controllerRoutePackages: ShippingPackageServiceProxy,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(){
        $('.kt-select2').select2();
        this.setIsEntityHistoryEnabled();
        this.getPackages();
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: PackageOptionsListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Package Option - " +role.packageName
        });
    }

    getPackageOptions(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._packageOptionsServiceProxy.getAllPackageOptions(
            undefined,
            this.filterText,
            String((<HTMLInputElement>document.getElementById('packageOptionPackageSelectInput')).value) != "null"
            ? String((<HTMLInputElement>document.getElementById('packageOptionPackageSelectInput')).value) 
            : undefined,
            this.codeFilter,
            this.keyFilter,
            this.valueFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getMaxResultCount(this.paginator, event),
            this.primengTableHelper.getSkipCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide()
        });
    }

    getPackages() {
        this.packages = [];
        this._controllerRoutePackages.getShippingPackageListOptions(undefined, undefined, undefined, undefined, undefined, undefined).subscribe(result => {
            result.forEach(element => {
                if (element.name !== 'Default') {
                    this.packages.push(element);
                }
            });
            console.log(this.packages)
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createPackageOption(): void {
        this.createOrEditPackageOptionsModal.show();
    }

    deletePackageOptions(id: number): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._packageOptionsServiceProxy.deletePackageOption(id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._packageOptionsServiceProxy.getPackageOptionsToExcel(
            this.filterText,
            String((<HTMLInputElement>document.getElementById('packageOptionPackageSelectInput')).value) != "null"
            ? String((<HTMLInputElement>document.getElementById('packageOptionPackageSelectInput')).value) 
            : undefined,
            this.codeFilter,
            this.keyFilter,
            this.valueFilter
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
