import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { AppCommonModule } from '@app/shared/common/app-common.module';
import { UtilsModule } from '@shared/utils/utils.module';
import { FileUploadModule } from 'ng2-file-upload';
import { ModalModule, PopoverModule, TabsModule, TooltipModule, BsDropdownModule, TimepickerModule } from 'ngx-bootstrap';
import { TimePickerModule } from '@progress/kendo-angular-dateinputs';
import { BsDatepickerModule, BsDatepickerConfig, BsDaterangepickerConfig, BsLocaleService } from 'ngx-bootstrap/datepicker';
import { AutoCompleteModule } from 'primeng/autocomplete';
import { EditorModule } from 'primeng/editor';
import { FileUploadModule as PrimeNgFileUploadModule } from 'primeng/fileupload';
import { InputMaskModule } from 'primeng/inputmask';
import { PaginatorModule } from 'primeng/paginator';
import { TableModule } from 'primeng/table';
import { TreeModule } from 'primeng/tree';
import { DragDropModule } from 'primeng/dragdrop';
import { TreeDragDropService } from 'primeng/api';
import { ContextMenuModule } from 'primeng/contextmenu';
import { NgxChartsModule } from '@swimlane/ngx-charts';
import { CountoModule } from 'angular2-counto';
import { TextMaskModule } from 'angular2-text-mask';
import { ImageCropperModule } from 'ngx-image-cropper';
import { AngularFontAwesomeModule } from 'angular-font-awesome';
import { NgxBootstrapDatePickerConfigService } from 'assets/ngx-bootstrap/ngx-bootstrap-datepicker-config.service';

import { QRCodeModule } from 'angular2-qrcode';
import { AgmCoreModule, LAZY_MAPS_API_CONFIG } from '@agm/core';
import { NgSelectModule } from '@ng-select/ng-select';
import { ColorPickerModule } from 'primeng/components/colorpicker/colorpicker';
import { AgmOverlays } from "agm-overlays";
import { AgmJsMarkerClustererModule } from '@agm/js-marker-clusterer';


import { CalendarModule } from 'primeng/calendar';
import { AccordionModule } from 'primeng/accordion';
import { ToastModule } from 'primeng/toast';

import { CarouselModule } from 'primeng/carousel';
import { ChipsModule } from 'primeng/chips';
import { MainModule } from '@app/main/main.module';

import { QuillModule } from 'ngx-quill';



import { NgxDropzoneModule } from 'ngx-dropzone';
import { RouteBuilderRoutingModule } from './route-builder.routing.module';
import { RouteBuilderComponent } from './route-builder.component';
import { DirectionsMapDirective } from '../fleet-management/driver-proximities/driver-proximities.directive';
import { SprintShipModule } from '../sprintship.module';
import { MovePendingOrderRouteBuilder } from './move-order-routebuilder.component';
import { UpdateEtaRouteBuilder } from './update-eta..component';
import { RouteBuilderFilter } from './route-builder-filter.component';
import { AppConsts } from '@shared/AppConsts';
import { GoogleMapsConfig } from 'AppPreBootstrap';
import { GenerateRoutesTrackingComponent } from '../controller/routes/generate-routes-tracking.component';
import { MovePendingOrder } from '../controller/route-detail/move-order-modal.component';

@NgModule({
    imports: [

        FormsModule,
        ReactiveFormsModule,
        CommonModule,
        FileUploadModule,
        ModalModule.forRoot(),
        TabsModule.forRoot(),
        TooltipModule.forRoot(),
        PopoverModule.forRoot(),
        BsDropdownModule.forRoot(),
        BsDatepickerModule.forRoot(),
        TimepickerModule.forRoot(),
        QuillModule.forRoot(),
        TimePickerModule,
        QRCodeModule,
        UtilsModule,
        AppCommonModule,
        TableModule,
        TreeModule,
        DragDropModule,
        ContextMenuModule,
        PaginatorModule,
        PrimeNgFileUploadModule,
        AutoCompleteModule,
        EditorModule,
        InputMaskModule,
        NgxChartsModule,
        CountoModule,
        TextMaskModule,
        ImageCropperModule,
        RouteBuilderRoutingModule,
        AngularFontAwesomeModule,
        AgmOverlays,
        AgmCoreModule.forRoot(),
        AgmJsMarkerClustererModule,
        NgSelectModule,
        ColorPickerModule,
        CalendarModule,
        AccordionModule,
        ToastModule,
        CarouselModule,
        ChipsModule,
        NgxDropzoneModule,
        MainModule,
        SprintShipModule
    ],
    declarations: [
        RouteBuilderComponent,
        UpdateEtaRouteBuilder,
        RouteBuilderFilter,
        MovePendingOrderRouteBuilder
    ],
    providers: [
        TreeDragDropService,
        { provide: BsDatepickerConfig, useFactory: NgxBootstrapDatePickerConfigService.getDatepickerConfig },
        { provide: BsDaterangepickerConfig, useFactory: NgxBootstrapDatePickerConfigService.getDaterangepickerConfig },
        { provide: BsLocaleService, useFactory: NgxBootstrapDatePickerConfigService.getDatepickerLocale },
        { provide: LAZY_MAPS_API_CONFIG, useClass: GoogleMapsConfig}
    ]
})
export class RouteBuilderModule {
    
}
