﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { RouteGeneratorLogsServiceProxy, RouteGeneratorLogDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { ViewRouteGeneratorLogModalComponent } from './view-routeGeneratorLog-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './routeGeneratorLogs.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./routeGeneratorLogs.component.less']
})
export class RouteGeneratorLogsComponent extends AppComponentBase {
    @ViewChild('viewRouteGeneratorLogModalComponent', { static: true }) viewRouteGeneratorLogModal: ViewRouteGeneratorLogModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    parametersFilter = '';
    maxStatusFilter : number;
		maxStatusFilterEmpty : number;
		minStatusFilter : number;
		minStatusFilterEmpty : number;
    maxAttemptsFilter : number;
		maxAttemptsFilterEmpty : number;
		minAttemptsFilter : number;
		minAttemptsFilterEmpty : number;
    resultFilter = '';
    emailSentFilter = -1;
    maxLoadDateFilter : moment.Moment;
		minLoadDateFilter : moment.Moment;




    constructor(
        injector: Injector,
        private _routeGeneratorLogsServiceProxy: RouteGeneratorLogsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getRouteGeneratorLogs(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._routeGeneratorLogsServiceProxy.getAll(
            this.filterText,
            this.parametersFilter,
            this.maxStatusFilter == null ? this.maxStatusFilterEmpty: this.maxStatusFilter,
            this.minStatusFilter == null ? this.minStatusFilterEmpty: this.minStatusFilter,
            this.maxAttemptsFilter == null ? this.maxAttemptsFilterEmpty: this.maxAttemptsFilter,
            this.minAttemptsFilter == null ? this.minAttemptsFilterEmpty: this.minAttemptsFilter,
            this.resultFilter,
            this.emailSentFilter,
            this.maxLoadDateFilter === undefined ? this.maxLoadDateFilter : moment(this.maxLoadDateFilter).endOf('day'),
            this.minLoadDateFilter === undefined ? this.minLoadDateFilter : moment(this.minLoadDateFilter).startOf('day'),
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
            console.log(result.items);
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    deleteRouteGeneratorLog(routeGeneratorLog: RouteGeneratorLogDto): void {
        this.message.confirm(
            this.l('RouteGenLogDeleteWarningMessage', routeGeneratorLog.jobId),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeGeneratorLogsServiceProxy.delete(routeGeneratorLog.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }
}
