import { Component, Injector, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { RouteListDto, Route4MeServiceProxy, RouteServiceProxy, CreateOptimizeInputDto, LocationServiceProxy, ControllerRouteDetailServiceProxy, LocationNameDto } from '@shared/service-proxies/service-proxies';

import * as moment from 'moment';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { Paginator } from 'primeng/components/paginator/paginator';
import { Table } from 'primeng/components/table/table';
import { HttpClient } from '@angular/common/http';
import { finalize } from 'rxjs/operators';
import { CreateRouteModalComponent } from './create-routes-modal.component';
import { ViewRouteModalComponent } from '@app/sprintship/routes/view-route-modal.component';
import * as _ from 'lodash';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
@Component({
    templateUrl: 'routes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})

export class RoutesComponent extends AppComponentBase implements OnInit {

    @ViewChild('dataTable', {static: true}) dataTable: Table;
    @ViewChild('paginator', {static: true}) paginator: Paginator;
    @ViewChild('createRouteModal', {static: false}) createRouteModal: CreateRouteModalComponent;
    @ViewChild('viewRouteModalComponent', {static: true}) viewRouteModal: ViewRouteModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.Routes.Route';
    entityHistoryEnabled = false;
    
    advancedFiltersAreShown = false;
    filterText: string;
    templateFilter = '';
    locationFilter = '';
    idFilter: any;
    saving = false;
    //locations: any;
    locationId: number;
    drivers:any;
    driverId: number;
    filteredLocations: LocationNameDto[];
    //date: any;

    routeId: CreateOptimizeInputDto = new CreateOptimizeInputDto();
    // filters: {
    //     id: number;
    //     location: string;
        
    // } = <any>{};
    _isLab: boolean = false;
    constructor(
        injector: Injector,
        private _locationService: LocationServiceProxy,
        private _routeService: RouteServiceProxy,
        private _routeOptimize: Route4MeServiceProxy,
        private _controllerRouteDetails: ControllerRouteDetailServiceProxy,
        
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
        $('.kt-select2').select2();
        this.filterLocation();
        this.getDrivers();
        this._isLab = abp.features.isEnabled('App.RouteDetails');
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: RouteListDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Route - " +role.routeTemplateName
        });
    }

    getRoutes(event?: LazyLoadEvent) {
        //let date = moment.utc(this.loadDateFilter);
        //this.date = moment.utc(this.loadDateFilter);
        // console.log('PRIMENG');
        // console.log(this.primengTableHelper);  
        
        this.locationId = Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) != 0 ? Number((<HTMLInputElement>document.getElementById('locationSelectInput')).value) : undefined;
        this.driverId = Number((<HTMLInputElement>document.getElementById('selectedDriverId')).value) != 0 ? Number((<HTMLInputElement>document.getElementById('selectedDriverId')).value) : undefined;
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }
        this.spinnerService.show();
        this._routeService.getRoute(
            this.idFilter,
            this.filterText,
            this.locationId,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            undefined,
            this.templateFilter,
            undefined,
            undefined,
            this.driverId,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;

            this.primengTableHelper.hideLoadingIndicator();
            this.spinnerService.hide();
            
            // console.log(result.items);          

            //console.log(this.incidentId.id);
            
        });       
    }


    
    createRoute(): void {
        this.createRouteModal.show();
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

     

    deleteRoute(route: RouteListDto): void {
            
        
        this.message.confirm(
            this.l('RouteWarningMessage', route.id),
            this.l('AreYouSure'),
            (isConfirmed) => {
                if (isConfirmed) {
                    this._routeService.deleteRoute(route.id).subscribe(() => {
                        this.reloadPage();
                        this.notify.info(this.l('SuccessfullyDeleted'));
                        
                    });
                }
            }
        );
    }

    routeOptimize(id): void {
        this.saving = true;
        this.routeId.routeId = id;
        this.spinnerService.show();
        
        this._routeOptimize.optimize(this.routeId)
                    .pipe(finalize(() => this.saving = false))
                    .subscribe(() => { 
                        this.spinnerService.hide();
                        this.notify.info(this.l('Route4MeOptimize'));
                        
        });         
     
    }

    // getLocation(){
    //     // this._locationService.getLocation(
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined,
    //     //     undefined
    //     // ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
    //     //     this.locations = result.items
            
    //     // });
    //     this._locationService.getLocationNames(
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined,
    //         undefined
    //     ).pipe(finalize(() => this.spinnerService.hide())).subscribe(result => {
    //         this.locations = result
            
    //     });
    // }
    filterLocation(): void {
        this._locationService.getLocationNames(undefined, undefined, undefined, undefined, undefined, undefined)
            .subscribe(result => {
                this.filteredLocations = result;
            });
    }

    getDrivers(){
        this._controllerRouteDetails.getDriverList().subscribe(result => {
            this.drivers = result;
            // console.log(this.drivers)
        });
    }
    
 

    
   

}
