import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { VersionsServiceProxy, CreateOrEditVersionDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';


@Component({
    selector: 'createOrEditVersionModal',
    templateUrl: './create-or-edit-version-modal.component.html'
})
export class CreateOrEditVersionModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: false }) modal: ModalDirective;


    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    version: CreateOrEditVersionDto = new CreateOrEditVersionDto();

            releasedDate: Date;


    constructor(
        injector: Injector,
        private _versionsServiceProxy: VersionsServiceProxy
    ) {
        super(injector);
    }

    show(versionId?: number): void {
this.releasedDate = null;

        if (!versionId) {
            this.version = new CreateOrEditVersionDto();
            this.version.id = versionId;

            this.active = true;
            this.modal.show();
        } else {
            this._versionsServiceProxy.getVersionForEdit(versionId).subscribe(result => {
                this.version = result.version;

                if (this.version.releasedDate) {
					this.releasedDate = this.version.releasedDate.toDate();
                }

                this.active = true;
                this.modal.show();
            });
        }
    }

    save(): void {
            this.saving = true;

			
        if (this.releasedDate) {
            if (!this.version.releasedDate) {
                this.version.releasedDate = moment(this.releasedDate).startOf('day');
            }
            else {
                this.version.releasedDate = moment(this.releasedDate);
            }
        }
        else {
            this.version.releasedDate = null;
        }
            this._versionsServiceProxy.createOrEdit(this.version)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {

        this.active = false;
        this.modal.hide();
    }
}
