import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { VersionsServiceProxy, VersionDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditVersionModalComponent } from './create-or-edit-version-modal.component';
import { ViewVersionModalComponent } from './view-version-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './versions.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class VersionsComponent extends AppComponentBase {

    @ViewChild('createOrEditVersionModal', { static: true }) createOrEditVersionModal: CreateOrEditVersionModalComponent;
    @ViewChild('viewVersionModalComponent' , { static: false }) viewVersionModal: ViewVersionModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    versionNumberFilter = '';
    maxReleasedDateFilter : moment.Moment;
		minReleasedDateFilter : moment.Moment;




    constructor(
        injector: Injector,
        private _versionsServiceProxy: VersionsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    getVersions(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._versionsServiceProxy.getAll(
            this.filterText,
            this.versionNumberFilter,
            this.maxReleasedDateFilter,
            this.minReleasedDateFilter,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createVersion(): void {
        this.createOrEditVersionModal.show();
    }

    deleteVersion(version: VersionDto): void {
        this.message.confirm(
            '','',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._versionsServiceProxy.delete(version.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._versionsServiceProxy.getVersionsToExcel(
        this.filterText,
            this.versionNumberFilter,
            this.maxReleasedDateFilter,
            this.minReleasedDateFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
