﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { SmsTemplatesServiceProxy, CreateOrEditSmsTemplateDto ,SmsTemplateSmsAccountLookupTableDto
					} from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditSmsTemplateModal',
    templateUrl: './create-or-edit-smsTemplate-modal.component.html'
})
export class CreateOrEditSmsTemplateModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    smsTemplate: CreateOrEditSmsTemplateDto = new CreateOrEditSmsTemplateDto();

    smsAccountAccount = '';

	allSmsAccounts: SmsTemplateSmsAccountLookupTableDto[];
					
    constructor(
        injector: Injector,
        private _smsTemplatesServiceProxy: SmsTemplatesServiceProxy
    ) {
        super(injector);
    }

    onShown():void{
        $('.kt-select2').select2();
    }
    show(smsTemplateId?: number): void {

        if (!smsTemplateId) {
            this.smsTemplate = new CreateOrEditSmsTemplateDto();
            this.smsTemplate.id = smsTemplateId;
            this.smsAccountAccount = '';

            this.active = true;
            this.modal.show();
        } else {
            this._smsTemplatesServiceProxy.getSmsTemplateForEdit(smsTemplateId).subscribe(result => {
                this.smsTemplate = result.smsTemplate;

                this.smsAccountAccount = result.smsAccountAccount;

                this.active = true;
                this.modal.show();
            });
        }
        this._smsTemplatesServiceProxy.getAllSmsAccountForTableDropdown().subscribe(result => {						
						this.allSmsAccounts = result;
					});
					
    }

    save(): void {
            this.saving = true;
            this.smsTemplate.smsAccountId =  Number((<HTMLInputElement>document.getElementById('smsAccountId')).value);
			
            this._smsTemplatesServiceProxy.createOrEdit(this.smsTemplate)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
