﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { SmsTemplatesServiceProxy, SmsTemplateDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditSmsTemplateModalComponent } from './create-or-edit-smsTemplate-modal.component';
import { ViewSmsTemplateModalComponent } from './view-smsTemplate-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './smsTemplates.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class SmsTemplatesComponent extends AppComponentBase {

    @ViewChild('createOrEditSmsTemplateModal', { static: true }) createOrEditSmsTemplateModal: CreateOrEditSmsTemplateModalComponent;
    @ViewChild('viewSmsTemplateModalComponent', { static: true }) viewSmsTemplateModal: ViewSmsTemplateModalComponent;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;

    advancedFiltersAreShown = false;
    filterText = '';
    descriptionFilter = '';
    templateFilter = '';
    isActiveFilter = -1;
    codeFilter = '';
        smsAccountAccountFilter = '';


    _entityTypeFullName = 'SprintTek.Shipping.Sms.SmsTemplate';
    entityHistoryEnabled = false;

    constructor(
        injector: Injector,
        private _smsTemplatesServiceProxy: SmsTemplatesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private router: Router,

    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
    }

    private setIsEntityHistoryEnabled(): boolean {
        let customSettings = (abp as any).custom;
        return this.isGrantedAny('Pages.Administration.AuditLogs') && customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    getSmsTemplates(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._smsTemplatesServiceProxy.getAll(
            this.filterText,
            this.descriptionFilter,
            this.templateFilter,
            this.isActiveFilter,
            this.codeFilter,
            this.smsAccountAccountFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createSmsTemplate(): void {
        this.createOrEditSmsTemplateModal.show();
    }

    showHistory(smsTemplate: SmsTemplateDto): void {
        this.entityTypeHistoryModal.show({
            entityId: smsTemplate.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "SMS Template - " +smsTemplate.description
        });
    }

    deleteSmsTemplate(smsTemplate: SmsTemplateDto): void {
        this.message.confirm(
            "","",
            (isConfirmed) => {
                if (isConfirmed) {
                    this._smsTemplatesServiceProxy.delete(smsTemplate.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._smsTemplatesServiceProxy.getSmsTemplatesToExcel(
        this.filterText,
            this.descriptionFilter,
            this.templateFilter,
            this.isActiveFilter,
            this.codeFilter,
            this.smsAccountAccountFilter,
        )
        .subscribe(result => {
            if(result!=undefined){
                if(result.fileUrl!=undefined){
                    this.router.navigate([]).then(result1 => {
                        window.open(result.fileUrl, '_blank');
                    });
                    this.spinnerService.hide();
                }else{
                    this.spinnerService.hide();
                }
            }else{
                this.message.warn('', "No data to export");
                this.spinnerService.hide();
            }
         });
    }
}
