import { Component, Injector, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { AppConsts } from '@shared/AppConsts';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TabsetComponent } from "ngx-bootstrap";
import { SupportTicketSettingServiceProxy, RouteServiceProxy, SupportTicketSettingList } from '@shared/service-proxies/service-proxies';

import * as _ from 'lodash';
import { catchError } from 'rxjs/operators';
import { throwError } from 'rxjs';
import {DomSanitizer,SafeResourceUrl,} from '@angular/platform-browser';

@Component({
    templateUrl: './support-FAQUrl.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./support-documentation.component.less']
})

export class SupportFAQComponent extends AppComponentBase implements OnInit {
    @ViewChild('documentsTab', { static: false }) documentsTabs: TabsetComponent;

    getSettings: SupportTicketSettingList;
    faqURL: string;

    constructor(
        injector: Injector,
        private _routeService: RouteServiceProxy,
        private _settingServiceProxy: SupportTicketSettingServiceProxy,
        private sanitizer:DomSanitizer        
    ) {
        super(injector);
    }
    ngOnInit(): void {
        this.getSupportSettings();
    }


    getSupportSettings() {
        this.spinnerService.show(); 
        this._settingServiceProxy.getSettingsForDisplay()
        .pipe(
            catchError((err) => {
                this.spinnerService.hide(); 
                return throwError(err);
            })                    
        )
        .subscribe((result) => {
            if(result!=null){
                this.faqURL = result[0].faqUrl;
            $('#iFrame').attr('src', this.faqURL);
            const iframe = document.getElementById('iFrame');
            const handleLoad = () => this.spinnerService.hide();

            iframe.addEventListener('load', handleLoad, true)
            }
        });
    }
}
