﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { SupportTicketNotificationsServiceProxy, SupportTicketNotificationDto, GetSupportTicketNotificationForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditSupportTicketNotificationModalComponent } from './create-or-edit-supportTicketNotification-modal.component';
import { ViewSupportTicketNotificationModalComponent } from './view-supportTicketNotification-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';

@Component({
    templateUrl: './supportTicketNotifications.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()],
    styleUrls: ['./support-ticket-notification.less']
})
export class SupportTicketNotificationsComponent extends AppComponentBase {

    @ViewChild('createOrEditSupportTicketNotificationModal', { static: true }) createOrEditSupportTicketNotificationModal: CreateOrEditSupportTicketNotificationModalComponent;
    @ViewChild('viewSupportTicketNotificationModalComponent', { static: true }) viewSupportTicketNotificationModal: ViewSupportTicketNotificationModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.SupportTickets.SupportTicketNotification';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    descriptionFilter = '';
    codeFilter = '';
    isActiveFilter = -1;
        orderNotificationTemplateDescriptionFilter = '';
        supportTicketStatusDescriptionFilter = '';
        smsTemplateDescriptionFilter = '';




    constructor(
        injector: Injector,
        private _supportTicketNotificationsServiceProxy: SupportTicketNotificationsServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
    }

  
    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetSupportTicketNotificationForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.supportTicketNotification.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Ticket Notification - " +role.orderNotificationTemplateDescription
        });
    }

    getSupportTicketNotifications(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._supportTicketNotificationsServiceProxy.getAll(
            this.filterText,
            this.descriptionFilter,
            this.codeFilter,
            this.isActiveFilter,
            this.orderNotificationTemplateDescriptionFilter,
            this.supportTicketStatusDescriptionFilter,
            this.smsTemplateDescriptionFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            this.spinnerService.hide();
        });
    }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createSupportTicketNotification(): void {
        this.createOrEditSupportTicketNotificationModal.show();
    }

    deleteSupportTicketNotification(supportTicketNotification: SupportTicketNotificationDto): void {
        this.message.confirm(
            "","",
            (isConfirmed) => {
                if (isConfirmed) {
                    this._supportTicketNotificationsServiceProxy.delete(supportTicketNotification.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._supportTicketNotificationsServiceProxy.getSupportTicketNotificationsToExcel(
        this.filterText,
            this.descriptionFilter,
            this.codeFilter,
            this.isActiveFilter,
            this.orderNotificationTemplateDescriptionFilter,
            this.supportTicketStatusDescriptionFilter,
            this.smsTemplateDescriptionFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
