import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { SupportTicketStatusServiceProxy, TicketStatusList, TicketStatusCodeDto } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditTicketStatusModalComponent } from './create-or-edit-status.component';
//import { ViewRouteTypeModalComponent } from './view-routeType-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import * as _ from 'lodash';
import * as moment from 'moment';

@Component({
    templateUrl: './support-ticket-status.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class TicketStatusComponent extends AppComponentBase {

    @ViewChild('createOrEditTicketStatusModal', { static: true }) createOrEditTicketStatusModal: CreateOrEditTicketStatusModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.SupportTickets.SupportTicketStatus';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    update: Array<any> = [];
    parentStatusType : TicketStatusCodeDto[];
  

    constructor(
        injector: Injector,
        private _statusServiceProxy: SupportTicketStatusServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService
    ) {
        super(injector);
    }

    ngOnInit(): void {
        //this.entityHistoryEnabled = this.setIsEntityHistoryEnabled();
        this.setIsEntityHistoryEnabled();
        this.getParentStatusType();        
    }

    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: TicketStatusList): void {
        this.entityTypeHistoryModal.show({
            entityId: role.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Ticket Status - " +role.description
        });
    }

    getStatus(event?: LazyLoadEvent) {
        //if (this.primengTableHelper.shouldResetPaging(event)) {
            //this.paginator.changePage(0);
            //return;
        //}

        //this.primengTableHelper.showLoadingIndicator();
        this.spinnerService.show();
        this._statusServiceProxy.getTicketStatus(
            this.filterText,
            undefined,
            undefined,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            for(var data in this.primengTableHelper.records){
                for(var data2 in this.parentStatusType){
                    if(this.primengTableHelper.records[data].parentStatusTypeId==this.parentStatusType[data2].id){
                        if(this.parentStatusType[data2].code=="TODO"){
                            this.parentStatusType[data2].code = "To Do"
                        }
                        this.primengTableHelper.records[data].parentStatusType = this.parentStatusType[data2].code;
                    }
                }
            }
            //this.primengTableHelper.hideLoadingIndicator();
            this.spinnerService.hide();

        });
    }

    reloadPage(): void {
        //this.paginator.changePage(this.paginator.getPage());
         this.getStatus();
    }

    createStatus(): void {
        this.createOrEditTicketStatusModal.show();
    }

    

    delete(id): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._statusServiceProxy.delete(id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    getParentStatusType(){
        this._statusServiceProxy.getParentTicketStatusType().subscribe(result =>{
            this.parentStatusType = result;
        });
    }
    
}
