﻿import { Component, ViewChild, Injector, Output, EventEmitter} from '@angular/core';
import { ModalDirective } from 'ngx-bootstrap';
import { finalize } from 'rxjs/operators';
import { SupportCategoryTypesServiceProxy, CreateOrEditSupportCategoryTypesDto } from '@shared/service-proxies/service-proxies';
import { AppComponentBase } from '@shared/common/app-component-base';
import * as moment from 'moment';

@Component({
    selector: 'createOrEditSupportCategoryTypesModal',
    templateUrl: './create-or-edit-supportCategoryTypes-modal.component.html'
})
export class CreateOrEditSupportCategoryTypesModalComponent extends AppComponentBase {

    @ViewChild('createOrEditModal', { static: true }) modal: ModalDirective;

    @Output() modalSave: EventEmitter<any> = new EventEmitter<any>();

    active = false;
    saving = false;

    supportCategoryTypes: CreateOrEditSupportCategoryTypesDto = new CreateOrEditSupportCategoryTypesDto();



    constructor(
        injector: Injector,
        private _supportCategoryTypesServiceProxy: SupportCategoryTypesServiceProxy
    ) {
        super(injector);
    }

    show(supportCategoryTypesId?: number): void {

        if (!supportCategoryTypesId) {
            this.supportCategoryTypes = new CreateOrEditSupportCategoryTypesDto();
            this.supportCategoryTypes.id = supportCategoryTypesId;
            this.supportCategoryTypes.color = "#000000";
            this.active = true;
            this.modal.show();
        } else {
            this._supportCategoryTypesServiceProxy.getSupportCategoryTypesForEdit(supportCategoryTypesId).subscribe(result => {
                this.supportCategoryTypes = result.supportCategoryTypes;
                this.supportCategoryTypes.color = result.supportCategoryTypes.color
                if(result.supportCategoryTypes.inactive == false){
                    this.supportCategoryTypes.inactive = true
                }else{
                    this.supportCategoryTypes.inactive = false
                }

                this.active = true;
                this.modal.show();
            });
        }
        
    }

    save(): void {
            this.saving = true;
            if(this.supportCategoryTypes.inactive == false){
                this.supportCategoryTypes.inactive = true
            }
            else{
                this.supportCategoryTypes.inactive = false
            }
			
            this._supportCategoryTypesServiceProxy.createOrEdit(this.supportCategoryTypes)
             .pipe(finalize(() => { this.saving = false;}))
             .subscribe(() => {
                this.notify.info(this.l('SavedSuccessfully'));
                this.close();
                this.modalSave.emit(null);
             });
    }







    close(): void {
        this.active = false;
        this.modal.hide();
    }
}
