﻿import { Component, Injector, ViewEncapsulation, ViewChild } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { SupportCategoryTypesServiceProxy, SupportCategoryTypesDto, GetSupportCategoryTypesForViewDto  } from '@shared/service-proxies/service-proxies';
import { NotifyService } from '@abp/notify/notify.service';
import { AppComponentBase } from '@shared/common/app-component-base';
import { TokenAuthServiceProxy } from '@shared/service-proxies/service-proxies';
import { CreateOrEditSupportCategoryTypesModalComponent } from './create-or-edit-supportCategoryTypes-modal.component';
import { ViewSupportCategoryTypesModalComponent } from './view-supportCategoryTypes-modal.component';
import { appModuleAnimation } from '@shared/animations/routerTransition';
import { Table } from 'primeng/components/table/table';
import { Paginator } from 'primeng/components/paginator/paginator';
import { LazyLoadEvent } from 'primeng/components/common/lazyloadevent';
import { FileDownloadService } from '@shared/utils/file-download.service';
import * as _ from 'lodash';
import * as moment from 'moment';
import { EntityTypeHistoryModalComponent } from '@app/shared/common/entityHistory/entity-type-history-modal.component';
import { AppSessionService } from '@shared/common/session/app-session.service';

@Component({
    templateUrl: './supportCategoryTypes.component.html',
    encapsulation: ViewEncapsulation.None,
    animations: [appModuleAnimation()]
})
export class SupportCategoryTypesComponent extends AppComponentBase {

    @ViewChild('createOrEditSupportCategoryTypesModal', { static: true }) createOrEditSupportCategoryTypesModal: CreateOrEditSupportCategoryTypesModalComponent;
    @ViewChild('viewSupportCategoryTypesModalComponent', { static: true }) viewSupportCategoryTypesModal: ViewSupportCategoryTypesModalComponent;
    @ViewChild('dataTable', { static: true }) dataTable: Table;
    @ViewChild('paginator', { static: true }) paginator: Paginator;
    @ViewChild('entityTypeHistoryModal', { static: true }) entityTypeHistoryModal: EntityTypeHistoryModalComponent;
    _entityTypeFullName = 'SprintTek.Shipping.SupportTickets.SupportCategoryTypes';
    entityHistoryEnabled = false;

    advancedFiltersAreShown = false;
    filterText = '';
    codeFilter = '';
    descriptionFilter = '';
    colorFilter = '';
    iconFilter = '';
    inactiveFilter = -1;
    hideHeader: boolean;
    resultGroup: any
    tenantIds: Array<any> = [];
    tenancy:boolean = false


    constructor(
        injector: Injector,
        private _supportCategoryTypesServiceProxy: SupportCategoryTypesServiceProxy,
        private _notifyService: NotifyService,
        private _tokenAuth: TokenAuthServiceProxy,
        private _activatedRoute: ActivatedRoute,
        private _fileDownloadService: FileDownloadService,
        private _appSessionService: AppSessionService
    ) {
        super(injector);
    }

    ngOnInit(): void {
        this.setIsEntityHistoryEnabled();
        this.resultGroup = [];
        this.hideHeader = true;
        if(this._appSessionService.tenant != undefined) {
            this.tenancy = true;
          }
    }

  
    private setIsEntityHistoryEnabled(): void {
        let customSettings = (abp as any).custom;
        this.entityHistoryEnabled = customSettings.EntityHistory && customSettings.EntityHistory.isEnabled && _.filter(customSettings.EntityHistory.enabledEntities, entityType => entityType === this._entityTypeFullName).length === 1;
    }

    showHistory(role: GetSupportCategoryTypesForViewDto): void {
        this.entityTypeHistoryModal.show({
            entityId: role.supportCategoryTypes.id.toString(),
            entityTypeFullName: this._entityTypeFullName,
            entityTypeDescription: "Category Type - " +role.supportCategoryTypes.description
        });
    }

    getSupportCategoryTypes(event?: LazyLoadEvent) {
        if (this.primengTableHelper.shouldResetPaging(event)) {
            this.paginator.changePage(0);
            return;
        }

        this.spinnerService.show();

        this._supportCategoryTypesServiceProxy.getAll(
            this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.colorFilter,
            this.iconFilter,
            this.inactiveFilter,
            this.primengTableHelper.getSorting(this.dataTable),
            this.primengTableHelper.getSkipCount(this.paginator, event),
            this.primengTableHelper.getMaxResultCount(this.paginator, event)
        ).subscribe(result => {
            this.resultGroup = [];
            let key, object, index;
            let arrTmp = [];
            if( result.items.length > 0)
            {
              if(result.items[0].mainTenant !== null)
              {
                 this.hideHeader = false;
              }
            }
            for(key in result.items)
            {

              let obj = this.resultGroup.find(o=> o.tenant === result.items[key].tenant);
              let index = this.resultGroup.findIndex(o=> o.tenant === result.items[key].tenant);
             
              if(obj === undefined)
              {
                
                arrTmp = [];
                arrTmp.push( result.items[key]);
                if(result.items[key].tenant == null)
                {
                  index = this.resultGroup.findIndex(o=> o.tenant === 'Host');
                  if(index < 0)
                  {
                    this.resultGroup.push({'tenant': 'Host', 'tenantId':  0, 'tenantDetails': []});
                    this.tenantIds.push('');
                    index = this.resultGroup.findIndex(o=> o.tenant === 'Host');
                  }
                  else
                  {
                     index = this.resultGroup.findIndex(o=> o.tenant === 'Host');
                  }
                }
                else
                {
                  this.resultGroup.push({'tenant': result.items[key].tenant, 'tenantId':  result.items[key].tenantId, 'tenantDetails': []});
                  this.tenantIds.push(result.items[key].tenantId);
                  index = this.resultGroup.findIndex(o=> o.tenant === result.items[key].tenant);
                }
                  
                  
                  this.resultGroup[index].tenantDetails.push(result.items[key]);
              }
              else
              {
                this.resultGroup[index].tenantDetails.push(result.items[key]);
                  
              }
            }
            if(localStorage.getItem('collapseTenants') == null)
            {
              localStorage.setItem('collapseTenants', this.tenantIds.toString());
            }

            this.primengTableHelper.totalRecordsCount = result.totalCount;
            this.primengTableHelper.records = result.items;
            // console.log(result.items)
            // console.log(this.resultGroup)
            this.primengTableHelper.records = this.resultGroup;
            this.spinnerService.hide();
        });
    }

    expandDetails(obj: string, tenantId: any) {
    //     console.log('TENANT IDS: ');
    //    console.log(this.tenantIds);
         if($('.tr-' + obj + '-cat').css('display') == 'none')
         {
             // $('.span-' +  obj).html(obj + ' Routes');
             $('.tr-' + obj + '-cat').css('display', 'table-row');
             $('.td-' + obj + '-cat').html('<i class="fa fa-chevron-down"></i>');
             if(tenantId == 0)
             {
               tenantId = "";
             }
             this.tenantIds.push(tenantId);
         }
         else
         {
             // $('.span-' +  obj).html(obj + ' Routes');
             $('.tr-' + obj + '-cat').css('display', 'none');
             $('.td-' + obj  + '-cat').html('<i class="fa fa-chevron-right"></i>');
             let index = this.tenantIds.findIndex(o=> o == tenantId);
             this.tenantIds.splice(index,1);
         }
        //  this.getKpiFilter(this.tenantIds);
        //  console.log(this.tenantIds)
         localStorage.setItem('collapseTenants', this.tenantIds.toString());
         
     }

    reloadPage(): void {
        this.paginator.changePage(this.paginator.getPage());
    }

    createSupportCategoryTypes(): void {
        this.createOrEditSupportCategoryTypesModal.show();
    }

    deleteSupportCategoryTypes(supportCategoryTypes: SupportCategoryTypesDto): void {
        this.message.confirm(
            '',
            '',
            (isConfirmed) => {
                if (isConfirmed) {
                    this._supportCategoryTypesServiceProxy.delete(supportCategoryTypes.id)
                        .subscribe(() => {
                            this.reloadPage();
                            this.notify.success(this.l('SuccessfullyDeleted'));
                        });
                }
            }
        );
    }

    exportToExcel(): void {
        this._supportCategoryTypesServiceProxy.getSupportCategoryTypesToExcel(
        this.filterText,
            this.codeFilter,
            this.descriptionFilter,
            this.colorFilter,
            this.iconFilter,
            this.inactiveFilter,
        )
        .subscribe(result => {
            this._fileDownloadService.downloadTempFile(result);
         });
    }
}
